#!/bin/sh
#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}

# File: archive/scripts/setbspriority.ksh
# SID: 1.2
# Date Modified: 04/10/20 13:52:12
#
# Module Description:
#   Updates the Business System priorty for an instance or set of instances.

#
# Usage
#
function Usage
{
  
  ASIShellMsg ASIMiscScripts 65 "sh SetBSPriority"
  print ""
  ASIShellMsg ASIMiscScripts 75
  print "                OR "
  print "        ["
  ASIShellMsg ASIMiscScripts 71
  ASIShellMsg ASIMiscScripts 205
  print "        ]"
  ASIShellMsg ASIMiscScripts 67
  ASIShellMsg ASIMiscScripts 207
  ASIShellMsg ASIMiscScripts 70
  ASIShellMsg ASIMiscScripts 73
  ASIShellMsg ASIMiscScripts 206
  print "  XML tags:"
  print ""
  print "  <methodgroup>"
  print "    <instances>"
  print "      <inst>"   
  print "        <class></class> or <prodver></prodver>"
  print "        <oid></oid> or <instid></instid> <uinstid></uinstid>"
  print "        <objpath></objpath> <nflag></nflag>"
  print "      </inst>"
  print "    </instances>"
  print "    <methods>"
  print "      <SetBSPriority>"
  print "        <NewPriority>value</NewPriority>"
  print "      </SetBSPriority>"
  print "    </methods>"
  print "  </methodgroup>"
  print ""
  print "  <inst> and <methodgroup> groups can occur multiple times"
  print ""
  print " Example:"
  print ""
  print "    sh SetBSPriority -Sdbserver -Usa -Psa_dbserver -cLOB -r54 -t6"
  print ""
  print "    sh SetBSPriority -Sdbserver -Usa -Psa_dbserver -o\"LIKE '%fredos1'\" -m2 -tM"
}


#
# CheckRequiredParams
#
function CheckRequiredParams
{

  # Check the database params
  if [ -z "$server_opt" ]; then
    ASIShellMsg ASIMiscScripts 69 "-S<DB Server>"
    Usage >&2
    exit 2
  elif [ -z "$e_opt" ]; then
      if [ -z "$user_opt" ]; then
        sql_opt="-S$server_opt -E"
      elif [ -z "$password_opt" ]; then
          ASIShellMsg ASIMiscScripts 69 "-P<DB Password>"
          Usage >&2
          exit 2
        else
          sql_opt="-S$server_opt -U$user_opt -P$password_opt"
      fi
    else      
      sql_opt="-S$server_opt -E"
  fi

  # Check the resource specification
  if [ -z "$sFile" ]; then
    if [ -z "$sclass" ]; then
      if [ -z "$sprod" ]; then
        if [ -z "$sobjpath" ]; then
          ASIShellMsg ASIMiscScripts 69 "-c<resource type> or -p<product> or -o<resource path> or -f<input_file>"
          Usage >&2
          exit 2
        fi  
      fi  
    fi
  fi
  
  if [ ! -z "$sclass" -o ! -z "$sprod" ]; then
    if [ -z "$soid" -a -z "$sinstid" ]; then
      ASIShellMsg ASIMiscScripts 69 "-r<resource id> or -i<instid>"
      Usage >&2
      exit 2
    fi  
  fi
  
  #Check additional parms
  if [ -z "$sFile" ]; then
    if [ -z "$spriority" ]; then
      ASIShellMsg ASIMiscScripts 69 "-t<priority>"
      Usage >&2
      exit 2
    fi	
  fi
}

#
# ParseFile
#   Parses the input XML file for the resource identification (cid, id, product/version, objpath)
#   and priority setting.  Processing stops if information is missing from the XML file, and an error message
#   is generated. Invalid XML tags are ignored.  Otherwise a bcp file is created for the 
#   asisp_loadSetBSPriority proc.
#
function ParseFile
{

# Format file with one XML tag per line
  tempout="setbspriority.txt"
  rm -f $tempout

  awk -v testout=$tempout '
  {
#  gsub(">[ ]*<",">\n<", $0)
    gsub(">[^0-9A-Za-x]*<",">\n<", $0)
    printf ("%s\n", $0) >> testout
  
  } 
  ' < $sFile

  awk -F'[<>]' -v tempinstbcp="$tempinstbcp" -v tempparmbcp="$tempparmbcp" '
    BEGIN {
    rowct = 0
    rCorr = 1

    valid["methodgroup"] = 1
    valid["instances"] = 1
    valid["inst"] = 1
    valid["class"] = 1
    valid["oid"] = 1
    valid["prodver"] = 1
    valid["objpath"] = 1
    valid["instid"] = 1
    valid["uinstid"] = 1
    valid["nflag"] = 1
    valid["/inst"] = 1
    valid["/instances"] = 1
    valid["SetBSPriority"] = 1
    valid["NewPriority"] = 1
    valid["/NewPriority"] = 1
    valid["/SetBSPriority"] = 1
    valid["/inst"] = 1
    valid["/instances"] = 1
    valid["/methodgroup"] = 1

    default()
    }

    function default() {
      tag["nflag"] = 1
#      tag["oid"] = 0
      tag["value"] = 0
    }
    
# Ignore comments
    {
     if (cmtflag == 1) {
       if ($0 ~ /-->/) {
         cmtflag = 0
       }  
       continue
     }   
    }

# Build instances

    $2 == "/inst" {
  
      if (tag["class"] == "" && tag["prodver"] == "" && tag["objpath"] == "") {
        system("ASIShellMsg ASIMiscScripts 77 \"resource identifier\" " rowct)
        system("rm -f " tempinstbcp)
        exit
      }

      if ((tag["class"] != "" || tag["prodver"] != "")) {
        if (tag["oid"] == "" && tag["instid"] == "") {
           system("ASIShellMsg ASIMiscScripts 77 \"resource identifier\" " rowct)
           system("rm -f " tempinstbcp)
           exit
        }
      } 
 
      printf("|%s|%s|%s|%s|%s|%s|%s|%s|\n",
          tag["oid"],
          rCorr,
          tag["class"],
          tag["prodver"],
          tag["instid"],
          tag["uinstid"],
          tag["objpath"],
          tag["nflag"]) >> tempinstbcp
      
      delete tag
      default()
    }

# Build #NewPriority Parms

    $2 == "/SetBSPriority" {
      
      if (tag["NewPriority"] == "") {
        system("ASIShellMsg ASIMiscScripts 77 \"NewPriority\" " rowct)
        system("rm -f " tempparmbcp)
        exit
      }
  
      printf("%s|%s|\n",
          rCorr,
          tag["NewPriority"]) >> tempparmbcp
      
      delete tag
      default()
      
    }

# Increment next correlator between #Instances and #NewPriority Parms

    $2 == "/methodgroup" {
      rCorr = rCorr + 1
    }
  
# Parse XML file instances and priority parms
    
    {

      rowct = rowct + 1
      if (NF == 5)
        if (valid[$2]) {
          tag[$2] = $3
        }  
        else {
          system("ASIShellMsg ASIMiscScripts 78 " rowct " " $2)
        }
    }
    
# Parse comments
    
    $0 ~ /<!--/ && /-->/ { continue; }
    $0 ~ /<!--/ { cmtflag = 1; continue; }
    
  ' < $tempout

  rm -f $tempout
}


#
# CallLoadParms
#   Loads temp table for asisp_loadSetBSPriority which calls
#   _SetBSPriority to update the resource priority for a set of cid/id.
#
function CallLoadParms
{

  errout=$(pwd)"/errtemp.txt"
  rm -f "$errout"
#  output=$(tbsmsql -b -n -w1000 $sql_opt <<- ENDSQL
  
  tbsmsql -b -n -w1000 $sql_opt -o"$errout" <<- ENDSQL

    SET NOCOUNT ON
    DECLARE @err INT
    CREATE TABLE #Instances
    (
      cid          ClassID        NULL,
      id           ObjID          NULL,
      rCorr        INT            NULL,
      className    NVARCHAR(24)   NULL,
      prodVer      NVARCHAR(85)   NULL,
      instid       NVARCHAR(425)  NULL,
      uinstid      NVARCHAR(425)  NULL,
      objPath      NVARCHAR(1375) NULL,
      maxRes       INT            NULL,
      errorId      INT            NULL
    )
    CREATE TABLE #BSPriorityParms
    (
      rCorr        INT            NULL,
      newValue     NVARCHAR(32)   NULL,
      error        INT            NULL
    )

    BULK INSERT #Instances FROM '$tempinstbcp'
      WITH ( FIELDTERMINATOR = '|' )
    SET @err = @@ERROR
    IF @err <> 0
      RETURN
    BULK INSERT #BSPriorityParms FROM '$tempparmbcp'
      WITH ( FIELDTERMINATOR = '|' )
    SET @err = @@ERROR
    IF @err <> 0
      RETURN
    
    EXEC asisp_loadSetBSPriority @outformat=2

ENDSQL

# Output error information with resolved cid/id and the original resource identification
  
  if [ $? != 0 -a ! -s "$errout" ]; then
    cat "$errout"
    ASIShellMsg ASIMiscScripts 160
    return
  fi
         
  if [ -s "$errout" ]; then
    
    awk -F'|' '
    
      $0 ~ /o-c-no errors/ {print "Set request processed with no errors"; exit }
  
      $1 !~ /o-c-/ { continue }
      $2 ~ /./ { sub(" ", "", $2); tag[2]="    <cid>"$2"</cid>\n" }
      $3 ~ /./ { tag[3]="    <oid>"$3"</oid>\n" }
      $4 ~ /./ { tag[4]="    <class>"$4"</class>\n" }
      $5 ~ /./ { tag[5]="    <prodver>"$5"</prodver>\n" }
      $6 ~ /./ { tag[6]="    <instid>"$6"</instid>\n" }
      $7 ~ /./ { tag[7]="    <uinstid>"$7"</uinstid>\n" }
      $8 ~ /./ { tag[8]="    <objpath>"$8"</objpath>\n" }
      $9 ~ /./ { tag[9]="  <errorid>"$9"</errorid>\n" }
      $10 ~ /./ { tag[10]="  <errordesc>"$10"</errordesc>\n" }
      $11 ~ /./ { tag[11]="    <NewPriority>"$11"</NewPriority>\n" }
    
      {
        print "<methodgroup>"
        print "  <inst>"
        print tag[2] tag[3] tag[4] tag[5] tag[6] tag[7] tag[8] "  </inst>" 
        print "  <SetBSPriority>"
        print tag[11] "  </SetBSPriority>"
        print tag[9] tag[10] "</methodgroup>"
       
        delete tag
      }
  
    ' < "$errout" 
  fi
  rm -f "$errout" 
  
}

#
# main (entry point)
#

# initialize variables
sprod=""
sver=""
sclass=""
soid=""
sobjpath=""
sinstid=""
suinstid=""
sFile=""
snflag=1
spriority=""
svflag=0
server_opt=$tbsm_server
user_opt=$tbsm_userid
password_opt=$tbsm_password
e_opt=""

# Get the database connection info from the registry
eval $(regdmp 'HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB' | awk '/Server/ {printf("export reg_server=\"%s\"\n",$3);} /User/ {printf("export reg_userid=\"%s\"\n",$3);} /Password/ {printf("export reg_password=\"%s\"\n",$3)}')
# Check if any of the predefined environment variables have been set
if [ -z "$tbsm_server" ]; then
    server_opt=$reg_server
fi
if [ -z "$tbsm_userid" ]; then
    user_opt=$reg_userid
fi
if [ -z "$tbsm_password" ]; then
    password_opt=$reg_password
fi

# process args
while getopts U:S:P:p:v:c:r:o:f:m:i:u:t:Eh c; do
  case $c in
  u)  if [ "$c" = "u" ]; then
        suinstid="$OPTARG"
      elif [ "$c" = "U" ]; then
        user_opt="$OPTARG"
      fi;;
  p)  if [ "$c" = "p" ]; then
        sprod="$OPTARG"
      elif [ "$c" = "P" ]; then
        password_opt="$OPTARG"
      fi;;
  S)  server_opt="$OPTARG";;
  E)  e_opt=1;;
  c)  sclass="$OPTARG";;
  r)  soid="$OPTARG";;
  o)  sobjpath="$OPTARG";;
  f)  sFile="$OPTARG";;
  m)  snflag="$OPTARG";;
  i)  sinstid="$OPTARG";;
  v)  sver="$OPTARG";;
  t)  spriority="$OPTARG";;
  h)  Usage; exit;;

  *)  Usage; exit;;
  esac
done

# Check for required parameters
CheckRequiredParams

# Invoke asisp_loadSetBSPriority proc to resolve cid/id resource identification required
# to set business system priority.  A file is created with the required parameters whether one or 
# multiple resources are specified.

tempparmbcp=$(pwd)"/bcpparmtemp.txt"
rm -f "$tempparmbcp"
tempinstbcp=$(pwd)"/bcpinsttemp.txt"
rm -f "$tempinstbcp"

if [ ! -z "$sFile" ]; then
  if [ -a "$sFile" ]; then
    ParseFile
  else
    ASIShellMsg ASIMiscScripts 76 "-f<$sFile>"
    exit 2
  fi
else
  print >> "$tempparmbcp" 1"|"$spriority"|"
  # Load the resource identification for #Instances
  # Build prod;ver string
  if [ ! -z "$sver" ]; then
    sver=";"$sver
  fi
  
  print >> "$tempinstbcp" "|"$soid"|"1"|"$sclass"|"$sprod$sver"|"$sinstid"|"$suinstid"|"$sobjpath"|"$snflag"|"
fi

if [ -s "$tempinstbcp" -a -s "$tempparmbcp" ]; then
  CallLoadParms
fi

rm -f "$tempinstbcp"
rm -f "$tempparmbcp"  

