-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/al_processgenericmsgs.sql
-- SID: 1.36
-- Date Modified: 04/10/15 15:55:12
--
-- Module Description
--   Defines the al_processGenericMsgs stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- al_processgenericmsgs.sql
PRINT 'Creating procedure al_processGenericMsgs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='al_processGenericMsgs')
  DROP PROCEDURE al_processGenericMsgs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/al_processgenericmsgs.sql
-- SID: 1.36
-- Date Modified: 04/10/15 15:55:12
CREATE PROCEDURE al_processGenericMsgs
AS
  SET NOCOUNT ON
  RAISERROR('al_processGenericMsgs: ', 0, 1) WITH NOWAIT
/*
  -- Tables created by caller:
  IF 0=1 BEGIN
    CREATE TABLE #al_generic (
      trid  INT,
      keyword NVARCHAR(64),
      value NVARCHAR(425) NULL
    )
    CREATE TABLE #al_log (
      trid  INT,
      status  NVARCHAR(8)   NULL,
      msg NVARCHAR(512)   NULL,
      time  DATETIME  DEFAULT GETDATE()
    )
  END
*/

  -- Temporary table definitions
  CREATE TABLE #bulkcreatetable (
    id    ObjID DEFAULT 0,
    name  NVARCHAR(255),
    work_int  INT DEFAULT 0
  )
  CREATE TABLE #al_bulk_upd (
    trid  INT,
    cid   ClassID NULL,
    id  ObjID NULL
  )
  CREATE TABLE #al_trids (
    trid  INT,
    work_int  INT   NULL
  )
  CREATE TABLE #al_msgs (
    trid  INT,
    class   NVARCHAR(128)   NULL,
    instid  NVARCHAR(425)   NULL,
    instname  NVARCHAR(255)   NULL,
    tcphost NVARCHAR(255)   NULL,
    tivhost NVARCHAR(255)   NULL,
    tivtmr  NVARCHAR(255)   NULL,
    probeid   NVARCHAR(255)   NULL,
    probename   NVARCHAR(255)   NULL,
    probeargs NVARCHAR(255)   NULL,
    probetype NVARCHAR(255)   NULL,
    msg   NVARCHAR(255)   NULL,
    severity  NVARCHAR(64)  NULL,
    eehost  NVARCHAR(255)   NULL,
    adaphost  NVARCHAR(255)   NULL,
    time_sql  DATETIME  NULL,
    ostype  INT     NULL,
    cid   ClassID   NULL,
    id    ObjID     NULL,
    comptype  NVARCHAR(8)   NULL,
    trtype  INT     NULL,
    work_int  INT     NULL
  )
  CREATE CLUSTERED INDEX trid#al_msgs ON #al_msgs (trid) 

  DECLARE @rc INT, @err INT, @fs INT, @r INT

  -- Get only needed attributes. The rest are done below after forwarding to CL
  INSERT #al_msgs (trid) SELECT DISTINCT trid FROM #al_generic
  
  UPDATE am SET am.class=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='class'
  UPDATE am SET am.instid=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='InstanceID'
  UPDATE am SET am.probeid=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='probeid'
  UPDATE am SET am.probename=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='probename'
  UPDATE am SET am.probeargs=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='probeargs'
  UPDATE am SET am.msg=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='msg'

  -- Get transaction type. Validate probeid
  UPDATE am SET am.trtype=st.trType FROM #al_msgs am
    JOIN AL_Stage st ON (st.trId=am.trid)
  UPDATE #al_msgs SET probeid=probename WHERE probeid IS NULL OR probeid=''
  UPDATE #al_msgs SET probeid=msg WHERE probeid IS NULL OR probeid=''
  UPDATE #al_msgs SET probeid=REPLACE(probeid+' '+probeargs, '|', '')
    WHERE probeargs IS NOT NULL
  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Event doesnt seem to be a real msg or exception (probeid is empty). Ignore it.' FROM #al_msgs
    WHERE probeid IS NULL OR probeid=''
  DELETE FROM #al_msgs WHERE probeid IS NULL OR probeid=''
  UPDATE #al_msgs SET msg='(No description provided)' WHERE msg IS NULL OR msg=''

  -- Remove trailing ';' from class
  UPDATE #al_msgs SET class = SUBSTRING(class, 1, LEN(class) - 1)
  WHERE (CHARINDEX(';', class, LEN(class)) = LEN(class))

  -- Get class id (supports product;version or product or cid or profile format)
  UPDATE am SET am.cid=gl.cid, am.comptype=gl.comptype FROM #al_msgs am, GEMLookupCID gl
    WHERE gl.prodname=SUBSTRING(am.class, 1, CHARINDEX(';', am.class)-1) AND
      gl.version=SUBSTRING(am.class, CHARINDEX(';', am.class)+1, LEN(am.class)-CHARINDEX(';', am.class)) AND
    am.class LIKE '%;%'
  UPDATE am SET am.cid=gl.cid, am.comptype=gl.comptype FROM #al_msgs am
    JOIN GEMLookupCID gl ON (am.class=gl.prodname)
    WHERE am.cid IS NULL AND gl.version=''
  UPDATE am SET am.cid=gl.cid, am.comptype=gl.comptype FROM #al_msgs am
    JOIN GEMLookupCID gl ON (am.class=gl.cid)
    WHERE am.cid IS NULL
  UPDATE am SET am.cid=gl.cid, am.comptype=gl.comptype FROM #al_msgs am
    JOIN GEM_DMtoCID gd ON (gd.profileName=am.class)
    JOIN GEMLookupCID gl ON (gl.cid=gd.CID)
    WHERE am.cid IS NULL


  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN al_processGenericMsgs
  ELSE
    SAVE TRAN al_processGenericMsgs

   -- Forward to Common Listener (if only through generic api ie trtype=4)
  CREATE TABLE #al_generic_cl (
    trid  INT
  )
  
  INSERT #al_generic_cl SELECT trid FROM #al_msgs
    WHERE cid IS NULL AND class<>'%' AND trtype=4
  SET @rc = @@ROWCOUNT

  IF @rc > 0 BEGIN
    EXEC al_sendToCL

    INSERT #al_log (trid, status, msg) SELECT trid, 'OK', 'Forwarded message to Common Listener' FROM #al_generic_cl

    -- Clean up #al_msgs
    DELETE FROM #al_msgs WHERE cid IS NULL AND class<>'%' AND trtype=4

    TRUNCATE TABLE #al_generic_cl
  END

  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Class is NULL' FROM #al_msgs
    WHERE cid IS NULL AND class IS NULL AND trtype=4
  DELETE FROM #al_msgs WHERE cid IS NULL AND class IS NULL AND trtype=4

  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Invalid Class ID' FROM #al_msgs
    WHERE cid IS NULL AND class<>'%'
  DELETE FROM #al_msgs WHERE cid IS NULL AND class<>'%'

  -- invalid message if no instance id for generic
  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'No InstanceID. Event is invalid' FROM #al_msgs
    WHERE comptype='generic' AND (instid IS NULL OR instid='')
  DELETE FROM #al_msgs WHERE comptype='generic' AND (instid IS NULL OR instid='')

  -- Adjust base epoch time from the local time to the gmt at that time
  DECLARE @deltaGMT INT, @basetime DATETIME
  EXEC master.dbo.xp_regread 'HKEY_LOCAL_MACHINE',
           'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
           'ActiveTimeBias',
           @deltaGMT OUTPUT
  SET @basetime = DATEADD(MINUTE, (-@deltaGMT), '01/01/1970 00:00:00')

  -- Get rest of attributes into #al_msgs (after forwarding to CL)
  UPDATE am SET am.instname=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='name'

/*
  In 211, we had one TCP hostname (lets call it hostname1) and in 31, we have 
  two - the old hostname1 and a new one lets say hostname2.
  hostname1 is fully qualified but is truncatable with -t or -T option on ee_config
  hostname2 is fully qualified but is truncatable with -T option on ee_config. 
  This is the one used by TEDW (warehouse).
  So hostname1 and hostname2 will be same if -t or -T are not used on ee_config.

  Now the way AL uses these hostnames is:
  - hostname2 is stored in TCPHost attribute of _V class table
  - hostname1 is used for everything else i.e, in creating parent tree, adding 
    into GEM_IDLookup cache for later lookup of the resource etc.
*/
  UPDATE am SET am.tcphost=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='TCPHost_211'
  UPDATE am SET am.tivhost=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='MgedSystemName'
  UPDATE am SET am.tivtmr=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='EPTTMR'
  UPDATE am SET am.probetype=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='probetype'
  UPDATE am SET am.severity=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='severity'
  UPDATE am SET am.time_sql=DATEADD(ss, CAST(g.value AS INT), @basetime) FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='time' AND ISNUMERIC(g.value)=1
  UPDATE am SET am.ostype=CAST(g.value AS INT) FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='ostype'
  UPDATE am SET am.eehost=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='EEhost'
  UPDATE am SET am.adaphost=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='adapterhost'

  -- Check to see if the resource already exists in the Agent Listener cache
  UPDATE am SET am.id=il.IID FROM #al_msgs am
    JOIN GEM_IDLookup il ON (am.cid=il.CID AND il.key_value=am.instid)
    WHERE il.key_name='KeyGen' AND am.comptype='generic'
  UPDATE am SET am.cid=il.CID, am.id=il.IID FROM #al_msgs am
    JOIN GEM_IDLookup il ON (il.key_value=am.instid)
    WHERE am.id IS NULL  AND il.key_name='KeyGen' AND am.comptype='generic' AND am.class='%'
  -- Check to see if the resource already exists in the Common Listener cache
  UPDATE am SET am.id = cl.oid FROM #al_msgs am
    JOIN CL_IDCache cl ON (am.cid = cl.cid AND am.instid = cl.value)
    WHERE cl.keyword = 'CL_InstID' AND cl.deleted = 0 AND am.comptype = 'generic'


  -- Compute the host/tmr id in tivtmr column and use it later to find oid
  UPDATE #al_msgs SET tivtmr=tivhost+'|'+CASE WHEN tivtmr IS NOT NULL AND tivtmr<>'' THEN tivtmr ELSE '%' END
    WHERE id IS NULL AND comptype<>'generic' AND tivhost IS NOT NULL AND tivhost<>''
  UPDATE am SET am.id=il.IID FROM #al_msgs am
    JOIN GEM_IDLookup il ON (am.cid=il.CID AND il.key_value=am.tivtmr)
    WHERE am.id IS NULL AND il.key_name='KeyTivName' AND am.comptype<>'generic' AND
    am.tivhost IS NOT NULL AND am.tivhost<>''

  UPDATE am SET am.id=il.IID FROM #al_msgs am
    JOIN GEM_IDLookup il ON (am.cid=il.CID AND il.key_value=am.tcphost)
    WHERE am.id IS NULL AND il.key_name='KeyIp' AND am.comptype<>'generic' AND
    am.tcphost IS NOT NULL AND am.tcphost<>''
  -- delete gone messages that are not yet created
  --INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Gone message received, no instance created' FROM #al_msgs
  --  WHERE probetype='GONE' AND id IS NULL
  --DELETE FROM #al_msgs WHERE probetype='GONE' AND id IS NULL

  -- only class type of generic can be created
  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Waiting for instance to be discovered' FROM #al_msgs
    WHERE comptype<>'generic' AND trtype=4 AND id IS NULL
  DELETE FROM #al_msgs WHERE comptype<>'generic' AND trtype=4 AND id IS NULL

  -- delete when class is wild-card and resource cant be found
  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Could not locate resource when class is wild-card' FROM #al_msgs
    WHERE id IS NULL AND class='%'
  DELETE FROM #al_msgs WHERE id IS NULL AND class='%'

  -- For SENTRY, instid is tivtmr and instname is tivhost
  UPDATE #al_msgs SET instid=tivtmr WHERE trtype=3
  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Cannot create instance, hostname is not given' FROM #al_msgs
    WHERE id IS NULL AND trtype=3 AND (tivhost IS NULL OR tivhost='')
  DELETE FROM #al_msgs WHERE id IS NULL AND trtype=3 AND (tivhost IS NULL OR tivhost='')
  -- set display name in #al_generic
  -- (use work_int=4)
  DELETE FROM #al_generic WHERE keyword='name' AND (value IS NULL OR value='')
  UPDATE #al_msgs SET work_int=4 WHERE instname IS NULL OR instname=''
  UPDATE #al_msgs SET instname=instid WHERE work_int=4 AND trtype<>3
  UPDATE #al_msgs SET instname=tivhost WHERE work_int=4 AND trtype=3
  INSERT #al_generic SELECT trid, 'name', instname FROM #al_msgs
    WHERE id IS NULL AND work_int=4

  UPDATE #al_msgs SET work_int=NULL

  CREATE TABLE #al_bulk_add (
         trid  INT,
         cid ClassID,
         instid  NVARCHAR(425),
         tcphost NVARCHAR(255) NULL,
         trtype  INT,
         tivtmr  NVARCHAR(255) NULL
       )

  DECLARE @exit_outer_loop INT
  DECLARE @exit_inner_loop INT
  DECLARE @last_trid INT
  SET @exit_outer_loop = 0
  SET @last_trid = 0

  WHILE @exit_outer_loop = 0 
  BEGIN

     SET @exit_inner_loop = 0

/*
    -- Defect 131647: AL: Generic Obj Inst Deleted if TCPhost Modified for Given Inst
    -- As part of the defect, commented out 2 statements (search for defect # if you need to uncomment them)
    -- With this change, a generic resource will no longer be deleted when TCPHost attribute changes

    -- Mark resources to delete when tcphost changed
    -- Need to update cache whenever _TCPHost attribute is changed
    UPDATE am SET work_int=2 FROM #al_msgs am
      JOIN GEM_IDLookup gl ON (am.cid=gl.CID AND am.id=gl.IID AND am.tcphost<>gl.key_value)
      WHERE gl.key_name='KeyIp' AND am.comptype='generic' AND
      am.tcphost IS NOT NULL AND am.tcphost<>''
*/
    
    -- Processing in batches till the next GONE/TCP_CHANGED
    -- work_int = 1 - resource does not exist, 2 - tcphost changed, 3 - gone
    DECLARE @ptype INT, @maxid INT, @begin INT, @end INT
    DECLARE @cid ClassID, @id ObjID, @sql NVARCHAR(128), @instid NVARCHAR(425)
    
    -- Mark resources that are yet to be created
    UPDATE #al_msgs SET work_int=1 WHERE id IS NULL
    UPDATE #al_msgs SET work_int=3 WHERE probetype='GONE'
    INSERT INTO #al_trids SELECT trid, work_int FROM #al_msgs WHERE work_int>=1 AND trid >= @last_trid
    SELECT @begin=MIN(trid) FROM #al_trids
    SELECT @maxid=MAX(trid) FROM #al_trids
    SELECT @end=MIN(trid) FROM #al_trids WHERE work_int>1 AND trid>=@begin
    IF @end IS NULL
      SET @end = @maxid
    ELSE IF @end <> @begin
      SET @end = @end - 1
    
    IF @begin IS NULL
    BEGIN
      SET @exit_outer_loop = 1
      SET @exit_inner_loop = 1
    END

    WHILE @exit_inner_loop = 0 
    BEGIN

      SELECT @ptype=work_int FROM #al_trids WHERE trid=@begin
    
      -- Delete if probetype is GONE or tcphost changed
      IF @ptype > 1 
      BEGIN
        
        SELECT @cid=cid, @id=id, @instid=instid FROM #al_msgs WHERE trid=@begin
        SET @sql = 'delete_' + @cid
        EXEC @rc = @sql @id

        DELETE FROM GEM_IDLookup WHERE CID=@cid AND IID=@id

        IF @ptype = 3
        BEGIN

          --Do not process events received prior to this one
          UPDATE am SET work_int=5 FROM #al_msgs am
            WHERE am.cid=@cid AND am.instid=@instid AND am.trid < @begin 

          -- Clear id for deleted resource
          UPDATE am SET id=NULL FROM #al_msgs am
             WHERE cid=@cid AND id=@id --AND trid > @begin 



          INSERT #al_log (trid, status, msg) SELECT @begin, 'OK', 'Deleted resource as its GONE'
          DELETE FROM #al_msgs WHERE trid=@begin
          SET @begin = @begin + 1

        END

        ELSE IF @ptype = 2 
        BEGIN

          --Do not process events received prior to this one
          UPDATE am SET work_int=5 FROM #al_msgs am
            WHERE am.cid=@cid AND am.instid=@instid AND am.trid < @begin 

          -- Clear id for deleted resource
          UPDATE am SET id=NULL FROM #al_msgs am
             WHERE cid=@cid AND id=@id -- AND trid >= @begin 

          -- The variable @begin is not incremented as a new resource has to be
          -- created for that message
          UPDATE #al_trids SET work_int=1 WHERE trid=@begin
          INSERT #al_log (trid, status, msg) SELECT @begin, 'OK', 'Deleted resource as tcphost changed'
        END

      -- need to exit this loop
      SET @exit_inner_loop = 1
      SET @last_trid=@begin

      END

      ELSE BEGIN  -- add
        TRUNCATE TABLE #al_bulk_add
    
          INSERT #al_bulk_add SELECT am.trid, am.cid, am.instid, am.tcphost, am.trtype, am.tivtmr FROM #al_msgs am
          JOIN #al_trids aid ON (am.trid=aid.trid)
          WHERE aid.trid BETWEEN @begin AND @end
    
        -- Delete any dups before creating
        DELETE FROM #al_bulk_add WHERE trid > (
          SELECT MIN(trid) FROM #al_bulk_add ba
            WHERE ba.cid=#al_bulk_add.cid AND ba.instid=#al_bulk_add.instid
        )
    
        DECLARE @createproc NVARCHAR(32)
        DECLARE add_c CURSOR FOR SELECT DISTINCT cid FROM #al_bulk_add
        OPEN add_c
        FETCH NEXT FROM add_c INTO @cid
        SET @fs = @@FETCH_STATUS
    
        WHILE @fs <> -1 BEGIN
          TRUNCATE TABLE #bulkcreatetable
          INSERT #bulkcreatetable(name)
            SELECT DISTINCT instid FROM #al_bulk_add WHERE cid=@cid
          
          SET @createproc = 'cl_process' + @cid
          EXEC @rc = @createproc @action='BULKCREATE'
          
          IF @@ERROR <> 0 OR @rc <> 0 BEGIN
            RAISERROR('Failed to create resources of class %s', 0, 1, @cid) WITH NOWAIT
            ROLLBACK TRAN al_processGenericMsgs
            INSERT #al_log (trid, status, msg) SELECT 0, 'ERROR', 'Failed to create resources of class (' + @cid + ')'
            RETURN -30000
          END

          ELSE BEGIN
            -- insert the new ids in cache
            INSERT GEM_IDLookup (CID, IID, key_name, key_value)
              SELECT DISTINCT @cid, bc.id, 'KeyGen', bc.name FROM #bulkcreatetable bc
              JOIN #al_bulk_add ba ON (bc.name=ba.instid)
              WHERE ba.trtype=4 AND bc.id>0

            IF @@ERROR <> 0 BEGIN
              RAISERROR('Failed to add KeyGen key after creating objects of class %s', 0, 1, @cid) WITH NOWAIT
              ROLLBACK TRAN al_processGenericMsgs
              INSERT #al_log (trid, status, msg) SELECT 0, 'ERROR', 'Failed to add KeyGen key after creating objects of class (' + @cid + ')'
              RETURN -30000
            END
    
            INSERT GEM_IDLookup (CID, IID, key_name, key_value)
              SELECT DISTINCT @cid, bc.id, 'KeyTivName', ba.tivtmr FROM #bulkcreatetable bc
              JOIN #al_bulk_add ba ON (bc.name=ba.instid)
              WHERE ba.trtype=3 AND bc.id>0

            IF @@ERROR <> 0 BEGIN
              RAISERROR('Failed to add KeyTivName key after creating objects of class %s', 0, 1, @cid) WITH NOWAIT
              ROLLBACK TRAN al_processGenericMsgs
              INSERT #al_log (trid, status, msg) SELECT 0, 'ERROR', 'Failed to add KeyTivName key after creating objects of class (' + @cid + ')'
              RETURN -30000
            END
    
            INSERT GEM_IDLookup (CID, IID, key_name, key_value)
              SELECT DISTINCT @cid, bc.id, 'KeyIp', ba.tcphost FROM #bulkcreatetable bc
              JOIN #al_bulk_add ba ON (ba.instid=bc.name)
              WHERE bc.id>0 AND ba.tcphost IS NOT NULL AND ba.tcphost<>''

            IF @@ERROR <> 0 BEGIN
              RAISERROR('Failed to add KeyIp key after creating objects of class %s', 0, 1, @cid) WITH NOWAIT
              ROLLBACK TRAN al_processGenericMsgs
              INSERT #al_log (trid, status, msg) SELECT 0, 'ERROR', 'Failed to add KeyIp key after creating objects of class (' + @cid + ')'
              RETURN -30000
            END

            INSERT #al_log (trid, status, msg) SELECT trid, 'OK', 'Created resource with instid ('+instid+')'
              FROM #al_bulk_add WHERE cid=@cid
    
          END
            
    
          FETCH NEXT FROM add_c INTO @cid
          SET @fs = @@FETCH_STATUS
        END
        DEALLOCATE add_c
        TRUNCATE TABLE #al_bulk_add 
    
        -- update the new ids in #al_msgs for events
        UPDATE am SET am.id=il.IID FROM #al_msgs am
          JOIN GEM_IDLookup il ON (am.cid=il.CID AND  il.key_value=am.instid)
          JOIN #al_trids aid ON (aid.trid=am.trid)
          WHERE il.key_name IN ('KeyGen', 'KeyTivName')

/*
        -- Defect 131647: AL: Generic Obj Inst Deleted if TCPhost Modified for Given Inst
        -- As part of the defect, commented out 2 statements (search for defect # if you need to uncomment them)
        -- With this change, a generic resource will no longer be deleted when TCPHost attribute changes

        UPDATE aid SET work_int=2 FROM #al_trids aid
          JOIN #al_msgs am ON (am.trid=aid.trid)
          JOIN GEM_IDLookup gl ON (am.cid=gl.CID AND am.id=gl.IID AND am.tcphost<>gl.key_value)
          WHERE gl.key_name='KeyIp' AND am.comptype='generic' AND
          am.tcphost IS NOT NULL AND am.tcphost<>''
          AND am.trid>@begin 
*/

        SET @rc = @@ROWCOUNT
        
        SELECT @begin=MIN(trid) FROM #al_trids WHERE work_int>1 AND trid>@begin
        
        IF @begin IS NULL
          SET @begin = @end + 1

        SET @last_trid=@begin - 1

      END

      -- Increment the loop
      SELECT @end=MIN(trid) FROM #al_trids WHERE work_int>1 AND trid>=@begin
      IF @end IS NULL
        SET @end = @maxid
      ELSE IF @end <> @begin
        SET @end = @end - 1

      -- If all trids processed exit both loops
      IF @begin > @maxid  
      BEGIN
         SET @exit_inner_loop = 1
         SET @exit_outer_loop = 1
      END


      INSERT #al_log (trid, status, msg) SELECT trid, 'OK', 'Event has been deleted. (instID='+instid+')'
        FROM #al_msgs WHERE work_int=5

      DELETE FROM #al_msgs WHERE work_int=5

    END
  
  TRUNCATE TABLE #al_trids

  END --@exit_outer_loop

  -- Check if id is known after bulk-create
  INSERT #al_log (trid, status, msg) SELECT trid, 'ERROR', 'Could not create or locate resource (instID='+instid+')'
    FROM #al_msgs WHERE id IS NULL
  DELETE FROM #al_msgs WHERE id IS NULL

  -- Update resources attributes
  CREATE TABLE #inst_attrs (
    oid ObjID,
    keyword NVARCHAR(64),
    value NVARCHAR(512) NULL
  )
  DECLARE @updateproc NVARCHAR(128)


  -- Selectively object-update only when msgtype is heartbeat
  -- Use class column in #al_msgs to stored msgtype setting
  UPDATE am SET am.class=g.value FROM #al_msgs am
    JOIN #al_generic g ON (am.trid=g.trid)
    WHERE g.keyword='msgtype'
  INSERT INTO #al_bulk_upd SELECT trid, cid, id FROM #al_msgs
    WHERE work_int IN (1) OR
          (comptype='generic' AND class='heartbeat')

  -- Update LastHbTimestamp
  INSERT #al_generic SELECT trid, 'LastHbTimestamp', CAST(CURRENT_TIMESTAMP AS NVARCHAR(30))
    FROM #al_bulk_upd

  -- Update resource using its last event if there are dups
  DELETE FROM #al_bulk_upd WHERE trid < (
    SELECT MAX(trid) FROM #al_bulk_upd dup
    WHERE dup.cid=#al_bulk_upd.cid AND dup.id=#al_bulk_upd.id
  )

  DECLARE upd_c CURSOR FOR SELECT DISTINCT cid FROM #al_bulk_upd
  OPEN upd_c
  FETCH NEXT FROM upd_c INTO @cid
  SET @fs = @@FETCH_STATUS

  WHILE @fs <> -1 BEGIN
    TRUNCATE TABLE #inst_attrs
    INSERT #inst_attrs SELECT a.id, b.keyword, b.value FROM #al_bulk_upd a
      JOIN #al_generic b ON (a.trid=b.trid) WHERE a.cid=@cid

    INSERT #inst_attrs SELECT a.id, 'Manufacturer', b.manufacturer FROM #al_bulk_upd a
      JOIN GEMLookupCID b ON (a.cid=b.cid) WHERE a.cid=@cid

    INSERT #inst_attrs SELECT a.id, 'Product', b.prodname FROM #al_bulk_upd a
      JOIN GEMLookupCID b ON (a.cid=b.cid) WHERE a.cid=@cid

    INSERT #inst_attrs SELECT a.id, 'Version', b.version FROM #al_bulk_upd a
      JOIN GEMLookupCID b ON (a.cid=b.cid) WHERE a.cid=@cid

    SET @updateproc = 'cl_process' + @cid
    EXEC @rc = @updateproc @action='ADD_UPDATE'

    IF @@ERROR <> 0 BEGIN
      RAISERROR('Failed to update attributes of class %s', 0, 1, @cid) WITH NOWAIT
      ROLLBACK TRAN al_processGenericMsgs
      INSERT #al_log (trid, status, msg) SELECT 0, 'ERROR', 'Failed to update attributes of class of class (' + @cid + ')'
      RETURN -30000
    END

    INSERT #al_log (trid, status, msg) SELECT trid, 'OK', 'Updated resource (' + @cid + ', ' + CAST(id AS NVARCHAR(9)) +')'
      FROM #al_bulk_upd WHERE cid=@cid

    SET @fs = 0
    FETCH NEXT FROM upd_c INTO @cid
    SET @fs = @@FETCH_STATUS
  END
  DEALLOCATE upd_c
  TRUNCATE TABLE #al_bulk_upd
  TRUNCATE TABLE #inst_attrs

  -- Evaluate who the parent is and create links for newly-created resources
  /* Users can use either (1) old logic to create parent or
     (2) new logic
     This can be set on per event basis. If not specified, it will pick up the
     default setting in the database which is initially set to old logic. But this
     can be changed to any rule in new logic.
  */

  CREATE TABLE #linkChanges (
    type NVARCHAR(4),
    src_cid ClassID,
    src_id ObjID,
    dst_cid ClassID,
    dst_id ObjID,
    action_type sysname
  )

  DECLARE @parentid ObjID, @parentclass ClassID, @msglocal NVARCHAR(255), @trid INT
  DECLARE @tcphostname NVARCHAR(255), @hostos INT, @eehost NVARCHAR(255), @prule NVARCHAR(255)
  DECLARE upd_d CURSOR FOR SELECT DISTINCT cid, id FROM #al_msgs
    WHERE work_int IN (1, 2)
  OPEN upd_d
  FETCH NEXT FROM upd_d INTO @cid, @id
  SET @fs = @@FETCH_STATUS

  WHILE @fs <> -1 BEGIN
    SET @parentid = 0
    SELECT @trid=MIN(trid) FROM #al_msgs WHERE cid=@cid AND id=@id
    SELECT @prule=value FROM #al_generic WHERE trid=@trid AND keyword='placement'

    EXEC @rc = al_autoPlacement @trid, @cid, @id, @prule, @parentclass OUTPUT, @parentid OUTPUT

    SET @err = @@ERROR
    IF @err <> 0 OR @rc <> 0 BEGIN
      RAISERROR('al_processGenericMsgs: Failed on call to al_autoPlacement for Class ID = %s, ID = %d, Placement Rule = %s, al_autoPlacement Return Code = %d, SQL Error Code = %d.', 0, 1, @cid, @id, @prule, @rc, @err) WITH NOWAIT
      IF @prule IS NULL
        SET @prule = 'null'
      INSERT #al_log (trid, status, msg) SELECT @trid, 'ERROR', 'al_processGenericMsgs: Failed on call to al_autoPlacement for Class ID = ' + @cid + ', ID = ' + CAST(@id AS NVARCHAR(9)) + ', Placement Rule = ' + @prule + ', al_autoPlacement Return Code = ' + CAST(@rc AS NVARCHAR(9)) + ', SQL Error Code = ' + CAST(@err AS NVARCHAR(9)) + '.'
      -- Ensure event is not lost and processed using default resource placement and/or event processing
      SET @parentid = 0
    END

    IF @parentid IS NULL OR @parentid <= 0 BEGIN
      SELECT @tcphostname=tcphost, @hostos=ostype, @eehost=eehost FROM #al_msgs
        WHERE trid=@trid
      EXEC gemsp_get_parent @cid, @tcphostname, @tcphostname, @hostos, @eehost,
                            @parentid OUTPUT, @parentclass OUTPUT, @msglocal OUTPUT
    END

    IF @parentid > 0 BEGIN

      EXEC @rc = asisp_createlink 'PHYC', @parentclass, @parentid, @cid, @id

      IF @@ERROR <> 0 OR @rc <> 0 BEGIN
        RAISERROR('Failed to create PHYC link for class %s', 0, 1, @cid) WITH NOWAIT
        ROLLBACK TRAN al_processGenericMsgs
        INSERT #al_log (trid, status, msg) SELECT @trid, 'ERROR', 'Failed to create PHYC link for class (' + @cid + ')'
        DEALLOCATE upd_d
        RETURN -30000
      END

      INSERT #linkChanges (type, src_cid, src_id, dst_cid, dst_id, action_type) VALUES 
           ('PHYC', @parentclass, @parentid, @cid, @id, 'LinkCreated')

      INSERT #al_log (trid, status, msg) SELECT @trid, 'OK', 'Creating parent link (' + @cid + '-' + CAST(@id AS NVARCHAR(9)) + ', ' + @parentclass + '-' + CAST(@parentid AS NVARCHAR(9)) + ')'
    END
    ELSE
      INSERT #al_log (trid, status, msg) SELECT @trid, 'ERROR', 'Failed to create parent link  for (' + @cid + '-' + CAST(@id AS NVARCHAR(9)) + ')'

    FETCH NEXT FROM upd_d INTO @cid, @id
    SET @fs = @@FETCH_STATUS
  END
  DEALLOCATE upd_d

  -- Insert new links into PendingNotification table
  EXEC cl_notifyLinkChanges

  -- Delete any overdue exception for APM_HEARTBEAT
  CREATE TABLE #hb_overdue (
    trid  INT          NOT NULL,
    cid   ClassID      NOT NULL,
    id    ObjID        NOT NULL
  )
  INSERT INTO #hb_overdue SELECT am.trid, am.cid, am.id FROM #al_msgs am
    JOIN Exception_C e ON e._ParentCID=am.cid AND e._ParentID=am.id
    WHERE am.comptype='generic' AND e.deleted=0 AND e._EXCP_NAME='APM_HEARTBEAT'

  INSERT StagedEXCP (
    parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, EXCP_OBJ_TYPE,
    EXCP_OBJ_NAME, EXCP_NAME, EXCP_CD, nEXCH, AlertState, Priority, Flags
  )
  SELECT cid, id, CURRENT_TIMESTAMP, '', '', '', '', '', 'APM_HEARTBEAT', 'OK', 1, -1, -1, 1
    FROM #hb_overdue

  INSERT #al_log (trid, status, msg) SELECT trid, 'OK', 'Removed overdue heartbeat for (' + cid + ', ' + CAST(id AS NVARCHAR(9)) + ')'
    FROM #hb_overdue
  TRUNCATE TABLE #hb_overdue

  -- Post events
  CREATE TABLE #staged_evt
  (
    eid         INT,
    type        NVARCHAR(16)  DEFAULT 'EXCP',
    action      NVARCHAR(255)   DEFAULT '',
    parent_cid  ClassID   NULL,
    parent_id   ObjID   NULL,
    evt_name    NVARCHAR(255) NULL,
    evt_cd      NVARCHAR(255)   DEFAULT '',
    state       NVARCHAR(40)  DEFAULT '',
    stateid     INT           NULL,
    alertstate  INT DEFAULT   1,
    priority    INT DEFAULT   5,
    time        DATETIME  NULL,
    severity    NVARCHAR(64)  NULL,
  )
  CREATE TABLE #bulk_events_ext
  (
    eid     INT,
    eventhandle   NVARCHAR(64)  NULL,
    serverhandle  NVARCHAR(64)  NULL,
    datereception NVARCHAR(16)  NULL,
    tecclass    NVARCHAR(64)  NULL,
    status    NVARCHAR(32)  NULL,
    eehost    NVARCHAR(128) NULL,
    admin   NVARCHAR(128) NULL,
    isstatus    INT       DEFAULT 0
  )

  CREATE CLUSTERED INDEX eid#bulk_events_ext ON #bulk_events_ext (eid) 

  INSERT #staged_evt (eid, type, parent_cid, parent_id, evt_name, evt_cd, time, severity)
    SELECT trid,
           CASE probetype WHEN 'MSG' THEN 'MSG' ELSE 'EXCP' END,
           cid, id, probeid, msg,
           CASE WHEN time_sql IS NULL THEN CURRENT_TIMESTAMP ELSE time_sql END,
           severity 
    FROM #al_msgs WHERE id IS NOT NULL


  UPDATE se SET se.alertstate=gs.AlertState, se.priority=gs.Priority FROM #staged_evt se
    JOIN GEMSeverities gs ON (se.severity=gs.Severity)

  -- Extended event attributes
  INSERT #bulk_events_ext (eid) SELECT trid from #al_msgs
  UPDATE be SET be.eventhandle=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='eventhandle'
  UPDATE be SET be.serverhandle=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='serverhandle'
  UPDATE be SET be.datereception=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='datereception'
  UPDATE be SET be.tecclass=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='tecclass'
  UPDATE be SET be.status=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='status'
  UPDATE be SET be.eehost=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='EEhost'
  UPDATE be SET be.admin=g.value FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='admin'
  UPDATE be SET be.isstatus=CAST(g.value AS INT) FROM #bulk_events_ext be
    JOIN #al_generic g ON (be.eid=g.trid)
    WHERE g.keyword='isstatus' AND ISNUMERIC(g.value)=1
  DELETE FROM #bulk_events_ext WHERE eventhandle IS NULL

  -- Set events and messages in staged tables
  CREATE TABLE #event_log (
    eid INT,
    status VARCHAR(16)  NULL,
    disp NVARCHAR(1024) NULL
  )

  EXEC cl_setBulkEvents 1
  INSERT #al_log (trid, status, msg) SELECT eid, status, disp FROM #event_log

  TRUNCATE TABLE #event_log
  TRUNCATE TABLE #staged_evt
  TRUNCATE TABLE #bulk_events_ext

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRAN al_processGenericMsgs

GO




