-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/al_processmsgs.sql
-- SID: 1.17
-- Date Modified: 04/10/22 13:01:46
--
-- Module Description
--   Defines the al_processMsgs stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- al_processmsgs.sql
PRINT 'Creating procedure al_processMsgs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='al_processMsgs')
  DROP PROCEDURE al_processMsgs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/al_processmsgs.sql
-- SID: 1.17
-- Date Modified: 04/10/22 13:01:46
CREATE PROCEDURE al_processMsgs
AS
  SET ANSI_WARNINGS OFF
  CREATE TABLE #al_generic (
    trid 	INT,
    keyword	NVARCHAR(64),
    value	NVARCHAR(255) 	NULL
  )
  CREATE TABLE #al_log (
    trid	INT,
    status 	NVARCHAR(8) 	NULL,
    msg		NVARCHAR(512) 	NULL,
    time	DATETIME	DEFAULT GETDATE()
  )
  CREATE TABLE #trids (
    trid 	INT,
    type	INT
  )

  DECLARE @btrid INT, @etrid INT, @maxtrid INT, @maxcnt INT, @mc NVARCHAR(128), @type INT
  DECLARE @rc INT, @err INT
  SET @maxcnt = 500
  SELECT @mc=_Value FROM GEM_Options WHERE _Option='max count of generic events to process'
  IF ISNUMERIC(@mc) = 1
    SET @maxcnt = CAST(@mc AS INT)

  INSERT #trids SELECT TOP 500 trId, trType FROM AL_Stage WHERE trState=1

  -- set type=3 for sentry and generic so they can be processed in set
  UPDATE #trids SET type=3 WHERE type=4
  SELECT @btrid=MIN(trid) FROM #trids
  SELECT @maxtrid=MAX(trid) FROM #trids

  SELECT @type=type FROM #trids WHERE trid=@btrid
  SELECT @etrid=MIN(trid) FROM #trids WHERE trid>@btrid
  IF @type = 3 BEGIN
    SET @etrid = NULL
    SELECT @etrid=MIN(trid) FROM #trids WHERE trid>@btrid AND type<>3

    IF @etrid IS NULL
      SET @etrid = @maxtrid+1
    IF @etrid-@btrid > @maxcnt
      SET @etrid = @btrid + @maxcnt
  END

  -- loop and process apm_heartbeat/apm_threshold/maintenance one at a time and 
  -- generic/sentry in set
  WHILE @btrid <= @maxtrid BEGIN
    -- sentry/generic processing
    IF @type = 3 BEGIN
      INSERT #al_generic SELECT e.trId, e.keyword, e.value FROM AL_StageEvData e
        JOIN #trids t ON (e.trId=t.trid)
        WHERE t.trid>=@btrid AND t.trid<@etrid

      -- Set state to processing 
      UPDATE AL_Stage SET trState=4, trTimeProcessed=CURRENT_TIMESTAMP WHERE trId IN 
         (SELECT trid FROM #trids WHERE trid>=@btrid AND trid<@etrid)

      EXEC @rc = al_processGenericMsgs  

      INSERT AL_Status SELECT * FROM #al_log
      UPDATE s SET trState=CASE WHEN @rc < 0 THEN 6 ELSE 5 END, trTimeProcessed=CURRENT_TIMESTAMP 
        FROM AL_Stage s
        JOIN #trids t ON (t.trid=s.trId)
        WHERE t.trid>=@btrid AND t.trid<@etrid

      -- Log in GEM_APM*_Msgs tables when option is turned on
      DECLARE @log_ingem NVARCHAR(8), @eid INT
      SELECT @log_ingem=_Value FROM GEM_Options WHERE _Option='log in gem_apm*_msgs tables'
      IF @log_ingem = 'yes' BEGIN
        SET @eid = @etrid - 1
        EXEC al_logInGemTables @btrid, @eid
      END
    
      TRUNCATE TABLE #al_log
      TRUNCATE TABLE #al_generic
    END
    ELSE BEGIN
      -- heartbeat/threshold/maintenance processing
      DECLARE @sql NVARCHAR(2048)
      SELECT @sql=value FROM AL_StageProcData WHERE trId=@btrid
      SET @err = 0

      IF @type = 1 
        SET @sql = 'gemsp_processAPMHeartBeatMsg ' + @sql
      ELSE IF @type = 2 
        SET @sql = 'gemsp_processAPMThresholdMsg ' + @sql
      ELSE IF @type = 5 
        SET @sql = 'gemsp_processMaintWindowEvent ' + @sql
      ELSE IF @type = 6 
        SET @sql = 'asisp_SLAViolation ' + @sql
      ELSE IF @type = 7 
        SET @sql = 'asisp_SLATrend ' + @sql
      ELSE IF @type = 8 
        SET @sql = 'asisp_SLATrendCancel ' + @sql
     
      IF @type IN (1, 2, 5, 6, 7, 8)
      BEGIN

        -- Set state to processing 
        UPDATE AL_Stage SET trState=4, trTimeProcessed=CURRENT_TIMESTAMP WHERE trId=@btrid

        PRINT @sql 
        EXEC(@sql)
        SET @err = @@ERROR

        UPDATE AL_Stage SET trState=CASE WHEN @err=0 THEN 5 ELSE 6 END, trTimeProcessed=CURRENT_TIMESTAMP
          WHERE trId=@btrid
      END
    END

    -- Increment the loop
    SELECT @btrid = @etrid
    SELECT @type=type FROM #trids WHERE trid=@btrid
    SELECT @etrid=MIN(trid) FROM #trids WHERE trid>@btrid
    IF @type = 3 BEGIN
      SET @etrid = NULL
      SELECT @etrid=MIN(trid) FROM #trids WHERE trid>@btrid AND type<>3
  
      IF @etrid IS NULL
        SET @etrid = @maxtrid+1
      IF @etrid-@btrid > @maxcnt
        SET @etrid = @btrid + @maxcnt
    END
  END

GO
