-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobbuildfunctiongrouptemplate.sql
-- SID: 1.8
-- Date Modified: 04/10/15 11:33:18
--
-- Module Description
--   Builds the alobFunctionGroupTemplate table initializing the key value pairs
--   based on the existing alobList-FUNCTIONNAME- tables
-- 

USE Object
GO

-------------------------------------------------------------------------
-- 
PRINT 'Creating procedure alobBuildFunctionGroupTemplate'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobBuildFunctionGroupTemplate' AND type = 'P')
  DROP PROCEDURE alobBuildFunctionGroupTemplate
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobbuildfunctiongrouptemplate.sql
-- SID: 1.8
-- Date Modified: 04/10/15 11:33:18
CREATE PROCEDURE alobBuildFunctionGroupTemplate
  @verbose    INT=1
AS

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  DECLARE @now NVARCHAR(40)

  

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: alobBuildFunctionGroupTemplate', 0, 1, @now) WITH NOWAIT
  END

  -- hardcoded approach, there has to be a dynamic way to do this ECS

  DELETE FROM alobFunctionGroupTemplate
  -- set all SetThreshold key-value pairs
  -- eventType
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetThresholds1:ThresholdObjects' + CAST(s.rCorr AS NVARCHAR), 'EventType', s.EventType
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetThresholds AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetThresholds' AND s.EventType IS NOT NULL

  -- alertState
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetThresholds1:ThresholdObjects' + CAST(s.rCorr AS NVARCHAR), 'AlertState', s.AlertState
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetThresholds AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetThresholds' AND s.AlertState IS NOT NULL

  -- priority
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetThresholds1:ThresholdObjects' + CAST(s.rCorr AS NVARCHAR), 'Priority', s.Priority
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetThresholds AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetThresholds' AND s.Priority IS NOT NULL

  -- value
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetThresholds1:ThresholdObjects' + CAST(s.rCorr AS NVARCHAR), 'Value', s.Value
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetThresholds AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetThresholds' AND s.Value IS NOT NULL

----------------------------------------------------------------------------------
  -- set all DefineAndUsePBTs key-value pairs
  -- PBTName
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'PBTName', s.PBTName
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.PBTName IS NOT NULL

  -- Salience
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'Salience', s.Salience
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.Salience IS NOT NULL

  -- Enabled
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'Enabled', s.Enabled
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.Enabled IS NOT NULL

  -- MinAlertState
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'MinAlertState', s.MinAlertState
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.MinAlertState IS NOT NULL

  -- MaxAlertState
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'MaxAlertState', s.MaxAlertState
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.MaxAlertState IS NOT NULL

  -- MinPercentage
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'MinPercentage', s.MinPercentage
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.MinPercentage IS NOT NULL

  -- MaxPercentage
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'MaxPercentage', s.MaxPercentage
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.MaxPercentage IS NOT NULL

  -- EventType
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventType', s.EventType
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventType IS NOT NULL

  -- EventName
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventName', s.EventName
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventName IS NOT NULL

  -- EventText
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventText', s.EventText
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventText IS NOT NULL

  -- EventAlertState
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventAlertState', s.EventAlertState
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventAlertState IS NOT NULL

  -- EventPriority
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventPriority', s.EventPriority
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventPriority IS NOT NULL

  -- EventFlags
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventFlags', s.EventFlags
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventFlags IS NOT NULL

  -- EventState
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineAndUsePBTs1:PBTsToDefine' + CAST(s.rCorr AS NVARCHAR), 'EventState', s.EventState
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineAndUsePBTs AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineAndUsePBTs' AND s.EventState IS NOT NULL


----------------------------------------------------------------------------------
  -- set all DefineServicesAndLinkToEvUsers key-value pairs
  -- DisplayName
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:ServiceDef' + CAST(s.rCorr AS NVARCHAR), 'DisplayName', s.DisplayName
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND s.DisplayName IS NOT NULL

  -- ServiceID
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:ServiceDef' + CAST(s.rCorr AS NVARCHAR), 'ServiceID', s.ServiceID
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND s.ServiceID IS NOT NULL

  -- BusinessRole
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:ServiceDef' + CAST(s.rCorr AS NVARCHAR), 'BusinessRole', s.BusinessRole
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND s.BusinessRole IS NOT NULL

  -- RedImpact
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:ServiceDef' + CAST(s.rCorr AS NVARCHAR), 'RedImpact', s.RedImpact
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND s.RedImpact IS NOT NULL

  -- YellowImpact
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:ServiceDef' + CAST(s.rCorr AS NVARCHAR), 'YellowImpact', s.YellowImpact
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND s.YellowImpact IS NOT NULL

  -- Flags
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:ServiceDef' + CAST(s.rCorr AS NVARCHAR), 'Flags', s.Flags
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND s.Flags IS NOT NULL

  -- UserIDs
  --select * from alobListUserIDGroup
  --select * from alobListDefineServicesAndLinkToEvUsers
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:DefineServicesAndLinkToEvUsers1:UserIDDef' + CAST(u.iIndex AS NVARCHAR), 'UserID', u.UserID
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListDefineServicesAndLinkToEvUsers AS s ON f.sInstance = s.sInstance
  JOIN        alobListUserIDGroup AS u ON s.UserIDGroup = u.sInstance
  WHERE       f.sFunctionName = 'DefineServicesAndLinkToEvUsers' AND  u.UserID IS NOT NULL

----------------------------------------------------------------------------------

  -- set all SetWeights key-value pairs
  -- NewWeight
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetWeights' + CAST(s.rCorr AS NVARCHAR), 'NewWeight', s.NewWeight
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetWeights AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetWeights' AND s.NewWeight IS NOT NULL

----------------------------------------------------------------------------------

  -- set all SetSendStateChangeEvent key-value pairs
  -- StateChange
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetSendStateChangeEvent' + CAST(s.rCorr AS NVARCHAR), 'StateChange', s.StateChange
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetSendStateChangeEvent AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetSendStateChangeEvent' AND s.StateChange IS NOT NULL

----------------------------------------------------------------------------------
  -- set all SetBSPriority key-value pairs
  -- NewPriority
  INSERT INTO alobFunctionGroupTemplate ( sFunctionGroup, root, keyword, value )
  SELECT                                  f.sFunctionGroup, 'method:SetBSPriority' + CAST(s.rCorr AS NVARCHAR), 'NewPriority', s.NewPriority
  FROM        alobListFunctionGroup AS f 
  JOIN        alobListSetBSPriority AS s ON f.sInstance = s.sInstance
  WHERE       f.sFunctionName = 'SetBSPriority' AND s.NewPriority IS NOT NULL


  
  IF @tcount = 0
    COMMIT TRANSACTION alobBuildFunctionGroupTemplate


GO

