-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp10.sql
-- SID: 1.3.1.6
-- Date Modified: 04/10/15 11:33:19
--
-- Module Description
--   Defines the alobLoadConfigBcp10 stored procedure which reads in the Version 1.2
--   CriteriaToFunctionGroup section.

Use Object
GO

-------------------------------------------------------------------------
-- alobLoadConfigBcp10
PRINT 'Creating procedure alobLoadConfigBcp10'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadConfigBcp10' AND type = 'P')
  DROP PROCEDURE alobLoadConfigBcp10
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp10.sql
-- SID: 1.3.1.6
-- Date Modified: 04/10/15 11:33:19
CREATE PROCEDURE alobLoadConfigBcp10
  @sFilename      NVARCHAR(256),
  @sCodepage      NVARCHAR(16),
  @verbose        TINYINT = 0
AS

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobLoadConfigBcp10
  ELSE
    SAVE TRAN alobLoadConfigBcp10

  -- Standard Announcement 
  RAISERROR('alobLoadConfigBcp10 @sFilename=''%s'', @sCodepage=''%s''', 0, 1, @sFilename, @sCodepage )

  -- Declare variables
  DECLARE @nError INTEGER
  DECLARE @sFunctionGroup  NVARCHAR(32)
  DECLARE @sFunctionName   NVARCHAR(128)
  DECLARE @iPattern        INTEGER
  DECLARE @sClass          NVARCHAR(4)
  DECLARE @iCriteria       INTEGER


  -- Bcp in the file since BULK INSERT is problematic
  EXEC @nError = alobLoadConfigFile @sFilename, 'alobFileCriteriaToFunctionGroup', @sCodepage, @verbose = @verbose

  IF @nError <> 0
  BEGIN
    RAISERROR('GTMAB1002E Unable to load configuration file %s', 3, 1, @sFilename)
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @sFilename

  -- Integrity check

  -- Check that all the columns that should have data do have data.
  -- iCriteria
  IF EXISTS ( SELECT * FROM alobFileCriteriaToFunctionGroup WHERE iCriteria IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'Criteria', 'CriteriaToFunctionGroup' )
    SELECT * FROM alobFileCriteriaToFunctionGroup WHERE iCriteria IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp10
    RETURN -30000
  END

  -- iPattern
  IF EXISTS ( SELECT * FROM alobFileCriteriaToFunctionGroup WHERE iPattern IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'Pattern', 'CriteriaToFunctionGroup' )
    SELECT * FROM alobFileCriteriaToFunctionGroup WHERE iPattern IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp10
    RETURN -30000
  END

  -- sFunctionGroup
  IF EXISTS ( SELECT * FROM alobFileCriteriaToFunctionGroup WHERE sFunctionGroup IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'FunctionGroup', 'CriteriaToFunctionGroup' )
    SELECT * FROM alobFileCriteriaToFunctionGroup WHERE sFunctionGroup IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp10
    RETURN -30000
  END

  -- Check that specified FunctionGroups exist
  SET ROWCOUNT 1
  SELECT   @sFunctionGroup = d.sFunctionGroup
  FROM alobListFunctionGroup AS fg
  RIGHT OUTER JOIN alobFileCriteriaToFunctionGroup AS d ON fg.sFunctionGroup = d.sFunctionGroup
  WHERE
           fg.sFunctionGroup IS NULL 
 
  SET ROWCOUNT 0
  IF @sFunctionGroup IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1030E FunctionGroup %s in the %s section does not exist in the FunctionGroup section.', 3, 1, @sFunctionGroup, 'CriteriaToFunctionGroup' )
    ROLLBACK TRANSACTION alobLoadConfigBcp10
    RETURN -30000
  END 


  -- populate alobListSetThresholds table

  DELETE FROM alobListCriteriaToFunctionGroup

  INSERT INTO alobListCriteriaToFunctionGroup (iCriteria, iPattern, sFunctionGroup)
  SELECT DISTINCT                              iCriteria, iPattern, sFunctionGroup
  FROM alobFileCriteriaToFunctionGroup

  -- Check for invalid methods on non-LOB classes
  SET ROWCOUNT 1
  SELECT @sClass = fp.iClass, @iCriteria = ctp.iCriteria, @iPattern = ctp.iPattern, @sFunctionName = fg.sFunctionName,
         @sFunctionGroup = fg.sFunctionGroup
  FROM alobListFunctionGroup AS fg
  JOIN alobListCriteriaToFunctionGroup AS cfg ON cfg.sFunctionGroup = fg.sFunctionGroup 
  JOIN alobFileCriteriaToPattern AS ctp ON ctp.iCriteria = cfg.iCriteria 
                                       AND ctp.iPattern  = cfg.iPattern
  JOIN alobFilePattern AS fp  ON           fp.iPattern   = ctp.iPattern
  WHERE fp.iClass <> 'LOB'
    AND fg.sFunctionName IN ('DefineAndUsePBTs', 'DefineServiesAndLinkToEvUsers', 'SetWeights', 'SetBSPriority')
  SET ROWCOUNT 0
  IF @sClass IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1038E FunctionName %s is not allowed for FunctionGroup %s because it is specified for Criteria %d, Pattern %d, and Class %s.  It is only valid for class ''LOB''', 3, 1, @sFunctionName, @sFunctionGroup,@iCriteria, @iPattern, @sClass )
    ROLLBACK TRANSACTION alobLoadConfigBcp10
    RETURN -30000
  END 


  -- AutoCopyEVService is not allowed in any Path FunctionGroups
  SET ROWCOUNT 1
  SELECT   @sFunctionGroup = c.sFunctionGroup
  FROM  alobListCriteriaToFunctionGroup AS c 
  JOIN alobListFunctionGroup AS fg ON fg.sFunctionName = 'AutoCopyEVService'
                                  AND fg.sFunctionGroup = c.sFunctionGroup
 
  SET ROWCOUNT 0
  IF @sFunctionGroup IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1045E The AutoCopyEVService FunctionName in FunctionGroup %s in the %s section is only valid for a FunctionGroup in the CriteriaToPath section.', 3, 1, @sFunctionGroup, 'CriteriaToFunctionGroup' )
    ROLLBACK TRANSACTION alobLoadConfigBcp8
    RETURN -30000
  END 


  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobLoadConfigBcp10
GO
