-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp11.sql
-- SID: 1.2.1.5
-- Date Modified: 04/10/15 11:33:20
--
-- Module Description
--   Defines the alobLoadConfigBcp11 stored procedure which reads in the FunctionGroup section

Use Object
GO

-------------------------------------------------------------------------
-- alobLoadConfigBcp11
PRINT 'Creating procedure alobLoadConfigBcp11'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadConfigBcp11' AND type = 'P')
  DROP PROCEDURE alobLoadConfigBcp11
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp11.sql
-- SID: 1.2.1.5
-- Date Modified: 04/10/15 11:33:20
CREATE PROCEDURE alobLoadConfigBcp11
  @sFilename      NVARCHAR(256),
  @sCodepage      NVARCHAR(16),
  @verbose        TINYINT = 0
AS

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobLoadConfigBcp11
  ELSE
    SAVE TRAN alobLoadConfigBcp11

  -- Standard Announcement 
  RAISERROR('alobLoadConfigBcp11 @sFilename=''%s'', @sCodepage=''%s''', 0, 1, @sFilename, @sCodepage )

  -- Declare variables
  DECLARE @nError INTEGER
  DECLARE @sInstance NVARCHAR(32)

  -- Bcp in the file since BULK INSERT is problematic
  EXEC @nError = alobLoadConfigFile @sFilename, 'alobFileFunctionGroup', @sCodepage, @verbose = @verbose

  IF @nError <> 0
  BEGIN
    RAISERROR('GTMAB1102E Unable to load configuration file %s', 3, 1, @sFilename)
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @sFilename

  -- Integrity check

  -- Check that all the columns that should have data do have data.

  -- sFunctionGroup
  IF EXISTS ( SELECT * FROM alobFileFunctionGroup WHERE sFunctionGroup IS NULL )
  BEGIN
    RAISERROR('GTMAB1121E There is a NULL value in the %s column in the %s section.', 3, 1, 'FunctionGroup', 'FunctionGroup' )
    SELECT * FROM alobFileFunctionGroup WHERE sFunctionGroup IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END

  -- sFunctionName
  IF EXISTS ( SELECT * FROM alobFileFunctionGroup WHERE sFunctionName IS NULL )
  BEGIN
    RAISERROR('GTMAB1121E There is a NULL value in the %s column in the %s section.', 3, 1, 'FunctionName', 'FunctionGroup' )
    SELECT * FROM alobFileFunctionGroup WHERE sFunctionName IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END

  -- sInstance
  IF EXISTS ( SELECT * FROM alobFileFunctionGroup WHERE sInstance IS NULL )
  BEGIN
    RAISERROR('GTMAB1121E There is a NULL value in the %s column in the %s section.', 3, 1, 'Instance', 'FunctionGroup' )
    SELECT * FROM alobFileFunctionGroup WHERE sInstance IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END


  -- Check that all SetThresholds instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListSetThresholds)
        AND sFunctionName = 'SetThresholds'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'SetThresholds' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Check that all SetWeights instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListSetWeights)
        AND sFunctionName = 'SetWeights'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'SetWeights' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Check that all SetSendStateChangeEvent instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListSetSendStateChangeEvent)
        AND sFunctionName = 'SetSendStateChangeEvent'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'SetSendStateChangeEvent' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Check that all DefineAndUsePBTs instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListDefineAndUsePBTs)
        AND sFunctionName = 'DefineAndUsePBTs'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'DefineAndUsePBTs' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Check that all DefineServicesAndLinkToEvUsers instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListDefineServicesAndLinkToEvUsers)
        AND sFunctionName = 'DefineServicesAndLinkToEvUsers'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'DefineServicesAndLinkToEvUsers' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Check that all AutoCopyEVService instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListAutoCopyEVService)
        AND sFunctionName = 'AutoCopyEVService'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'AutoCopyEVService' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Check that all SetBSPriority instances exist
  SET ROWCOUNT 1
  SELECT   @sInstance = sInstance
  FROM alobFileFunctionGroup 
  WHERE sInstance NOT IN (SELECT sInstance from alobListSetBSPriority)
        AND sFunctionName = 'SetBSPriority'

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1031E Instance %s in section FunctionGroup for FunctionName %s does not exist.', 3, 1, @sInstance, 'SetBSPriority' )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 

  -- Make sure FunctionNames are valid
  SET ROWCOUNT 1
  SELECT   @sInstance = sFunctionName
  FROM alobFileFunctionGroup 
  WHERE sFunctionName NOT IN ('SetThresholds','SetSendStateChangeEvent',
                          'SetWeights', 'DefineAndUsePBTs', 'DefineServicesAndLinkToEvUsers',
                          'AutoCopyEVService','SetBSPriority')

  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1035E Invalid FunctionName %s specified in section FunctionGroup', 3, 1, @sInstance )
    ROLLBACK TRANSACTION alobLoadConfigBcp11
    RETURN -30000
  END 




  DELETE FROM alobListFunctionGroup
  -- populate list table
  INSERT INTO alobListFunctionGroup ( sFunctionGroup, sFunctionName, sInstance)
  SELECT DISTINCT                     sFunctionGroup, sFunctionName, sInstance
  FROM alobFileFunctionGroup
  ORDER BY sFunctionGroup,sFunctionName,sInstance



  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobLoadConfigBcp11
GO
