-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp20.sql
-- SID: 1.1
-- Date Modified: 04/10/15 11:29:45
--
-- Module Description
--   Defines the alobLoadConfigBcp20 stored procedure which reads in the SetBSPriority section

Use Object
GO

-------------------------------------------------------------------------
-- alobLoadConfigBcp20
PRINT 'Creating procedure alobLoadConfigBcp20'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadConfigBcp20' AND type = 'P')
  DROP PROCEDURE alobLoadConfigBcp20
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp20.sql
-- SID: 1.1
-- Date Modified: 04/10/15 11:29:45
CREATE PROCEDURE alobLoadConfigBcp20
  @sFilename      NVARCHAR(256),
  @sCodepage      NVARCHAR(16),
  @verbose        TINYINT = 0
AS

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobLoadConfigBcp20
  ELSE
    SAVE TRAN alobLoadConfigBcp20

  -- Standard Announcement 
  RAISERROR('alobLoadConfigBcp20 @sFilename=''%s'', @sCodepage=''%s''', 0, 1, @sFilename, @sCodepage )

  -- Declare variables
  DECLARE @nError INTEGER
  DECLARE @errorMessage NVARCHAR(255)
  DECLARE @sInstance    NVARCHAR(32)


  -- Bcp in the file since BULK INSERT is problematic
  EXEC @nError = alobLoadConfigFile @sFilename, 'alobFileSetBSPriority', @sCodepage, @verbose = @verbose

  IF @nError <> 0
  BEGIN
    RAISERROR('GTMAB1002E Unable to load configuration file %s', 3, 1, @sFilename)
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @sFilename

  -- Integrity check

  -- Check that all the columns that should have data do have data.

  -- sInstance
  IF EXISTS ( SELECT * FROM alobFileSetBSPriority WHERE sInstance IS NULL )
  BEGIN
    RAISERROR('GTMAB1321E There is a NULL value in the %s column in the %s section.', 3, 1, 'Instance', 'SetBSPriority' )
    SELECT * FROM alobFileSetBSPriority WHERE sInstance IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp20
    RETURN -30000
  END

  -- NewPriority
  IF EXISTS ( SELECT * FROM alobFileSetBSPriority WHERE NewPriority IS NULL )
  BEGIN
    RAISERROR('GTMAB1321E There is a NULL value in the %s column in the %s section.', 3, 1, 'NewPriority', 'SetBSPriority' )
    SELECT * FROM alobFileSetBSPriority WHERE sInstance IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp20
    RETURN -30000
  END


  -- populate alobListSetBSPriority table

  DELETE FROM alobListSetBSPriority

  SELECT * from alobFileSetBSPriority
  INSERT INTO alobListSetBSPriority (sInstance, NewPriority)
  SELECT DISTINCT                 sInstance, NewPriority
  FROM alobFileSetBSPriority
  ORDER BY sInstance

  -- Check for duplicate instances
  SET ROWCOUNT 1
  SELECT @sInstance = sInstance
  FROM alobListSetBSPriority 
  WHERE rCorr > (SELECT MIN(rCorr) FROM alobListSetBSPriority AS s 
                 WHERE s.sInstance = alobListSetBSPriority.sInstance) 
  SET ROWCOUNT 0
  IF @sInstance IS NOT NULL 
  BEGIN
    RAISERROR('GTMAB1034E Instance %s must be unique in section %s.' , 3, 1, @sInstance, 'SetBSPriority' )
    ROLLBACK TRANSACTION alobLoadConfigBcp13
    RETURN -30000
  END 


  -- set up for call to asisp_loadSetBSPriority
 CREATE TABLE #BSPriorityParms (
    rCorr       INT          NULL,
    newValue    NVARCHAR(32) NULL,
    error       INT          NULL
  )


  CREATE TABLE #Instances (
    cid        ClassID        NULL,
    id         ObjID          NULL,
    rCorr      INT            NOT NULL,             
    className  NVARCHAR(24)   NULL,
    prodVer    NVARCHAR(85)   NULL,     
    instid     NVARCHAR(425)  NULL,
    uinstid    NVARCHAR(425)  NULL,
    objPath    NVARCHAR(1375) NULL,
    maxRes     INT            NULL,
    errorId    INT            NULL
  ) 

  INSERT INTO #BSPriorityParms ( rCorr, newValue)
  SELECT                     rCorr, NewPriority
  FROM alobListSetBSPriority

  SELECT * from alobListSetBSPriority

  SELECT * from #BSPriorityParms
  -- pass an empty #Instances table to _SetBSPriority
  EXEC @nError = asisp_loadSetBSPriority   @resolveCidId=0, @ExecMode = 5, @outformat = 3, @verbose = @verbose

  SELECT * from #BSPriorityParms

  UPDATE alobListSetBSPriority
  SET ErrorID = p.error
  FROM #BSPriorityParms AS p WHERE alobListSetBSPriority.rCorr = p.rCorr

  IF (EXISTS (SELECT * FROM alobListSetBSPriority WHERE ErrorID IS NOT NULL AND ErrorID <> 0))
  BEGIN
    SET ROWCOUNT 1
    SELECT @errorMessage = m.description, @sInstance = t.sInstance
    FROM master..sysmessages AS m
    JOIN alobListSetBSPriority AS t ON t.ErrorID = m.error
    SET ROWCOUNT 0
    RAISERROR('GTMAB1032E %s For Instance %s in Section %s', 3, 1, @errorMessage, @sInstance, 'SetBSPriority')
    ROLLBACK TRANSACTION alobLoadConfigBcp13
    RETURN -30000
  END


  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobLoadConfigBcp20
GO
