-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/asisp_cleanlinks.sql
-- SID: 1.4.1.8
-- Date Modified: 04/10/20 13:32:25

USE Object
GO
---------------------------------------------------------------------------------------------------
-- asisp_cleanlinks
PRINT 'Creating procedure asisp_cleanlinks'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_cleanlinks' AND type = 'P')
  DROP PROCEDURE asisp_cleanlinks
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\asisp_cleanlinks.sql
-- SID: 1.4.1.8
-- Date Modified: 04/10/20 13:32:25
-- Procedure to perform garbage collection on deleted links
CREATE PROCEDURE asisp_cleanlinks
  @limit  INT=1000
AS
  SET NOCOUNT ON
  DECLARE @sql NVARCHAR(1000), @table sysname, @rc INT, @steprc INT, @table_a sysname
  DECLARE @src_cid ClassID, @dst_cid ClassID, @link_type LinkType, @len INT
  DECLARE @time_str NVARCHAR(32)
  DECLARE @cid ClassID, @total_rc INT
  SET @total_rc = 0

  -- Delete all directly contained objects marked as deleted
  -- OA09083 - We now exclude all event based tables, as deleting from these tables could potentially
  -- delete active events. Since these are handled by seperate cleanup jobs, no need to worry about
  -- them here.
  DECLARE obj_class_c INSENSITIVE CURSOR FOR
    SELECT oc.cid, oc.cname+'_C' FROM obj_class AS oc
      JOIN sysobjects AS so ON so.name = oc.cname+'_C'
    WHERE
          oc.flag_abstract = 0
      AND oc.cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO')
      AND oc.cid NOT IN ('ROOT', 'GMFA', 'GMFF', 'EXCP', 'EXCH', 'MESG', 'CHEV', 'CHVH', 'EEXC', 'EMSG')
  OPEN obj_class_c
  FETCH NEXT FROM obj_class_c INTO @cid, @table
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF EXISTS (SELECT * FROM sysobjects WHERE name = @table + '_BASE')
    BEGIN
      RAISERROR('Cannot delete from %s yet due to BASE/LOCAL division', 0, 1, @table) WITH NOWAIT
    END
    ELSE
    BEGIN
      -- OA09083 - Special case for BatchCycle because of foreign key constraints
      IF @table = 'BatchCycle_C'
      BEGIN
        DELETE FROM opc_operation WHERE btcy_id IN (SELECT id FROM BatchCycle_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): opc_operation', 0, 1, @time_str, @rc) WITH NOWAIT

        DELETE li FROM OPCLoad..opc_app_instance AS li 
          JOIN Object..opc_app_instance AS oi ON oi.plan_name = li.plan_name AND oi.app_name = li.app_name
        WHERE 
          oi.btcy_id IN (SELECT id FROM Object..BatchCycle_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): OPCLoad..opc_app_instance', 0, 1, @time_str, @rc) WITH NOWAIT

        DELETE FROM opc_app_instance WHERE btcy_id IN (SELECT id FROM BatchCycle_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): opc_app_instance', 0, 1, @time_str, @rc) WITH NOWAIT
      END

      -- OA09083 - Special case for BATCH because of foreign key constraints
      IF @table = 'BATCH_C'
      BEGIN
        DELETE FROM opc_operation WHERE job_id IN (SELECT id FROM BATCH_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): opc_operation', 0, 1, @time_str, @rc) WITH NOWAIT

        DELETE FROM BATCH_Zeke_A WHERE id IN (SELECT id FROM BATCH_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): BATCH_Zeke_A', 0, 1, @time_str, @rc) WITH NOWAIT
      END

      -- OA09302 - Special case for NetView% classes because of _A table foreign key constraints
      IF @table LIKE 'NetView%'
      BEGIN
        SET @len = LEN(@table)
        SET @table_a = REPLACE(@table,SUBSTRING(@table, @len, 1),'A')
        SET @sql = 'DELETE a FROM ' + @table_a + ' AS a WHERE id IN (SELECT id FROM ' + @table + ' WHERE deleted = 1)'
        EXEC(@sql)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): %s', 0, 1, @time_str, @rc, @table_a) WITH NOWAIT
      END

      -- OA09302 - Special case for COMPLEX because of foreign key constraint
      IF @table = 'COMPLEX_C'
      BEGIN
        DELETE FROM COMP_BATC WHERE comp_id IN (SELECT id FROM COMPLEX_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): COMP_BATC', 0, 1, @time_str, @rc) WITH NOWAIT
      END

      -- OA09302 - Special case for GenericTrapCategory because of foreign key constraint
      IF @table = 'GenericTrapCategory_C'
      BEGIN
        DELETE FROM GenericTrapDescription_C WHERE _ParentID IN (SELECT id FROM GenericTrapCategory_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): GenericTrapDescription_C', 0, 1, @time_str, @rc) WITH NOWAIT
      END

      -- OA09302 - Special case for Note because of foreign key constraint
      IF @table = 'Note_C'
      BEGIN
        DELETE FROM NoteHistory WHERE noteid IN (SELECT id FROM Note_C WHERE deleted = 1)
        SET @rc = @@ROWCOUNT
        SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
        RAISERROR('%s: %d row(s): NoteHistory', 0, 1, @time_str, @rc) WITH NOWAIT
      END

      SET @sql = 'DELETE o FROM ' + @table + ' AS o WHERE o.deleted = 1'
      SET ROWCOUNT @limit
      SET @rc = 1
      SET @steprc = 0
      WHILE @rc > 0
      BEGIN 
        EXEC(@sql)
        SET @rc = @@ROWCOUNT
        SET @total_rc = @total_rc + @rc  
        SET @steprc = @steprc + @rc
      END
      SET ROWCOUNT 0
      SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
      RAISERROR('%s: %d row(s): %s', 0, 1, @time_str, @steprc, @sql) WITH NOWAIT
    END
    FETCH NEXT FROM obj_class_c INTO @cid, @table
  END
  DEALLOCATE obj_class_c

  -- DELETE resources from LinkState tables to avoid conflict with Foreign Key constraint in the next step
  SET ROWCOUNT @limit
  SET @rc = 1
  SET @steprc = 0
  WHILE @rc > 0
  BEGIN 
    DELETE FROM ls FROM LinkState_A AS ls
      JOIN link AS l ON l.link_type = ls._link_type
                    AND l.src_cid   = ls._src_cid
                    AND l.src_id    = ls._src_id
                    AND l.dst_cid   = ls._dst_cid
                    AND l.dst_id    = ls._dst_id
    WHERE 
          l.deleted = 1
      AND (ls._src_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO')
           OR ls._dst_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO'))
      AND (ls._src_cid NOT IN ('GMFA', 'GMFF') OR ls._dst_cid NOT IN ('GMFA', 'GMFF'))
    SET @rc = @@ROWCOUNT
    SET @steprc = @steprc + @rc
    SET @total_rc = @total_rc + @rc
  END
  SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
  RAISERROR('%s: %d row(s) deleted from LinkState_A table.', 0, 1, @time_str, @steprc) WITH NOWAIT

  SET @rc = 1
  SET @steprc = 0
  WHILE @rc > 0
  BEGIN  
    DELETE FROM lsc FROM LinkState_C AS lsc
      WHERE lsc.id NOT IN (SELECT lsa.id FROM LinkState_A AS lsa)
    SET @rc = @@ROWCOUNT
    SET @steprc = @steprc + @rc
    SET @total_rc = @total_rc + @rc
  END
  SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
  RAISERROR('%s: %d row(s) deleted from LinkState_C table.', 0, 1, @time_str, @steprc) WITH NOWAIT

  -- OA09083 - Delete from opc_app_link table to avoid conflict with Foreign Key constraint in the next step
  SET @rc = 1
  SET @steprc = 0
  WHILE @rc > 0
  BEGIN  
    DELETE o FROM opc_app_link AS o
      JOIN link AS l ON l.link_type = o.link_type 
                    AND l.src_cid   = o.src_cid 
                    AND l.src_id    = o.src_id 
                    AND l.dst_cid   = o.btcy_cid 
                    AND l.dst_id    = o.btcy_id
    WHERE
      l.deleted = 1
    SET @rc = @@ROWCOUNT
    SET @steprc = @steprc + @rc
    SET @total_rc = @total_rc + @rc
  END
  SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
  RAISERROR('%s: %d row(s) deleted from opc_app_link table.', 0, 1, @time_str, @steprc) WITH NOWAIT
  SET ROWCOUNT 0

  -- Delete from the individual link tables
  DECLARE link_c INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, dst_cid, link_type FROM link WHERE deleted=1
  OPEN link_c
  FETCH NEXT FROM link_c INTO @src_cid, @dst_cid, @link_type
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @table = RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type 
    IF EXISTS (SELECT * FROM sysobjects WHERE name = @table AND type = 'U')
    BEGIN 
      SET @sql = 'DELETE FROM i FROM ' + @table + ' AS i JOIN link AS l ON ' +
                 'l.link_type=''' + @link_type + ''' ' +
                 'AND l.src_cid=''' + RTRIM(@src_cid) + ''' AND i.src_id=l.src_id ' +
                 'AND l.dst_cid=''' + RTRIM(@dst_cid) + ''' AND i.dst_id=l.dst_id ' +
                 'WHERE l.deleted=1 ' +
                 'AND (l.src_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = ''RDMO'') ' +
                 '  OR l.dst_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = ''RDMO'')) ' +
                 'AND (l.src_cid NOT IN (''GMFA'', ''GMFF'') OR l.dst_cid NOT IN (''GMFA'', ''GMFF'')) '

      SET ROWCOUNT @limit
      SET @rc = 1
      SET @steprc = 0
      WHILE @rc > 0
      BEGIN 
        EXEC(@sql)
        SET @rc = @@ROWCOUNT
        SET @total_rc = @total_rc + @rc
        SET @steprc = @steprc + @rc
      END
      SET ROWCOUNT 0
      SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
      RAISERROR('%s: %d row(s): %s', 0, 1, @time_str, @steprc, @sql) WITH NOWAIT
    END
    FETCH NEXT FROM link_c INTO @src_cid, @dst_cid, @link_type
  END
  DEALLOCATE link_c

  -- Delete the marked link records
  SET ROWCOUNT @limit
  SET @rc = 1
  SET @steprc = 0
  WHILE @rc > 0
  BEGIN 
    DELETE FROM l FROM link AS l WHERE l.deleted=1
      AND (l.src_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO')
           OR l.dst_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO'))
      AND (l.src_cid NOT IN ('GMFA', 'GMFF') OR l.dst_cid NOT IN ('GMFA', 'GMFF'))
    SET @rc = @@ROWCOUNT
    SET @total_rc = @total_rc + @rc
    SET @steprc = @steprc + @rc
  END
  SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
  RAISERROR('%s: %d rows: %s', 0, 1, @time_str, @steprc, 'DELETE FROM link WHERE deleted=1') WITH NOWAIT

  -- Delete the marked containment records
  SET @rc = 1
  SET @steprc = 0
  WHILE @rc > 0
  BEGIN 
    DELETE FROM c FROM containment AS c WHERE c.deleted=1
      AND (c.src_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO')
           OR c.dst_cid NOT IN (SELECT ias.cid FROM isa_chain AS ias WHERE base_cid = 'RDMO'))
      AND (c.src_cid NOT IN ('GMFA', 'GMFF') OR c.dst_cid NOT IN ('GMFA', 'GMFF'))
    SET @rc = @@ROWCOUNT
    SET @total_rc = @total_rc + @rc
    SET @steprc = @steprc + @rc
  END
  SET @time_str = CONVERT(NVARCHAR(32), GETDATE(), 14)
  RAISERROR('%s: %d rows: %s', 0, 1, @time_str, @steprc, 'DELETE FROM containment WHERE deleted=1') WITH NOWAIT
  SET ROWCOUNT 0

  RETURN @total_rc
