-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_exportlobasxml.sql
-- SID: 1.49
-- Date Modified: 04/10/18 14:17:19
--
-- Module Description
--   Defines the asisp_exportLOBAsXML stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_exportLOBAsXML
PRINT 'Creating procedure asisp_exportLOBAsXML'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_exportLOBAsXML' AND type = 'P')
  DROP PROCEDURE asisp_exportLOBAsXML
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_exportlobasxml.sql
-- SID: 1.49
-- Date Modified: 04/10/18 14:17:19
-- Processes the business tree, writing it in XML to a file
CREATE PROCEDURE asisp_exportLOBAsXML
  @cid      ClassID = NULL,        -- cid of the object (LOBC or LOB)
  @id       ObjID = NULL,          -- id of the object
  @path     NVARCHAR(255),         -- the path and name of the output file
  @name     NVARCHAR(108) = NULL,  -- LOB name
  @verbose  TINYINT = 0            -- indicates the amount of diagnostic information output by the procedure
AS

  SET NOCOUNT ON

  DECLARE @obj_path NVARCHAR(2000), @obj_upath NVARCHAR(400), @src_cid ClassID, @src_id ObjID
  DECLARE @rc INT, @now NVARCHAR(40)

  -- If resource not specified, default to the root business system
  IF @cid IS NULL AND @id IS NULL AND @name IS NULL
  BEGIN
    SET @cid = 'LOBC'
    SET @id = 1
  END
  ELSE
  BEGIN
    IF @name IS NOT NULL 
    BEGIN
      DECLARE @count  INT
      SELECT @count=COUNT(*) FROM LineOfBusiness_V WHERE name = @name AND deleted = 0
      --
      -- If more than one line of business exists for this name,
      -- display messages showing the existing LOBs with this name.
      IF @count <> 1
      BEGIN
        IF @count = 0
        BEGIN
          RAISERROR(65000, 4, 1, @name)
          RETURN -30000
        END
        ELSE
        BEGIN
          RAISERROR(65005, 4, 1, @count, @name)
          
          CREATE TABLE #dupnames (
            cid    ClassID,
            id     ObjID,
            instid NVARCHAR(425),
            ident  INT IDENTITY(1,1)
          )
          
          INSERT INTO #dupnames (cid, id, instid)
          SELECT cl.cid, cl.oid, cl.value
          FROM CL_IDCache AS cl
            JOIN LineOfBusiness_V AS lob ON lob.cid = cl.cid AND lob.id = cl.oid
          WHERE cl.keyword = 'CL_InstID' AND lob.name = @name AND lob.deleted = 0
          
          DECLARE @re_cid    ClassID
          DECLARE @re_id     ObjID
          DECLARE @re_instid NVARCHAR(425)
           
          DECLARE c CURSOR FOR 
            SELECT  cid, id, instid FROM #dupnames

          OPEN c
          FETCH NEXT FROM c INTO @re_cid, @re_id, @re_instid
          WHILE @@FETCH_STATUS <> -1
          BEGIN 
            RAISERROR(65007, 4, 1, @re_cid, @re_id, @re_instid)
            FETCH NEXT FROM c INTO @re_cid, @re_id, @re_instid
          END
          CLOSE c
          DEALLOCATE c 
          
          RETURN -30000
        END
      END
      ELSE
      BEGIN
        SELECT @cid=cid, @id=id FROM LineOfBusiness_V WHERE name = @name AND deleted = 0
      END
    END
    ELSE
    BEGIN
      --
      -- If only a cid or an id was provided or an invalid 
      -- value for these then error
      IF (@cid = 'LOBC' AND @id <> 1) OR (@cid = 'LOB' AND @id < 1) OR (@cid <> 'LOB' AND @cid <> 'LOBC')
      BEGIN
        RAISERROR(65001, 4, 1)
        RETURN -30000
      END
      ELSE
      BEGIN
        --
        -- If the cid and id specified do not exist, then error
        IF @cid = 'LOB'
        BEGIN
          IF NOT EXISTS (SELECT * FROM LineOfBusiness_V WHERE id = @id AND deleted = 0)
          BEGIN
            RAISERROR(65000, 4, 1)
            RETURN -30000
          END  
        END
      END
    END  
  END


  IF @verbose = 7
  BEGIN
    DECLARE @start DATETIME
    DECLARE @start1 DATETIME
    DECLARE @start2 DATETIME
    SET @start = CURRENT_TIMESTAMP
    SET @start1 = CURRENT_TIMESTAMP
    DECLARE @elapsed_ms INT
    DECLARE @sqlCounter1 INT
    DECLARE @sqlCounter2 INT
    DECLARE @sqlCounter2001 INT
    DECLARE @sqlCounter2002 INT
    DECLARE @sqlCounter2003 INT
    DECLARE @sqlCounter3 INT
    DECLARE @sqlCounter4 INT
    DECLARE @sqlCounter5 INT
    DECLARE @sqlCounter6 INT
    DECLARE @sqlCounter7 INT
    DECLARE @sqlCounter7001 INT
    DECLARE @sqlCounter8 INT
    DECLARE @sqlCounter9 INT
    DECLARE @sqlCounter10 INT
    DECLARE @sqlCounter11 INT
    DECLARE @sqlCounter12 INT
    DECLARE @sqlCounter13 INT
    DECLARE @sqlCounter14 INT
    DECLARE @sqlCounter15 INT
    DECLARE @sqlCounter16 INT
    DECLARE @sqlCounter17 INT
    DECLARE @sqlCounter18 INT
    DECLARE @sqlCounter19 INT
    
    SET @sqlCounter1 = 0
    SET @sqlCounter2 = 0
    SET @sqlCounter2001 = 0
    SET @sqlCounter2002 = 0
    SET @sqlCounter2003 = 0
    SET @sqlCounter3 = 0
    SET @sqlCounter4 = 0
    SET @sqlCounter5 = 0
    SET @sqlCounter6 = 0
    SET @sqlCounter7 = 0
    SET @sqlCounter7001 = 0
    SET @sqlCounter8 = 0
    SET @sqlCounter9 = 0
    SET @sqlCounter10 = 0
    SET @sqlCounter11 = 0
    SET @sqlCounter12 = 0
    SET @sqlCounter13 = 0
    SET @sqlCounter14 = 0
    SET @sqlCounter15 = 0
    SET @sqlCounter16 = 0
    SET @sqlCounter17 = 0
    SET @sqlCounter18 = 0
    SET @sqlCounter19 = 0
  END
  
  DECLARE @countlob       INT
  DECLARE @countlink      INT
  DECLARE @countlobc      INT
  DECLARE @countlobl      INT
  DECLARE @countgenl      INT
  DECLARE @countmethod    INT
  DECLARE @countpbt       INT
  DECLARE @countthreshold INT
  DECLARE @countservice   INT
  DECLARE @countsendevent INT
  DECLARE @countweight    INT
  DECLARE @countobjpri    INT
  
  -- temp tables
  CREATE TABLE #lob
  (
    cid         ClassID,
    id          ObjID,
    name        NVARCHAR(100)  NULL,
    path        NVARCHAR(2000) NULL,
    upath       NVARCHAR(400)  NULL,
    gen         INT,
    description NVARCHAR(255)  NULL,
    delCid      ClassID        NULL,
    delId       ObjID          NULL,
    delName     NVARCHAR(54)   NULL,
    flags       INT            NOT NULL DEFAULT 0,
    src_cid     ClassID        NULL,
    src_id      ObjID          NULL,
    done        TINYINT        NOT NULL DEFAULT 0,
    instID      NVARCHAR(425)  NULL,
    instrID     NVARCHAR(255)  NULL,
    ident       INT IDENTITY(1,1),
    lnkx_ident  INT NOT NULL DEFAULT 0
  )
  CREATE TABLE #lobparent
  (
    cid         ClassID,
    id          ObjID,
    path        NVARCHAR(2000) NULL,
    upath       NVARCHAR(400)  NULL,
    ident       INT IDENTITY(1,1)
  )
  CREATE TABLE #phys
  (
    cid     ClassID,
    id      ObjID,
    name    NVARCHAR(100)  NULL,
    path    NVARCHAR(2000) NULL,
    instID  NVARCHAR(425)  NULL,
    instrID NVARCHAR(255)  NULL,
    flags   INT NOT NULL DEFAULT 0,
    ident   INT IDENTITY(1,1)
  )
  CREATE TABLE #links
  (
    src_cid    ClassID,
    src_id     ObjID,
    dst_cid    ClassID,
    dst_id     ObjID,
    link_type  NVARCHAR(4),
    id         INT IDENTITY(1,1),
    is_lnkx    TINYINT NOT NULL DEFAULT 0
  )
  CREATE TABLE #lnkx
  (
    cid         ClassID,
    id          ObjID,
    name        ObjName NULL,
    description NVARCHAR(255) NULL,
    src_cid     ClassID,
    src_id      ObjID,
    dst_cid     ClassID,
    dst_id      ObjID,
    link_type   ClassID,
    direction   INT,
    ident       INT IDENTITY(1,1),
    link_ident  INT NOT NULL DEFAULT 0
  )
  CREATE TABLE #thresholds
  (
    cid   ClassID,
    id    ObjID,
    MI       INT NOT NULL,
    AI       INT NOT NULL,
    Critical INT NULL,
    High     INT NULL,
    Medium   INT NULL,
    Low      INT NULL,
    Ignore   INT NULL
  )
  CREATE TABLE #pbts
  (
    pbt_id INT,
    ident  INT IDENTITY(1,1)
  )
  CREATE TABLE #pbt_usedby
  (
    pbt_id INT,
    lob_id INT,
    ident  INT IDENTITY(1,1)
  )
  CREATE TABLE #services
  (
    cid     ClassID,
    id      ObjID,
    name    ObjName,
    DN      ObjName,
    SID     NVARCHAR(63),
    BR      NVARCHAR(500),
    RI      NVARCHAR(500),
    YI      NVARCHAR(500),
    F       INT,
    servid  ObjID,
    ident   INT IDENTITY(1,1)
  )
  CREATE TABLE #method_calls
  (
    mname  sysname,
    cid    ClassID,
    id     ObjID,
    param1 NVARCHAR(100) NULL,
    param2 NVARCHAR(100) NULL,
    param3 NVARCHAR(500) NULL,
    param4 NVARCHAR(500) NULL,
    param5 NVARCHAR(500) NULL,
    param6 NVARCHAR(100) NULL,
    param7 NVARCHAR(100) NULL,
    param8 NVARCHAR(100) NULL,
    param9 NVARCHAR(100) NULL,
    param10 NVARCHAR(100) NULL,
    param11 NVARCHAR(255) NULL,
    param12 NVARCHAR(255) NULL,
    param13 NVARCHAR(100) NULL,
    param14 NVARCHAR(100) NULL,
    param15 NVARCHAR(100) NULL,
    param16 NVARCHAR(255) NULL,
    ident    INT IDENTITY(1,1),
    md_ident INT NOT NULL DEFAULT 0
  )
  CREATE TABLE #method_distinct
  (
    mname  sysname,
    param1 NVARCHAR(100) NULL,
    param2 NVARCHAR(100) NULL,
    param3 NVARCHAR(500) NULL,
    param4 NVARCHAR(500) NULL,
    param5 NVARCHAR(500) NULL,
    param6 NVARCHAR(100) NULL,
    param7 NVARCHAR(100) NULL,
    param8 NVARCHAR(100) NULL,
    param9 NVARCHAR(100) NULL,
    param10 NVARCHAR(100) NULL,
    param11 NVARCHAR(255) NULL,
    param12 NVARCHAR(255) NULL,
    param13 NVARCHAR(100) NULL,
    param14 NVARCHAR(100) NULL,
    param15 NVARCHAR(100) NULL,
    param16 NVARCHAR(255) NULL,
    ident  INT IDENTITY(1,1)
  )
  CREATE TABLE #obj
  (
    cid   ClassID,
    id    ObjID,
    tmpd  INT NOT NULL DEFAULT 0,
    ident INT IDENTITY(1,1)
  )
  CREATE TABLE #class
  (
    cid   ClassID       NOT NULL,
    btag  NVARCHAR(16)  NOT NULL,
    cdata NVARCHAR(128) NOT NULL,
    etag  NVARCHAR(16)  NOT NULL
  )
  CREATE TABLE #lobs_fail (
    cid   ClassID        NOT NULL,
    id    ObjID          NOT NULL,
    name  NVARCHAR(100)  NOT NULL,
    path  NVARCHAR(2000) NULL
  )
  CREATE TABLE #crls (
    id       ObjID         NOT NULL,
    userid   NVARCHAR(255) NULL,
    username NVARCHAR(255) NULL
  )
  
  -- Get all class data upfront
  INSERT INTO #class (cid, btag, cdata, etag)
  SELECT cid, '<class>', cname, '</class>'
  FROM obj_class 
  WHERE cid NOT IN (SELECT cid FROM GEMLookupCID)
  INSERT INTO #class (cid, btag, cdata, etag)
  SELECT cid, '<prodver>', prodname + ';' + version, '</prodver>'
  FROM GEMLookupCID

  -- Populate the path and name variables
  IF @cid = 'LOB'
  BEGIN
    INSERT INTO #lob (cid, id, name, path, upath, gen, description, delCid, delId, delName, flags)
    SELECT @cid, id, name, name, @cid + ':' + CAST(id AS NVARCHAR(9)), 0, _Desc, _DeletedCID, _DeletedID, _DeletedName, _flags
    FROM LineOfBusiness_V
    WHERE id = @id
      AND deleted = 0
    SET @rc = @@ROWCOUNT
    -- Check to ensure that the LOB exists
    IF @rc = 0
    BEGIN
      RAISERROR(65000, 4, 1)
      -- SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      -- RAISERROR('%s: asisp_exportLOBAsXML: ERROR: LOB %d does not exist or has been deleted.', 11, 1, @now, @id) WITH NOWAIT
      RETURN -30000
    END

    --
    -- Get the path info for this LOB
    --
    -- Start with the resources name
    SELECT @obj_path = name, @obj_upath = cid + ':' + CAST(id AS NVARCHAR(9)), @src_cid = cid, @src_id = id
    FROM #lob
    WHERE cid = @cid AND id = @id
    -- Get the source object
    SELECT @src_cid = c.src_cid, @src_id = c.src_id
    FROM containment AS c
    WHERE c.deleted = 0
      AND c.dst_cid = @src_cid AND c.dst_id = @src_id
    -- loop
    WHILE @rc > 0 AND @src_cid IS NOT NULL
    BEGIN
      IF @src_cid = 'LOB'
      BEGIN
        SELECT @obj_path = l.name + '/' + @obj_path, @obj_upath = 'LOB:' + CAST(l.id AS NVARCHAR(9)) + '/' + @obj_upath,
          @src_cid = c.src_cid, @src_id = c.src_id
        FROM LineOfBusiness_V AS l
          JOIN containment AS c ON c.dst_cid = 'LOB' AND c.dst_id = l.id
        WHERE c.deleted = 0 AND l.deleted = 0
          AND c.dst_cid = @src_cid AND c.dst_id = @src_id
        SET @rc = @@ROWCOUNT
      END -- IF @src_cid = 'LOB'
      ELSE
      BEGIN
        SET @obj_path = 'LOBC/' + @obj_path
        SET @obj_upath = 'LOBC:1/' + @obj_upath
        SET @rc = 0
      END
    END -- WHILE @rc > 0

    -- Check to make sure we were able to go all the way up the hierarchy
    IF @src_cid <> 'LOBC'
    BEGIN
      RAISERROR(65002, 4, 1, @id)
      -- SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      -- RAISERROR('%s: asisp_exportLOBAsXML: ERROR: Could not get to LOBC from LOB %d.  The LOB is orphaned.', 11, 1, @now, @id) WITH NOWAIT
      RETURN -30000
    END

    -- Update the LOB with the new path info
    UPDATE #lob SET path = @obj_path, upath = @obj_upath
    
  END -- IF @cid = 'LOB'
  ELSE -- LOBC
    INSERT INTO #lob (cid, id, name, path, upath, gen)
    SELECT @cid, 1, 'LOBC', 'LOBC', 'LOBC:1', 0

  
  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
    SET @start1 = CURRENT_TIMESTAMP
  END  
  
  --
  -- Traverse the LOB tree, populating the #lob temp table
  --
  DECLARE @continue TINYINT, @gen INT
  SET @continue =  1
  SET @gen = 1
  WHILE @continue = 1
  BEGIN

    IF @verbose = 7
    BEGIN
      SET @start2 = CURRENT_TIMESTAMP
    END 
    
    -- Get the link information
    INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
    SELECT DISTINCT c.src_cid, c.src_id, c.dst_cid, c.dst_id, 'LOBC'
    FROM #lob AS parent
      JOIN containment AS c ON c.src_cid = parent.cid AND c.src_id = parent.id AND c.dst_cid = 'LOB'
      LEFT OUTER JOIN #links AS e ON e.src_cid = c.src_cid AND e.src_id = c.src_id
        AND e.dst_cid = c.dst_cid AND e.dst_id = c.dst_id AND e.link_type = 'LOBC'
    WHERE e.dst_cid IS NULL
      AND c.deleted = 0

    IF @verbose = 7
    BEGIN
      SET @elapsed_ms = DATEDIFF(MS, @start2, CURRENT_TIMESTAMP)
      SET @sqlCounter2001 = @elapsed_ms + @sqlCounter2001
      SET @start2 = CURRENT_TIMESTAMP
    END 
     
    -- Get children via LOBC links
    INSERT INTO #lob (cid, id, name, path, upath, gen, description, delCid, delId, delName, flags)
    SELECT 'LOB', child.id, child.name, parent.path + '/' + child.name, parent.upath + '/LOB:' + CAST(child.id AS NVARCHAR(9)), @gen, child.description, child._DeletedCID, child._DeletedID, child._DeletedName, child._flags
    FROM #lob AS parent
      JOIN #links AS c ON c.src_cid = parent.cid AND c.src_id = parent.id AND c.dst_cid = 'LOB' AND c.link_type = 'LOBC'
      JOIN LineOfBusiness_V AS child ON child.id = c.dst_id
    WHERE child.deleted = 0
      -- AND c.deleted = 0
      AND parent.gen = @gen - 1
    SET @rc = @@ROWCOUNT

    IF @verbose = 7
    BEGIN
      SET @elapsed_ms = DATEDIFF(MS, @start2, CURRENT_TIMESTAMP)
      SET @sqlCounter2002 = @elapsed_ms + @sqlCounter2002
      SET @start2 = CURRENT_TIMESTAMP
    END 
     
    -- set the end-of-loop condition
    IF @rc = 0
      SET @continue = 0

    IF @verbose = 7
    BEGIN
      SET @elapsed_ms = DATEDIFF(MS, @start2, CURRENT_TIMESTAMP)
      SET @sqlCounter2003 = @elapsed_ms + @sqlCounter2003
      SET @start2 = CURRENT_TIMESTAMP
    END 
     
    SET @gen = @gen + 1
  END -- WHILE @bcontinue = 1

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter2 = @elapsed_ms + @sqlCounter2
    SET @start1 = CURRENT_TIMESTAMP
  END  
    
  --
  -- If a specific LOB is exported, traverse the tree upwards to
  -- get the parents
  --
  IF @cid = 'LOB'
  BEGIN
    DECLARE @topcid ClassID
    DECLARE @topid  ObjID
    
    SET @topcid=@cid
    SET @topid=@id
    
    SET @continue = 1
    --
    -- Add the links moving up the tree
    --
    WHILE @continue > 0 
    BEGIN
      -- Get the link information
      INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
      SELECT DISTINCT c.src_cid, c.src_id, c.dst_cid, c.dst_id, 'LOBC'
      FROM containment AS c
        LEFT OUTER JOIN #links AS e ON e.src_cid = c.src_cid AND e.src_id = c.src_id
          AND e.dst_cid = c.dst_cid AND e.dst_id = c.dst_id AND e.link_type = 'LOBC'
      WHERE e.dst_cid IS NULL
        AND c.deleted = 0
        AND c.dst_cid = @topcid AND c.dst_id = @topid
     
      SELECT DISTINCT @topcid=src_cid, @topid=src_id
      FROM #links 
      WHERE dst_cid = @topcid AND dst_id = @topid
  
      INSERT INTO #lobparent (cid, id) VALUES(@topcid, @topid)
      
      IF @topcid = 'LOBC'
      BEGIN
        SET @continue = 0
      END
    END
    
    --
    -- If we have something other than the root LOBC, then
    -- build the path and upath for each line of business
    IF EXISTS (SELECT * FROM #lobparent WHERE cid = 'LOB')
    BEGIN
      SET @obj_path = 'LOBC'
      SET @obj_upath = 'LOBC:1'
      SELECT @continue=MAX(ident) FROM #lobparent
      
      WHILE @continue > 1
      BEGIN
        SET @continue = @continue - 1
        
        SELECT @obj_path = @obj_path + '/' + l.name, @obj_upath = @obj_upath + '/' + 'LOB:' + CAST(l.id AS NVARCHAR(9))
        FROM LineOfBusiness_V AS l
          JOIN #lobparent AS c ON c.cid = 'LOB' AND c.id = l.id
        WHERE c.ident=@continue  
          
        UPDATE #lobparent SET path=@obj_path, upath=@obj_upath
        WHERE ident=@continue
        
      END
      
      --
      -- If we have something other than the root LOBC, then
      -- add them to the #lob table
      INSERT INTO #lob (cid, id, name, path, upath, gen, description, delCid, delId, delName, flags)
      SELECT 'LOB', child.id, child.name, c.path, c.upath, 0, child.description, child._DeletedCID, child._DeletedID, child._DeletedName, child._flags
      FROM LineOfBusiness_V AS child
        JOIN #lobparent AS c ON c.cid = child.cid AND c.id = child.id
      WHERE child.deleted = 0
   
    END
  END
  
  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter3 = @elapsed_ms + @sqlCounter3
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- eliminate anything that is not an LOB
  DELETE FROM #lob WHERE cid <> 'LOB'

  -- Get the LOBL links
  INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
  SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, 'LOBL'
  FROM #lob AS t
    JOIN link AS l ON l.src_cid = t.cid AND l.src_id = t.id
  WHERE l.deleted = 0
    AND l.link_type = 'LOBL'

  -- Get the GENL links where the LOB is the source
  INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
  SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, 'GENL'
  FROM #lob AS t
    JOIN link AS l ON l.src_cid = t.cid AND l.src_id = t.id
  WHERE l.deleted = 0
    AND l.link_type = 'GENL'

  -- Get the GENL links where the LOB is the destination
  INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
  SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, 'GENL'
  FROM #lob AS t
    JOIN link AS l ON l.dst_cid = t.cid AND l.dst_id = t.id
  WHERE l.deleted = 0
    AND l.link_type = 'GENL' AND l.src_cid <> 'LOB'

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter4 = @elapsed_ms + @sqlCounter4
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  --
  -- Identify GENL links represented by LNKX objects
  --

  -- forward direction
  INSERT INTO #lnkx (cid, id, name, description, src_cid, src_id, dst_cid, dst_id, link_type, direction)
  SELECT DISTINCT 'LNKX', ls.id, ls.name, ls.description, l.src_cid, l.src_id, l.dst_cid, l.dst_id, l.link_type, ls._direction
  FROM #links AS l
    JOIN LinkState_V AS ls ON ls._src_cid = l.src_cid AND ls._src_id = l.src_id
      AND ls._dst_cid = l.dst_cid AND ls._dst_id = l.dst_id AND ls._link_type = l.link_type
  WHERE l.link_type = 'GENL'
    AND ls._direction IN (0, 1, 3)

  -- reversed direction
  INSERT INTO #lnkx (cid, id, name, description, src_cid, src_id, dst_cid, dst_id, link_type, direction)
  SELECT DISTINCT 'LNKX', ls.id, ls.name, ls.description, l.src_cid, l.src_id, l.dst_cid, l.dst_id, l.link_type, ls._direction
  FROM #links AS l
    JOIN LinkState_V AS ls ON ls._src_cid = l.dst_cid AND ls._src_id = l.dst_id
      AND ls._dst_cid = l.src_cid AND ls._dst_id = l.src_id AND ls._link_type = l.link_type
  WHERE l.link_type = 'GENL'
    AND ls._direction IN (0, 2, 3)

  -- Add the linked physical resources
  INSERT INTO #phys (cid, id)
  SELECT DISTINCT l.dst_cid, l.dst_id
  FROM #links AS l
  WHERE l.link_type IN ('LOBL', 'GENL')

  -- Add the physical resources where the LOB is the destination
  INSERT INTO #phys (cid, id)
  SELECT DISTINCT l.src_cid, l.src_id
  FROM #links AS l
  WHERE l.link_type = 'GENL'
    AND l.src_cid <> 'LOB'

  -- Link the entries in #links with those in #lnkx
  UPDATE ls
  SET ls.link_ident = l.id
  FROM #links AS l
    JOIN #lnkx AS ls ON ls.src_cid = l.src_cid AND ls.src_id = l.src_id
      AND ls.dst_cid = l.dst_cid AND ls.dst_id = l.dst_id AND ls.link_type = l.link_type
  WHERE l.link_type = 'GENL'
    AND ls.direction IN (0, 1, 3)
  UPDATE ls
  SET ls.link_ident = l.id
  FROM #links AS l
    JOIN #lnkx AS ls ON ls.src_cid = l.dst_cid AND ls.src_id = l.dst_id
      AND ls.dst_cid = l.src_cid AND ls.dst_id = l.src_id AND ls.link_type = l.link_type
  WHERE l.link_type = 'GENL'
    AND ls.direction IN (0, 2, 3)

  -- Update the is_lnkx column for the #links table
  UPDATE l
  SET l.is_lnkx = 1
  FROM #links AS l
    JOIN #lnkx AS ls ON ls.link_ident = l.id

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter5 = @elapsed_ms + @sqlCounter5
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Add the LNKX objects to the #lob table
  INSERT INTO #lob (cid, id, name, description, gen, lnkx_ident)
  SELECT lx.cid, lx.id, lx.name, lx.description, 0, lx.ident
  FROM #lnkx AS lx

  --
  -- Get the names of the physical resources
  --
  DECLARE @c_cid ClassID, @c_cname sysname
  DECLARE c_phys INSENSITIVE CURSOR FOR
    SELECT DISTINCT RTRIM(oc.cid), oc.cname
    FROM obj_class AS oc
      JOIN #phys AS p ON p.cid = oc.cid
  OPEN c_phys
  FETCH NEXT FROM c_phys INTO @c_cid, @c_cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- Update the name and path columns
    EXEC('
      UPDATE p
      SET name = o.name
      FROM #phys AS p
        JOIN ' + @c_cname + '_V AS o ON o.id = p.id
      WHERE p.cid = ''' + @c_cid + '''
    ')
    FETCH NEXT FROM c_phys INTO @c_cid, @c_cname
  END -- WHILE @@FETCH_STATUS <> -1
  DEALLOCATE c_phys

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter6 = @elapsed_ms + @sqlCounter6
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Parse the instID from CL cache to make sure (CID:ID) is at end before stripping out
  CREATE TABLE #instid_str (
    cid     ClassID        NOT NULL,
    id      ObjID          NOT NULL, 
    value   NVARCHAR(425)  NOT NULL,
    instrid NVARCHAR(255)  NOT NULL,
    unq_cid ClassID        NULL,
    unq_id  ObjID          NULL,
    unq_val NVARCHAR(425)  NULL
  )
  INSERT INTO #instid_str (cid, id, value, instrid)
    SELECT c.cid, c.oid, c.value, c.instrID FROM #lob AS l
    JOIN CL_IDCache AS c ON c.cid = l.cid AND c.oid = l.id
  WHERE c.keyword = 'CL_InstID'
    AND c.deleted = 0

  INSERT INTO #instid_str (cid, id, value, instrid)
    SELECT c.cid, c.oid, c.value, c.instrID FROM CL_IDCache AS c
    JOIN #phys AS p ON c.cid = p.cid AND c.oid = p.id
    LEFT OUTER JOIN #instid_str ci ON ci.cid = p.cid AND ci.id = p.id
  WHERE c.keyword = 'CL_InstID'
    AND c.deleted = 0
    AND ci.cid IS NULL

  -- For LOB CL instance IDs, parse the instance ID and the u-qualifier at the end (if exists)
  UPDATE #instid_str
    SET unq_cid = REVERSE(SUBSTRING(REVERSE(value), CHARINDEX(':', REVERSE(value)) + 1, CHARINDEX('(', REVERSE(value)) - CHARINDEX(':', REVERSE(value)) - 1)),
        unq_id  = REVERSE(SUBSTRING(REVERSE(value), 2, CHARINDEX(':', REVERSE(value)) - 2)),
        unq_val = REVERSE(SUBSTRING(REVERSE(value), CHARINDEX('(', REVERSE(value)) + 1, LEN(value) - CHARINDEX('(', REVERSE(value))))
  WHERE cid = 'LOB' AND REVERSE(value) LIKE ')%[1-9]:%(%'
  UPDATE #instid_str SET unq_val = value 
    WHERE unq_cid IS NULL OR unq_id IS NULL OR ISNUMERIC(unq_id) = 0

  -- Get the instID value from CL_IDCache table
  UPDATE p
  SET instID  = c.unq_val, instrID = c.instrid
  FROM #phys AS p
    JOIN #instid_str AS c ON c.cid = p.cid AND c.id = p.id

  -- Get the instID value from GEM_IDLookup table
  UPDATE p
  SET instID = c.key_value
  FROM #phys AS p
    JOIN GEM_IDLookup AS c ON c.CID = p.cid AND c.IID = p.id
  WHERE c.key_name = 'KeyGen' AND p.instID IS NULL

  -- Set done=1 when LOB cannot be exported
  -- when there are more than one BS folders with same name under same parent
  -- when there are more that one BS resources with same name linked to same resource under same parent
  UPDATE l
  SET done = 1
  FROM #lob AS l
    JOIN #instid_str AS c ON l.cid = c.cid AND l.id = c.id AND l.cid = c.unq_cid AND l.id = c.unq_id
  WHERE ISNUMERIC(c.unq_id) = 1

  -- Set done=2 when linked LOB doesnt need a <u tag ie, <uinstid> or <upath>
  -- we dont need these tags when user explicitly specifies instance id ie, no (CID:ID) at end
  -- putting this info in #lob (done column) and #phys (flags column) tables
  UPDATE l
  SET done = 2
  FROM #lob AS l
    JOIN #instid_str AS c ON l.cid = c.cid AND l.id = c.id
  WHERE c.unq_cid IS NULL OR c.unq_id IS NULL OR ISNUMERIC(c.unq_id) = 0
  UPDATE p
    SET flags = 2
  FROM #phys AS p
    JOIN #instid_str AS c ON p.cid = c.cid AND p.id = c.id
  WHERE c.unq_cid IS NULL OR c.unq_id IS NULL OR ISNUMERIC(c.unq_id) = 0

  -- Get the path info from the ObjPathCache if needed
  -- And set path only if its unique enough to identify the resource
  CREATE TABLE #ocache (
    cid    ClassID        NOT NULL,
    id     ObjID          NOT NULL,
    resp   NVARCHAR(2000) NOT NULL,
    dup    INT            NULL
  )
  CREATE TABLE #dupresp (
    cid    ClassID        NOT NULL,
    resp   NVARCHAR(2000) NOT NULL, 
    dup    INT            NULL
  )
  INSERT INTO #ocache (cid, id, resp)
  SELECT o.cid, o.id, o.resourcepath
  FROM ObjPathCache AS o
    JOIN #phys AS p ON p.cid = o.cid AND p.id = o.id

  INSERT INTO #dupresp (cid, resp, dup)
  SELECT cid, resourcepath, COUNT(*)
  FROM ObjPathCache AS o
  WHERE resourcepath IN 
    (SELECT resp FROM #ocache)
  GROUP BY cid, resourcepath

  UPDATE o
  SET dup = d.dup
  FROM #ocache AS o
    JOIN #dupresp AS d ON o.cid = d.cid AND o.resp = d.resp

  UPDATE p
  SET path = o.resp
  FROM #ocache AS o
    JOIN #phys AS p ON p.cid = o.cid AND p.id = o.id
  WHERE p.instID IS NULL AND o.dup = 1

  -- Get the instID value from the CL_IDCache table for LOBs
  UPDATE l
  SET instID  = c.unq_val, instrID = c.instrid
  FROM #lob AS l
    JOIN #instid_str AS c ON c.cid = l.cid AND c.id = l.id

  -- Cleanup any LOBs that cannot be exported
  INSERT INTO #lobs_fail (cid, id, name, path)
    SELECT cid, id, name, path 
  FROM #lob WHERE done = 1
  INSERT INTO #lobs_fail (cid, id, name, path)
    SELECT l.cid, l.id, l.name, l.path
  FROM #lob AS l
    JOIN #links AS lnk ON lnk.src_cid = l.cid AND lnk.src_id = l.id
    JOIN #phys AS p ON p.cid = lnk.dst_cid AND p.id = lnk.dst_id
  WHERE p.instID IS NULL AND p.path IS NULL AND l.done <> 1 AND l.done <> 2

  DELETE l 
  FROM #lob AS l
    JOIN #lobs_fail lf ON l.cid = lf.cid AND l.id = lf.id
  DELETE p
  FROM #phys AS p
    JOIN #lobs_fail lf ON p.cid = lf.cid AND p.id = lf.id
  DELETE l
  FROM #links AS l
    JOIN #lobs_fail lf ON (l.src_cid = lf.cid AND l.src_id = lf.id) OR
                          (l.dst_cid = lf.cid AND l.dst_id = lf.id)

  -- insert LOBs into the #obj table
  INSERT INTO #obj (cid, id, tmpd)
  SELECT cid, id, done
  FROM #lob

  -- insert physical objects into the #obj table (if not already in #obj)
  INSERT INTO #obj (cid, id, tmpd)
  SELECT DISTINCT p.cid, p.id, p.flags
  FROM #phys p
  LEFT OUTER JOIN #obj o ON o.cid = p.cid AND o.id = o.id
  WHERE o.cid IS NULL

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter7 = @elapsed_ms + @sqlCounter7
    SET @start1 = CURRENT_TIMESTAMP
  END
  
  -- Convert any XML reserved characters to their 
  -- XML entity equivalents
  --
  UPDATE #lob SET path = (SELECT REPLACE(path, '&', '&amp;')), name = (SELECT REPLACE(name, '&', '&amp;')), instID = (SELECT REPLACE(instID, '&', '&amp;')), description = (SELECT REPLACE(description, '&', '&amp;')), delName = (SELECT REPLACE(delName, '&', '&amp;'))  
  FROM #lob 
  UPDATE #lob SET path = (SELECT REPLACE(path, '<', '&lt;')), name = (SELECT REPLACE(name, '<', '&lt;')), instID = (SELECT REPLACE(instID, '<', '&lt;')), description = (SELECT REPLACE(description, '<', '&lt;')), delName = (SELECT REPLACE(delName, '<', '&lt;'))  
  FROM #lob 
  UPDATE #lob SET path = (SELECT REPLACE(path, '>', '&gt;')), name = (SELECT REPLACE(name, '>', '&gt;')), instID = (SELECT REPLACE(instID, '>', '&gt;')), description = (SELECT REPLACE(description, '>', '&gt;')), delName = (SELECT REPLACE(delName, '>', '&gt;'))
  FROM #lob 
  UPDATE #lob SET path = (SELECT REPLACE(path, '"', '&quot;')), name = (SELECT REPLACE(name, '"', '&quot;')), instID = (SELECT REPLACE(instID, '"', '&quot;')), description = (SELECT REPLACE(description, '"', '&quot;')), delName = (SELECT REPLACE(delName, '"', '&quot;'))
  FROM #lob 
  UPDATE #lob SET description = (SELECT REPLACE(description, '''', '&apos;'))
  FROM #lob 
  
  UPDATE #phys SET path = (SELECT REPLACE(path, '&', '&amp;')), name = (SELECT REPLACE(name, '&', '&amp;')), instID = (SELECT REPLACE(instID, '&', '&amp;'))
  FROM #phys
  UPDATE #phys SET path = (SELECT REPLACE(path, '<', '&lt;')), name = (SELECT REPLACE(name, '<', '&lt;')), instID = (SELECT REPLACE(instID, '<', '&lt;'))
  FROM #phys
  UPDATE #phys SET path = (SELECT REPLACE(path, '>', '&gt;')), name = (SELECT REPLACE(name, '>', '&gt;')), instID = (SELECT REPLACE(instID, '>', '&gt;'))
  FROM #phys
  UPDATE #phys SET path = (SELECT REPLACE(path, '"', '&quot;')), name = (SELECT REPLACE(name, '"', '&quot;')), instID = (SELECT REPLACE(instID, '"', '&quot;'))
  FROM #phys
        
  -- Collect LOB and Link Statistics
  SELECT @countlob=COUNT(*) FROM #lob
  SELECT @countlink=COUNT(*) FROM #links
  SELECT @countlobc=COUNT(*) FROM #links
  WHERE link_type='LOBC'
  SELECT @countlobl=COUNT(*) FROM #links
  WHERE link_type='LOBL'
  SELECT @countgenl=COUNT(*) FROM #links
  WHERE link_type='GENL'
  
  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter7001 = @elapsed_ms + @sqlCounter7001
    SET @start1 = CURRENT_TIMESTAMP
  END
  
  -- temp table to hold XML
  CREATE TABLE #output
  (
    inst_pre NVARCHAR(10),
    inst_id  INT,
    sub_id   INT NOT NULL DEFAULT 0,
    line     NVARCHAR(3500),
    id       INT IDENTITY(1,1),
    not_used INT NULL DEFAULT NULL
  )

  --
  -- Populate the #thresholds table
  --

  -- Insert #thresholds values for LOBs
  INSERT INTO #thresholds (cid, id, MI, AI, Critical, High, Medium, Low, Ignore)
  SELECT l.cid, l.id, ps.MatrixIndex, ps.ArrayIndex, ps.Critical, ps.High, ps.Medium, ps.Low, ps.Ignore
  FROM #lob AS l
    JOIN PropagationSlot AS ps ON ps.OwnerCID = l.cid AND ps.OwnerID = l.id
  WHERE ps.SettingIndex = 1

  -- Insert #thresholds values for physical resources
  INSERT INTO #thresholds (cid, id, MI, AI, Critical, High, Medium, Low, Ignore)
  SELECT l.cid, l.id, ps.MatrixIndex, ps.ArrayIndex, ps.Critical, ps.High, ps.Medium, ps.Low, ps.Ignore
  FROM #phys AS l
    JOIN PropagationSlot AS ps ON ps.OwnerCID = l.cid AND ps.OwnerID = l.id
  WHERE ps.SettingIndex = 1

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter8 = @elapsed_ms + @sqlCounter8
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  --
  -- Populate the #services table
  --

  INSERT INTO #services (cid, id, name, DN, SID, BR, RI, YI, F, servid)
  SELECT DISTINCT lob.cid, lob.id, s.name, _DisplayName, _ServiceID, _BusinessRole, _RedImpact, _YellowImpact, _flags & 1, s.id
  FROM #lob AS lob
    JOIN EXES_LOB_EVSL AS l ON lob.cid = 'LOB' AND lob.id = l.dst_id
    JOIN EVService_V AS s ON s.id = l.src_id
  WHERE l.deleted = 0 AND s.deleted = 0

  --  Populate users for the #services
  CREATE TABLE #seruser
  (
    userid       NVARCHAR(255),
    lobid        ObjID
  )
  INSERT INTO #seruser (userid, lobid)
  SELECT _UserID, ser.id
  FROM #services AS ser
    JOIN EXEI_EXES_EVPL AS evul ON evul.dst_id=ser.servid
    JOIN EVUser_C AS evu ON evu.id=evul.src_id
  INSERT INTO #seruser (userid, lobid)
  SELECT _UserID, ser.id
  FROM #services AS ser
    JOIN EXEU_EXES_EVPL AS evul ON evul.dst_id=ser.servid
    JOIN EVUser_C AS evu ON evu.id=evul.src_id


  UPDATE #services SET name = (SELECT REPLACE(name, '&', '&amp;')), DN = (SELECT REPLACE(DN, '&', '&amp;')), SID = (SELECT REPLACE(SID, '&', '&amp;')), BR = (SELECT REPLACE(BR, '&', '&amp;')), RI = (SELECT REPLACE(RI, '&', '&amp;')), YI = (SELECT REPLACE(YI, '&', '&amp;'))
  FROM #services
  UPDATE #services SET name = (SELECT REPLACE(name, '<', '&lt;')), DN = (SELECT REPLACE(DN, '<', '&lt;')), SID = (SELECT REPLACE(SID, '<', '&lt;')), BR = (SELECT REPLACE(BR, '<', '&lt;')), RI = (SELECT REPLACE(RI, '<', '&lt;')), YI = (SELECT REPLACE(YI, '<', '&lt;'))
  FROM #services
  UPDATE #services SET name = (SELECT REPLACE(name, '>', '&gt;')), DN = (SELECT REPLACE(DN, '>', '&gt;')), SID = (SELECT REPLACE(SID, '>', '&gt;')), BR = (SELECT REPLACE(BR, '>', '&gt;')), RI = (SELECT REPLACE(RI, '>', '&gt;')), YI = (SELECT REPLACE(YI, '>', '&gt;')) 
  FROM #services
  UPDATE #services SET name = (SELECT REPLACE(name, '"', '&quot;')),DN = (SELECT REPLACE(DN, '"', '&quot;')), SID = (SELECT REPLACE(SID, '"', '&quot;')), BR = (SELECT REPLACE(BR, '"', '&quot;')), RI = (SELECT REPLACE(RI, '"', '&quot;')), YI = (SELECT REPLACE(YI, '"', '&quot;'))
  FROM #services
  UPDATE #services SET name = (SELECT REPLACE(name, '''', '&apos;')),DN = (SELECT REPLACE(DN, '''', '&apos;')), SID = (SELECT REPLACE(SID, '''', '&apos;')), BR = (SELECT REPLACE(BR, '''', '&apos;')), RI = (SELECT REPLACE(RI, '''', '&apos;')), YI = (SELECT REPLACE(YI, '''', '&apos;'))
  FROM #services

  UPDATE #seruser  SET userid = (SELECT REPLACE(userid, '&', '&amp;'))  
  FROM #seruser
  UPDATE #seruser  SET userid = (SELECT REPLACE(userid, '<', '&lt;'))  
  FROM #seruser
  UPDATE #seruser  SET userid = (SELECT REPLACE(userid, '>', '&gt;'))  
  FROM #seruser
  UPDATE #seruser  SET userid = (SELECT REPLACE(userid, '"', '&quot;'))  
  FROM #seruser
  UPDATE #seruser  SET userid = (SELECT REPLACE(userid, '''', '&apos;'))  
  FROM #seruser

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter9 = @elapsed_ms + @sqlCounter9
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  --
  -- Populate the PBT tables
  --

  INSERT INTO #pbt_usedby (pbt_id, lob_id)
  SELECT p.pbt_id, p.obj_id
  FROM #lob AS l
    JOIN PBTUsedBy AS p ON p.obj_id = l.id AND p.obj_cid = l.cid
  WHERE l.cid = 'LOB'
    AND p.enabled = 1

  INSERT INTO #pbts (pbt_id)
  SELECT p.id
  FROM #pbt_usedby AS pu
    JOIN PercentageBasedThreshold AS p ON p.id = pu.pbt_id

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter10 = @elapsed_ms + @sqlCounter10
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  --
  -- Populate the #method_calls table
  --

  -- Find the flag enumeration
  DECLARE @val INT, @flags_eid INT
  SELECT @flags_eid = eid
  FROM enumeration
  WHERE name = 'MNO2FlagValues'
  IF @flags_eid IS NULL
  BEGIN
    RAISERROR(65003, 4, 1)
    RETURN -30000
  END

  -- Find the enum_value for the flag
  SELECT @val = value
  FROM enum_value
  WHERE eid = @flags_eid
    AND name = 'SendAlertStateChangeEvent'
  IF @val IS NULL
  BEGIN
    RAISERROR(65004, 4, 1)
    RETURN -30000
  END

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter11 = @elapsed_ms + @sqlCounter11
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Populate with state change event flag when set
  INSERT INTO #method_calls (mname, cid, id, param1)
  SELECT 'SetSendStateChangeEvent', p.cid, p.id, '1'
  FROM #lob AS p
  WHERE p.flags & @val <> 0

  -- Populate with weights when set
  INSERT INTO #method_calls (mname, cid, id, param1)
  SELECT 'SetWeights', p.cid, p.id, v._Weight
  FROM #lob AS p
  JOIN LineOfBusiness_V AS v ON p.id = v.id
  WHERE p.cid = 'LOB' AND v._Weight <> 100

  -- Populate with object priority
  INSERT INTO #method_calls (mname, cid, id, param1)
  SELECT 'SetBSPriority', p.cid, p.id, v._PriorityID
  FROM #lob AS p
  JOIN LineOfBusiness_V AS v ON p.id = v.id
  WHERE p.cid = 'LOB'

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter12 = @elapsed_ms + @sqlCounter12
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Populate with threshold calls
  INSERT INTO #method_calls (mname, cid, id, param1, param2, param3, param4)
  SELECT 'SetThresholds', t.cid, t.id, CAST(t.MI AS NVARCHAR(9)), CAST(t.AI AS NVARCHAR(9)), 1, t.Critical
  FROM #thresholds AS t
  WHERE t.Critical IS NOT NULL
  INSERT INTO #method_calls (mname, cid, id, param1, param2, param3, param4)
  SELECT 'SetThresholds', t.cid, t.id, CAST(t.MI AS NVARCHAR(9)), CAST(t.AI AS NVARCHAR(9)), 2, t.High
  FROM #thresholds AS t
  WHERE t.High IS NOT NULL
  INSERT INTO #method_calls (mname, cid, id, param1, param2, param3, param4)
  SELECT 'SetThresholds', t.cid, t.id, CAST(t.MI AS NVARCHAR(9)), CAST(t.AI AS NVARCHAR(9)), 3, t.Medium
  FROM #thresholds AS t
  WHERE t.Medium IS NOT NULL
  INSERT INTO #method_calls (mname, cid, id, param1, param2, param3, param4)
  SELECT 'SetThresholds', t.cid, t.id, CAST(t.MI AS NVARCHAR(9)), CAST(t.AI AS NVARCHAR(9)), 4, t.Low
  FROM #thresholds AS t
  WHERE t.Low IS NOT NULL
  INSERT INTO #method_calls (mname, cid, id, param1, param2, param3, param4)
  SELECT 'SetThresholds', t.cid, t.id, CAST(t.MI AS NVARCHAR(9)), CAST(t.AI AS NVARCHAR(9)), 5, t.Ignore
  FROM #thresholds AS t
  WHERE t.Ignore IS NOT NULL

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter13 = @elapsed_ms + @sqlCounter13
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Populate with PBT creation calls
  -- The @cid and @id values are meaningless for this method call
  INSERT INTO #method_calls (mname, cid, id,
    param1, param2, param3, param4, param5, param6, param7, param8, param9,
    param10, param11, param12, param13, param14, param15, param16)
  SELECT 'DefinePBTs', 'LOB', pt.lob_id,
    NULL, NULL, pbt.name, pbt.salience, pbt.enabled, pbt.MinAlertState, pbt.MaxAlertState, pbt.MinPercentage, pbt.MaxPercentage,
    pbt.EventType, pbt.EventName, pbt.EventText, pbt.EventAlertState, pbt.EventPriority, pbt.EventFlags, pbt.EventState
  FROM #pbt_usedby AS pt
    JOIN PercentageBasedThreshold AS pbt ON pbt.id = pt.pbt_id

  UPDATE #method_calls SET param3 = (SELECT REPLACE(param3, '&', '&amp;')), param11 = (SELECT REPLACE(param11, '&', '&amp;')), param12 = (SELECT REPLACE(param12, '&', '&amp;')), param16 = (SELECT REPLACE(param16, '&', '&amp;'))
  FROM #method_calls
  WHERE mname = 'DefinePBTs'
  UPDATE #method_calls SET param3 = (SELECT REPLACE(param3, '<', '&lt;')), param11 = (SELECT REPLACE(param11, '<', '&lt;')), param12 = (SELECT REPLACE(param12, '<', '&lt;')), param16 = (SELECT REPLACE(param16, '<', '&lt;'))
  FROM #method_calls
  WHERE mname = 'DefinePBTs'
  UPDATE #method_calls SET param3 = (SELECT REPLACE(param3, '>', '&gt;')), param11 = (SELECT REPLACE(param11, '>', '&gt;')), param12 = (SELECT REPLACE(param12, '>', '&gt;')), param16 = (SELECT REPLACE(param16, '>', '&gt;'))
  FROM #method_calls
  WHERE mname = 'DefinePBTs'
  UPDATE #method_calls SET param3 = (SELECT REPLACE(param3, '"', '&quot;')), param11 = (SELECT REPLACE(param11, '"', '&quot;')), param12 = (SELECT REPLACE(param12, '"', '&quot;')), param16 = (SELECT REPLACE(param16, '"', '&quot;'))
  FROM #method_calls
  WHERE mname = 'DefinePBTs'
  UPDATE #method_calls SET param3 = (SELECT REPLACE(param3, '''', '&apos;')), param11 = (SELECT REPLACE(param11, '''', '&apos;')), param12 = (SELECT REPLACE(param12, '''', '&apos;')), param16 = (SELECT REPLACE(param16, '''', '&apos;'))
  FROM #method_calls
  WHERE mname = 'DefinePBTs'
  
  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter14 = @elapsed_ms + @sqlCounter14
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Populate with service configurations
  INSERT INTO #method_calls (mname, cid, id, param1, param2, param3, param4, param5, param6)
  SELECT 'DefineEVService', s.cid, s.id, s.DN, s.SID, s.BR, s.RI, s.YI, s.F
  FROM #services AS s

  -- Determine the distinct set of method calls
  INSERT INTO #method_distinct (mname, param1, param2, param3, param4, param5, param6, param7, param8, param9,
    param10, param11, param12, param13, param14, param15, param16)
  SELECT DISTINCT mname, param1, param2, param3, param4, param5, param6, param7, param8, param9,
    param10, param11, param12, param13, param14, param15, param16
  FROM #method_calls

  -- Update the md_ident column
  UPDATE mc
  SET md_ident = md.ident
  FROM #method_calls AS mc
    JOIN #method_distinct AS md ON md.mname = mc.mname AND ISNULL(mc.param1, '*NULL*') = ISNULL(md.param1, '*NULL*') AND ISNULL(mc.param2, '*NULL*') = ISNULL(md.param2, '*NULL*')
      AND ISNULL(mc.param3, '*NULL*') = ISNULL(md.param3, '*NULL*') AND ISNULL(mc.param4, '*NULL*') = ISNULL(md.param4, '*NULL*')
      AND ISNULL(mc.param5, '*NULL*') = ISNULL(md.param5, '*NULL*') AND ISNULL(mc.param6, '*NULL*') = ISNULL(md.param6, '*NULL*')
      AND ISNULL(mc.param7, '*NULL*') = ISNULL(md.param7, '*NULL*') AND ISNULL(mc.param8, '*NULL*') = ISNULL(md.param8, '*NULL*')
      AND ISNULL(mc.param9, '*NULL*') = ISNULL(md.param9, '*NULL*')

  IF @verbose = 6
  BEGIN
    SELECT '#lob', * FROM #lob
    SELECT '#links', * FROM #links
    SELECT '#phys', * FROM #phys
    SELECT '#lnkx', * FROM #lnkx
  END
  
  -- Collect Method Statistics
  SELECT @countmethod=COUNT(*) FROM #method_calls
  SELECT @countsendevent=COUNT(*) FROM #method_calls
  WHERE mname='SetSendStateChangeEvent'
  SELECT @countweight=COUNT(*) FROM #method_calls
  WHERE mname='SetWeights'
  SELECT @countthreshold=COUNT(*) FROM #method_calls
  WHERE mname='SetThresholds'
  SELECT @countpbt=COUNT(*) FROM #method_calls
  WHERE mname='DefinePBTs'
  SELECT @countservice=COUNT(*) FROM #method_calls
  WHERE mname='DefineEVService'
  SELECT @countobjpri=COUNT(*) FROM #method_calls
  WHERE mname='SetBSPriority'
  
  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter15 = @elapsed_ms + @sqlCounter15
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  --
  -- Create the XML
  --
  
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<tbsm:addObjectInstances>'
  FROM #lob

  -- Create the opening <inst> line for each LOB
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<inst>'
  FROM #lob

  -- Create the instance tags
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LOB', l.ident, c.btag + c.cdata + c.etag
  FROM #lob AS l
    JOIN #class AS c ON c.cid = l.cid

  -- Create the instid tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<instid>' + ISNULL(CASE WHEN instID IS NULL THEN path ELSE instID END, '') + '</instid>'
  FROM #lob

  -- insert the physical object's uinstid / upath opening tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LOB', lob.ident,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' +
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #lob AS lob 
    JOIN #links AS l ON lob.cid = l.src_cid AND lob.id = l.src_id
    -- The source of the link maps to the dst object from the link table
    JOIN #phys AS p ON p.cid = l.dst_cid AND p.id = l.dst_id
  WHERE l.link_type = 'LOBL' AND lob.done <> 2

  -- Create the instrid tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<instrid>' + ISNULL(instrID, '') + '</instrid>'
  FROM #lob
  WHERE instrID IS NOT NULL AND instID IS NOT NULL

  -- Create the name tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<name>' + ISNULL(name, '') + '</name>'
  FROM #lob

  -- Create the CRL userid and username tags
  INSERT INTO #crls (id, userid, username)
  SELECT l.id, up.userid, up.display_name 
  FROM #lob AS l
    JOIN UserProfiles AS up ON l.id = up.crl_id

  -- Convert any XML reserved characters to their XML entity equivalents
  --
  UPDATE #crls SET userid = REPLACE(userid, '&', '&amp;'), username = REPLACE(username, '&', '&amp;')
  UPDATE #crls SET userid = REPLACE(userid, '<', '&lt;'), username = REPLACE(username, '<', '&lt;')
  UPDATE #crls SET userid = REPLACE(userid, '>', '&gt;'), username = REPLACE(username, '>', '&gt;')
  UPDATE #crls SET userid = REPLACE(userid, '"', '&quot;'), username = REPLACE(username, '"', '&quot;')
  UPDATE #crls SET userid = REPLACE(userid, '''', '&apos;')

  -- <crl_userid> tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<crl_userid>' + ISNULL(c.userid, '') + '</crl_userid>'
  FROM #lob AS l
    JOIN #crls AS c ON l.id = c.id

  -- <crl_username> tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<crl_username>' + ISNULL(c.username, '') + '</crl_username>'
  FROM #lob AS l
    JOIN #crls AS c ON l.id = c.id


  -- Create the opening <attr> tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<attr>'
  FROM #lob

  -- Create attributes in here.  Should be in a loop over attributes. For now, do description
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<desc>' + description + '</desc>'
  FROM #lob
  WHERE description IS NOT NULL

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<DeletedCID>' + delCid + '</DeletedCID>'
  FROM #lob l
  WHERE delCid IS NOT NULL AND delCid <> '' AND 
        delCid NOT IN (SELECT cid FROM GEMLookupCID)

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<DeletedCID>' + g.prodname + ';' + g.version + '</DeletedCID>'
  FROM #lob AS l
  JOIN GEMLookupCID AS g ON g.cid = l.delCid
  WHERE delCid IS NOT NULL AND delCid <> ''

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '<DeletedName>' + delName + '</DeletedName>'
  FROM #lob
  WHERE delName IS NOT NULL AND delName <> ''

  -- For RODM objects, need to export DeletedID too as they can
  -- come back alive after being a ghostbuster
  -- For RODM objects (any cid derived from RDMO superclass), _RODMClass and _RODMMyName is unique
  CREATE TABLE #rodm (
    cid         ClassID       NOT NULL,
    id          ObjID         NOT NULL,
    rodm_class  NVARCHAR(60)  NULL,
    rodm_myname NVARCHAR(255) NULL
  )
  INSERT INTO #rodm (cid, id, rodm_class, rodm_myname)
    SELECT l.delCid, l.delId, rodm._RODMClass, rodm._RODMMyName
    FROM #lob l
    JOIN isa_chain isa ON l.delCid = isa.cid
    JOIN RODMManagedObject rodm ON rodm.cid = l.delCid AND rodm.id = l.delId
    WHERE isa.base_cid = 'RDMO'
 
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<DeletedIDRODMClass>' + r.rodm_class + '</DeletedIDRODMClass>'
  FROM #lob l
  JOIN #rodm r ON l.delCid = r.cid AND l.delId = r.id
  WHERE l.delId > 0
  
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<DeletedIDRODMMyName>' + r.rodm_myname + '</DeletedIDRODMMyName>'
  FROM #lob l
  JOIN #rodm r ON l.delCid = r.cid AND l.delId = r.id
  WHERE l.delId > 0


  --
  -- Do any LNKX attributes here
  --

  -- direction attr
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<direction>' + CAST(lx.direction AS NVARCHAR(9)) + '</direction>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident

  -- src attr opening tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<src><id>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident

  -- src class tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, c.btag + c.cdata + c.etag
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident
    JOIN #class AS c ON c.cid = lx.src_cid

  -- Create the src instid tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<instid>' + ISNULL(src.instID, ISNULL(src.path, '')) + '</instid>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident
    JOIN #lob AS src ON src.cid = lx.src_cid AND src.id = lx.src_id

  -- Create the src tags for physical resources
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LOB', lob.ident,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #lob AS lob 
    JOIN #lnkx AS lx ON lx.ident = lob.lnkx_ident
    JOIN #phys AS p ON p.cid = lx.src_cid AND p.id = lx.src_id
  WHERE lob.done <> 2

  -- Create the src instrid tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<instrid>' + ISNULL(src.instrID, '') + '</instrid>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident
    JOIN #lob AS src ON src.cid = lx.src_cid AND src.id = lx.src_id

  -- src attr closing tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '</id></src>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident

  -- dst attr opening tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<dst><id>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident

  -- dst class tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, c.btag + c.cdata + c.etag
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident
    JOIN #class AS c ON c.cid = lx.dst_cid

  -- Create the dst instid tag for LOBs
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<instid>' + ISNULL(dst.instID, ISNULL(dst.path, '')) + '</instid>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident
    JOIN #lob AS dst ON dst.cid = lx.dst_cid AND dst.id = lx.dst_id

  -- Create the dst tags for physical resources
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LOB', lob.ident,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #lob AS lob 
    JOIN #lnkx AS lx ON lx.ident = lob.lnkx_ident
    JOIN #phys AS p ON p.cid = lx.dst_cid AND p.id = lx.dst_id
  WHERE lob.done <> 2

  -- Create the dst instrid tag for LOBs
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '<instrid>' + ISNULL(dst.instrID, '') + '</instrid>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident
    JOIN #lob AS dst ON dst.cid = lx.dst_cid AND dst.id = lx.dst_id

  -- dst attr closing tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', l.ident, '</id></dst>'
  FROM #lob AS l
    JOIN #lnkx AS lx ON lx.ident = l.lnkx_ident

  -- End LNKX section

  -- Create the closing <attr> tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '</attr>'
  FROM #lob

  -- Create the closing <inst> tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '</inst>'
  FROM #lob

  -- create the closing tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, '</tbsm:addObjectInstances>'
  FROM #lob

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter16 = @elapsed_ms + @sqlCounter16
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Insert blank line for formatting
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LOB', ident, ''
  FROM #lob

  --
  -- Insert LINK instances
  --

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<tbsm:addLinkInstances>'
  FROM #links

  -- Create the opening tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<link>'
  FROM #links
  WHERE is_lnkx = 0

  -- Create the <type> tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<type>' + link_type + '</type>'
  FROM #links
  WHERE is_lnkx = 0

  -- Create the target data
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<tgt>'
  FROM #links
  WHERE is_lnkx = 0
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<id>'
  FROM #links
  WHERE is_lnkx = 0
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', l.id, c.btag + c.cdata + c.etag
  FROM #links AS l
    -- The target of the link maps to the src object from the link table
    JOIN #class AS c ON c.cid = l.src_cid
  WHERE is_lnkx = 0

  -- add the instid tag for LOBs
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', l.id, '<instid>' + CASE l.src_cid WHEN 'LOBC' THEN 'LOBC' ELSE ISNULL(lob.instID, ISNULL(lob.path, ISNULL(p.instID, p.path))) END + '</instid>'
  FROM #links AS l
    -- The target of the link maps to the src object from the link table
    LEFT OUTER JOIN #lob AS lob ON lob.cid = l.src_cid AND lob.id = l.src_id
    LEFT OUTER JOIN #phys AS p ON p.cid = l.src_cid AND p.id = l.src_id
  WHERE l.src_cid IN ('LOB', 'LOBC')
    AND l.is_lnkx = 0

  -- add the uinstid / upath tags for physical resources
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', ll.id,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #links AS l
    -- The source of the link maps to the dst object from the link table
    JOIN #phys AS p ON p.cid = l.dst_cid AND p.id = l.dst_id AND p.cid <> 'LOB'
    JOIN #links AS ll ON ll.src_cid = l.src_cid AND ll.src_id = l.src_id
  WHERE l.src_cid = 'LOB'
    AND l.is_lnkx = 0
    AND p.flags <> 2

/*
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', l.id,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #links AS l
    -- The source of the link maps to the dst object from the link table
    JOIN #phys AS p ON p.cid = l.dst_cid AND p.id = l.dst_id AND p.cid <> 'LOB'
  WHERE l.dst_cid <> 'LOB'
    AND l.src_cid = 'LOB'
    AND l.is_lnkx = 0
    AND p.flags <> 2
*/

  -- insert the physical object's  uinstid / upath for source lobs when GENL 
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', l.id,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #links AS l 
    JOIN #lob AS lob ON lob.cid = l.src_cid AND lob.id = l.src_id
    JOIN #links AS ll ON ll.src_cid = l.src_cid AND ll.src_id = l.src_id
    JOIN #phys AS p ON p.cid = ll.dst_cid AND p.id = ll.dst_id
  WHERE l.link_type = 'GENL'
    AND l.is_lnkx = 0
    AND lob.done <> 2

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '</id>'
  FROM #links
  WHERE is_lnkx = 0
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '</tgt>'
  FROM #links
  WHERE is_lnkx = 0

  -- Create the <src> data
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<src>'
  FROM #links
  WHERE is_lnkx = 0
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '<id>'
  FROM #links
  WHERE is_lnkx = 0

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', l.id, c.btag + c.cdata + c.etag
  FROM #links AS l
    -- The source of the link maps to the dst object from the link table
    JOIN #class AS c ON c.cid = l.dst_cid
  WHERE l.is_lnkx = 0

  -- insert the LOB's instid tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', l.id, 
   CASE WHEN lob.instID IS NOT NULL OR p.instID IS NOT NULL OR lob.path IS NOT NULL THEN '<instid>'
        ELSE '<path>' END
   + ISNULL(lob.instID, ISNULL(lob.path, ISNULL(p.instID, p.path))) + 
   CASE WHEN lob.instID IS NOT NULL OR p.instID IS NOT NULL OR lob.path IS NOT NULL THEN '</instid>'
        ELSE '</path>' END
  FROM #links AS l
    -- The source of the link maps to the dst object from the link table
    LEFT OUTER JOIN #lob AS lob ON lob.cid = l.dst_cid AND lob.id = l.dst_id
    LEFT OUTER JOIN #phys AS p ON p.cid = l.dst_cid AND p.id = l.dst_id
  WHERE l.is_lnkx = 0

  -- insert the physical object's uinstid / upath opening tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT DISTINCT 'LINK', ll.id,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #links AS l 
    JOIN #lob AS lob ON lob.cid = l.src_cid AND lob.id = l.src_id
    -- The source of the link maps to the dst object from the link table
    JOIN #phys AS p ON p.cid = l.dst_cid AND p.id = l.dst_id AND p.cid <> 'LOB'
    JOIN #links AS ll ON ll.dst_cid = l.src_cid AND ll.dst_id = l.src_id
  WHERE l.src_cid = 'LOB'
    AND l.is_lnkx = 0
    AND lob.done <> 2

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '</id>'
  FROM #links
  WHERE is_lnkx = 0
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '</src>'
  FROM #links
  WHERE is_lnkx = 0

  -- Create the closing tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '</link>'
  FROM #links
  WHERE is_lnkx = 0

  -- create the closing tag
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, '</tbsm:addLinkInstances>'
  FROM #links

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter17 = @elapsed_ms + @sqlCounter17
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Insert blank line for formatting
  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'LINK', id, ''
  FROM #links
  WHERE is_lnkx = 0


  -- Determine the highest ident for lobs and physical resources
  DECLARE @obj_ident INT
  SELECT @obj_ident = MAX(ident) 
  FROM #obj
  SET @obj_ident = ISNULL(@obj_ident, 0) + 1

  --
  -- Create the method tags
  --

  INSERT INTO #output (inst_pre, inst_id, line)
  SELECT 'METHOD', ident, '<tbsm:methodExecution>'
  FROM #method_distinct

  -- Create the <methodgroupdefinitions> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', mc.ident, 0, '<methodgroupdefinitions>'
  FROM #method_distinct AS mc
  ORDER BY mc.ident

  -- Create the <methodgroup> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', mc.ident, 0, '<methodgroup>'
  FROM #method_distinct AS mc
  ORDER BY mc.ident

  -- Create the <instances> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, 0, '<instances>'
  FROM #method_distinct AS md
  ORDER BY md.ident

  -- Create the opening <inst> tags
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT DISTINCT 'METHOD', md.ident, o.ident, '<inst>'
  FROM #method_distinct AS md
    JOIN #method_calls AS mc ON mc.md_ident = md.ident
    JOIN #obj AS o ON o.cid = mc.cid AND o.id = mc.id
  ORDER BY md.ident

  -- Create the <class> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT DISTINCT 'METHOD', md.ident, o.ident, c.btag + c.cdata + c.etag
  FROM #method_distinct AS md
    JOIN #method_calls AS mc ON mc.md_ident = md.ident
    JOIN #obj AS o ON o.cid = mc.cid AND o.id = mc.id
    JOIN #class AS c ON c.cid = o.cid
  ORDER BY md.ident

  -- Create the <instid> tag for LOBs
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT DISTINCT 'METHOD', md.ident, o.ident, '<instid>' + ISNULL(l.instID, l.path) + '</instid>'
  FROM #method_distinct AS md
    JOIN #method_calls AS mc ON mc.md_ident = md.ident
    JOIN #obj AS o ON o.cid = mc.cid AND o.id = mc.id
    JOIN #lob AS l ON l.cid = o.cid AND l.id = o.id
  ORDER BY md.ident

  -- Create the identifier tags for physical resources
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT DISTINCT 'METHOD', md.ident, o.ident,
    CASE WHEN p.instID IS NOT NULL THEN '<uinstid>' ELSE '<upath>' END +
    p.cid + ':' + 
    ISNULL(p.instID, p.path) +
    CASE WHEN p.instID IS NOT NULL THEN '</uinstid>' ELSE '</upath>' END
  FROM #method_distinct AS md
    JOIN #method_calls AS mc ON mc.md_ident = md.ident
    JOIN #obj AS o ON o.cid = mc.cid AND o.id = mc.id
    JOIN #links AS l ON l.src_cid = o.cid AND l.src_id = o.id
    JOIN #phys AS p ON p.cid = l.dst_cid AND p.id = l.dst_id
  WHERE l.link_type = 'LOBL'
    AND o.tmpd <> 2
  ORDER BY md.ident

  -- Create the closing <inst> tags
  INSERT INTO #output (inst_pre, inst_id, sub_id, line, not_used)
  SELECT DISTINCT 'METHOD', md.ident, o.ident, '</inst>', md.ident
  FROM #method_distinct AS md
    JOIN #method_calls AS mc ON mc.md_ident = md.ident
    JOIN #obj AS o ON o.cid = mc.cid AND o.id = mc.id
  ORDER BY md.ident

  -- Create the closing <instances> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, '</instances>'
  FROM #method_distinct AS md
  ORDER BY md.ident

  -- Create the opening <methods> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, '<methods>'
  FROM #method_distinct AS md
  ORDER BY md.ident

  -- Create the opening method tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, '<' + md.mname + '>'
  FROM #method_distinct AS md
  ORDER BY md.ident

  -- Create the parameter tags
  DECLARE @param_id INT, @param_id_str NVARCHAR(9), @obj_ident_str NVARCHAR(9)
  SET @param_id = 1
  SET @param_id_str = CAST(@param_id AS NVARCHAR(9))
  SET @obj_ident_str = CAST(@obj_ident AS NVARCHAR(9))
  WHILE @param_id < 17
  BEGIN
    IF @param_id = 1 BEGIN
      INSERT INTO #output (inst_pre, inst_id, sub_id, line)
      SELECT 
        'METHOD', ident, @obj_ident, 
        CASE mname 
          WHEN 'SetThresholds'           THEN '<ThresholdObjects>' 
          WHEN 'DefinePBTs'              THEN '<PBTsToDefine>'
          WHEN 'DefineEVService'         THEN '<ServiceDef>'
          ELSE '<DUMMY>'
        END
      FROM #method_distinct
      ORDER BY ident

      -- Create the parameter tags for SetSendStateChangeEvent method
      INSERT INTO #output (inst_pre, inst_id, sub_id, line)
      SELECT 'METHOD', ident, @obj_ident, '<StateChange>' + param1 + '</StateChange>'
      FROM #method_distinct 
      WHERE mname = 'SetSendStateChangeEvent'
      ORDER BY ident
    END

    EXEC('
    INSERT INTO #output (inst_pre, inst_id, sub_id, line)
    SELECT ''METHOD'', md.ident, ' + @obj_ident_str + ', ''<'' + RIGHT(sc.name, LEN(sc.name)-1) + ''>'' + md.param' + @param_id_str + ' + ''</'' + RIGHT(sc.name, LEN(sc.name)-1) + ''>''
    FROM #method_distinct AS md
      JOIN sysobjects AS so ON so.name = ''_'' + md.mname AND so.type = ''P''
      JOIN syscolumns AS sc ON sc.id = so.id AND sc.colid = ' + @param_id_str + ' + 2 -- the first two are cid and id, so we do not count them
    WHERE md.param' + @param_id_str + ' IS NOT NULL
    ORDER BY md.ident
    ')

    IF @param_id = 4
      INSERT INTO #output (inst_pre, inst_id, sub_id, line)
      SELECT 'METHOD', ident, @obj_ident, '</ThresholdObjects>' 
      FROM #method_distinct
      WHERE mname = 'SetThresholds'
      ORDER BY ident

    IF @param_id = 6
    BEGIN
      INSERT INTO #output (inst_pre, inst_id, sub_id, line)
      SELECT 'METHOD', ident, @obj_ident, '</ServiceDef>' 
      FROM #method_distinct
      WHERE mname = 'DefineEVService'
      ORDER BY ident

      -- Insert the UserIDs for each ServiceDef
      INSERT INTO #output (inst_pre, inst_id, sub_id, line)
      SELECT 'METHOD', md.ident, @obj_ident, '<UserIDDef><UserID>' + ser.userid + '</UserID></UserIDDef>'
      FROM #method_distinct AS md
        JOIN #method_calls AS mc ON mc.md_ident=md.ident
        JOIN #seruser AS ser ON ser.lobid=mc.id
      WHERE md.mname = 'DefineEVService'
      ORDER BY md.ident
    END

    IF @param_id = 16
      INSERT INTO #output (inst_pre, inst_id, sub_id, line)
      SELECT 'METHOD', ident, @obj_ident, '</PBTsToDefine>' 
      FROM #method_distinct
      WHERE mname = 'DefinePBTs'
      ORDER BY ident

    SET @param_id = @param_id + 1
    SET @param_id_str = CAST(@param_id AS NVARCHAR(9))

  END -- WHILE @param_id < 17
  DELETE FROM #output WHERE line = '<DUMMY>'
  
  -- Create the closing method tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, '</' + md.mname + '>'
  FROM #method_distinct AS md
  ORDER BY md.ident
  -- Replace DefinePBTs to DefineAndUsePBTs
  UPDATE #output SET line = REPLACE(line, 'DefinePBTs', 'DefineAndUsePBTs')
    WHERE line LIKE '<%DefinePBTs>'
  -- Replace DefineEVService to DefineServicesAndLinkToEvUsers
  UPDATE #output SET line = REPLACE(line, 'DefineEVService', 'DefineServicesAndLinkToEvUsers')
    WHERE line LIKE '<%DefineEVService>'
  -- Replace Salience with Relevance
  UPDATE #output SET line = REPLACE(line, 'Salience', 'Relevance')
    WHERE line LIKE '<%Salience>'

  -- Create the closing <methods> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, '</methods>'
  FROM #method_distinct AS md
  ORDER BY md.ident

  -- Create the closing <methodgroup> tag
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, '</methodgroup>'
  FROM #method_distinct AS md
  ORDER BY md.ident

  -- create the closing tag
  DECLARE @max_sub_id INT
  SELECT @max_sub_id = MAX(sub_id) FROM #output WHERE inst_pre = 'METHOD'
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', ident, @max_sub_id, '</methodgroupdefinitions>'
  FROM #method_distinct
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', ident, @max_sub_id, '</tbsm:methodExecution>'
  FROM #method_distinct

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  -- Insert a blank line for formatting
  INSERT INTO #output (inst_pre, inst_id, sub_id, line)
  SELECT 'METHOD', md.ident, @obj_ident, ''
  FROM #method_distinct AS md
  ORDER BY md.ident

  IF @verbose = 6 BEGIN
    SELECT '#obj', * FROM #obj
    SELECT '#method_calls', * FROM #method_calls
    SELECT '#method_distinct', * FROM #method_distinct
    SELECT '#output', * FROM #output ORDER BY inst_pre, inst_id, sub_id, id
  END
  
  -- If no path specified, then output to screen
  IF @path IS NULL
  BEGIN
    SELECT line
    FROM #output
    WHERE inst_pre IN ('METHOD', 'LOB', 'LINK')
    ORDER BY inst_pre, inst_id, sub_id, id
  END

  IF @path IS NOT NULL
  BEGIN
    IF @path <> 'not_to_stdout' 
    BEGIN
    
      -- output the data to the file specified
      DECLARE @line      NVARCHAR(4000)
      DECLARE @intTime   INTEGER
      DECLARE @tableName NVARCHAR(64)
      DECLARE @dynSQL    NVARCHAR(512)
      DECLARE @utcTime   DATETIME
      
      SET @utcTime=GETUTCDATE()
      SET @intTime=CAST(@utcTime AS INTEGER)
      

      DECLARE @schemaLocation NVARCHAR(255)
      EXEC master..xp_regread 'HKEY_LOCAL_MACHINE', 'SOFTWARE\Accessible Software, Inc.\Access1\1.0', 'InstallDirectory', @schemaLocation OUTPUT
      SET @schemaLocation = @schemaLocation + "\xml\lob.xsd"
      
      
      -- Create the table name
      SET @tableName='##EXPORT_XML_' + CAST(@intTime AS NVARCHAR(30)) + '_' + CAST(DATEPART(hour,@utcTime) AS NVARCHAR(3)) + CAST(DATEPART(minute,@utcTime) AS NVARCHAR(3)) + CAST(DATEPART(second,@utcTime) AS NVARCHAR(3))
      
      -- Create a global temp table for the output
      SET @dynSQL=N'CREATE TABLE ' + @tableName + ' ( id   INTEGER IDENTITY(1,1), line NVARCHAR(3500) )'
      EXEC sp_executesql @dynSQL
      -- Insert first XML tag
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<?xml version="1.0" encoding="UTF-8"?>'''
      EXEC sp_executesql @dynSQL
      -- Insert schemaLocation
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<tbsm:xmltoolkit xmlns:tbsm="http://www.ibm.com/tbsmxmltoolkit"  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"  xsi:schemaLocation="http://www.ibm.com/tbsmxmltoolkit file:' + @schemaLocation + '">'''
      EXEC sp_executesql @dynSQL
      -- Insert the creation date into the output table
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<!--  CreateDate:' + CAST(@intTime AS NVARCHAR(32)) + '  -->'''
      EXEC sp_executesql @dynSQL
      -- Insert the creation hour into the output table
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<!--  CreateHour:' + CAST(DATEPART(hour,@utcTime) AS NVARCHAR(16)) + '  -->'''
      EXEC sp_executesql @dynSQL
      -- Insert the creation minute into the output table
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<!--  CreateMinute:' + CAST(DATEPART(minute,@utcTime) AS NVARCHAR(16)) + '  -->'''
      EXEC sp_executesql @dynSQL
      -- Insert the template version
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<!--  Version: 1 -->'''
      EXEC sp_executesql @dynSQL
      -- Insert the template version
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''<!--  ' + CAST(CURRENT_TIMESTAMP AS NVARCHAR(30)) + ' -->'''
      EXEC sp_executesql @dynSQL
      -- Insert the XML into the output table
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT CAST(line AS NVARCHAR(3500)) FROM #output ORDER BY inst_pre, inst_id, sub_id, id'
      EXEC sp_executesql @dynSQL
      -- Insert final XML tag
      SET @dynSQL='INSERT INTO ' + @tableName + ' (line) SELECT ''</tbsm:xmltoolkit>'''
      EXEC sp_executesql @dynSQL
      -- Write the contents of the output table to a file
      SET @line = 'sh exportbcptofile.sh -f "' + @path + '" -t ' + @tableName
      EXEC master..xp_cmdshell @line, NO_OUTPUT
      
    END -- IF @path IS NOT NULL
  END  

  IF @verbose = 7
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter19 = @elapsed_ms + @sqlCounter19
    SET @start1 = CURRENT_TIMESTAMP
  END
    
  IF @verbose = 7
  BEGIN
  
    SET @elapsed_ms = DATEDIFF(MS, @start, CURRENT_TIMESTAMP)
    RAISERROR('   cl_validateCache took %d ms', 0, 1,@elapsed_ms) WITH NOWAIT
    RAISERROR('     sqlCounter1     took %d ms', 0, 1,@sqlCounter1) WITH NOWAIT
    RAISERROR('     sqlCounter2001  took %d ms', 0, 1,@sqlCounter2001) WITH NOWAIT
    RAISERROR('     sqlCounter2002  took %d ms', 0, 1,@sqlCounter2002) WITH NOWAIT
    RAISERROR('     sqlCounter2003  took %d ms', 0, 1,@sqlCounter2003) WITH NOWAIT
    RAISERROR('     sqlCounter2     took %d ms', 0, 1,@sqlCounter2) WITH NOWAIT
    RAISERROR('     sqlCounter3     took %d ms', 0, 1,@sqlCounter3) WITH NOWAIT
    RAISERROR('     sqlCounter4     took %d ms', 0, 1,@sqlCounter4) WITH NOWAIT
    RAISERROR('     sqlCounter5     took %d ms', 0, 1,@sqlCounter5) WITH NOWAIT
    RAISERROR('     sqlCounter6     took %d ms', 0, 1,@sqlCounter6) WITH NOWAIT
    RAISERROR('     sqlCounter7     took %d ms', 0, 1,@sqlCounter7) WITH NOWAIT
    RAISERROR('     sqlCounter7001  took %d ms', 0, 1,@sqlCounter7001) WITH NOWAIT
    RAISERROR('     sqlCounter8     took %d ms', 0, 1,@sqlCounter8) WITH NOWAIT
    RAISERROR('     sqlCounter9     took %d ms', 0, 1,@sqlCounter9) WITH NOWAIT
    RAISERROR('     sqlCounter10    took %d ms', 0, 1,@sqlCounter10) WITH NOWAIT
    RAISERROR('     sqlCounter11    took %d ms', 0, 1,@sqlCounter11) WITH NOWAIT
    RAISERROR('     sqlCounter12    took %d ms', 0, 1,@sqlCounter12) WITH NOWAIT
    RAISERROR('     sqlCounter13    took %d ms', 0, 1,@sqlCounter13) WITH NOWAIT
    RAISERROR('     sqlCounter14    took %d ms', 0, 1,@sqlCounter14) WITH NOWAIT
    RAISERROR('     sqlCounter15    took %d ms', 0, 1,@sqlCounter15) WITH NOWAIT
    RAISERROR('     sqlCounter16    took %d ms', 0, 1,@sqlCounter16) WITH NOWAIT
    RAISERROR('     sqlCounter17    took %d ms', 0, 1,@sqlCounter17) WITH NOWAIT
    RAISERROR('     sqlCounter18    took %d ms', 0, 1,@sqlCounter18) WITH NOWAIT
    RAISERROR('     sqlCounter19    took %d ms', 0, 1,@sqlCounter19) WITH NOWAIT
  END
  
  RAISERROR(65008, 0, 1, @countlob, @countlink, @countlobc, @countlobl, @countgenl)
  RAISERROR(65009, 0, 1, @countmethod, @countthreshold, @countpbt, @countservice, @countsendevent, @countweight, @countobjpri)

  -- Print out LOBs that couldnt be exported
  IF EXISTS (SELECT 1 FROM #lobs_fail)
  BEGIN
    -- TODO: Load this into schemamessages
    RAISERROR(65010, 0, 1)

    DECLARE @maxname INT, @p_cid ClassID, @p_id ObjID, @p_name NVARCHAR(100)
    DECLARE @p_path NVARCHAR(2000), @p_str NVARCHAR(3000), @loop TINYINT
    SELECT @maxname = MAX(LEN(name)) FROM #lobs_fail
    IF @maxname < 5 SET @maxname = 4

    SET @loop = 1
    DECLARE pr_c CURSOR FOR SELECT cid, id, name, path FROM #lobs_fail
      ORDER BY cid, id
    OPEN pr_c
    FETCH NEXT FROM pr_c INTO @p_cid, @p_id, @p_name, @p_path

    WHILE @@FETCH_STATUS <> -1
    BEGIN
      IF @loop = 1
      BEGIN
        SET @p_str = 'ClassID  ResourceID  Name  ' + SPACE(@maxname - 4) + 'ResourcePath'
        RAISERROR('%s', 0, 1, @p_str)  
        SET @p_str = '-------  ----------  ----  ' + SPACE(@maxname - 4) + '------------'
        RAISERROR('%s', 0, 1, @p_str)  
        SET @loop = 2
      END

      SET @p_str = @p_cid + SPACE(9 - LEN(@p_cid)) + 
                   CAST(@p_id AS NVARCHAR(9)) + SPACE(12 - LEN(@p_id)) + 
                   @p_name + SPACE(@maxname+2 - LEN(@p_name)) +
                   ISNULL(@p_path, '')
      RAISERROR('%s', 0, 1, @p_str)
 
      FETCH NEXT FROM pr_c INTO @p_cid, @p_id, @p_name, @p_path
    END
    DEALLOCATE pr_c
  END
GO      
