-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_forwardDB2SubsystemEvent.sql
-- SID: 1.20
-- Date Modified: 04/10/14 15:04:07
--
-- Module Description:
--   Creates the asisp_forwardDB2SubsystemEvent stored procedure

USE Object
GO

SET ANSI_DEFAULTS OFF
SET CONCAT_NULL_YIELDS_NULL OFF
GO

---------------------------------------------------------------------------------------------------
-- asisp_forwardDB2SubsystemEvent
PRINT 'Creating procedure asisp_forwardDB2SubsystemEvent'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_forwardDB2SubsystemEvent' AND type = 'P')
  DROP PROCEDURE asisp_forwardDB2SubsystemEvent
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_forwardDB2SubsystemEvent.sql
-- SID: 1.20
-- Date Modified: 04/10/14 15:04:07
-- Called when a DB2 subsystem event is received on a DataSharingGroup
CREATE PROCEDURE asisp_forwardDB2SubsystemEvent
  @cid        ClassID,        -- Subsystem cid
  @id         ObjID,          -- Subsystem id
  @name       ObjName,        -- Subsystem Name
  @MESSAGE    NVARCHAR(255),  -- Event name, either DB2_SUBSYSTEM_DOWN or DB2_SUBSYSTEM_UP
  @TIME       DATETIME,       -- Original timestamp of event
  @SEQ_NO     NVARCHAR(100),  -- Original sequence number
  @verbose    TINYINT = 0
AS
  SET NOCOUNT ON
  DECLARE @now NVARCHAR(40), @rc INT, @err INT
  DECLARE @d2ss_id ObjID, @d2dg_id ObjID
  SET @d2ss_id = @id

  IF @cid <> 'D2SS'
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: @cid %s not supported - must be D2SS.', 0, 1, @now, @cid) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: @d2ss_id=%d, @name=%s, @MESSAGE=%s, @SEQ_NO=%s', 0, 1, @now, @d2ss_id, @name, @MESSAGE, @SEQ_NO) WITH NOWAIT
  END

--------------------------------------------------------------------------------------------
-- Make sure ForwardDB2SubsystemEvent is up to date, and determine @d2dg_id for this @d2ss_id
-- The d2ss_state and d2dg_state fields are used to determine if the events is forward to the children
-- This table has has fields that are helpful for diagnosis purposes.
--------------------------------------------------------------------------------------------
  -- Delete obsolete entries
  DELETE f
    FROM ForwardDB2SubsystemEvent AS f
    LEFT OUTER JOIN link          AS d2ss_d2dg ON d2ss_d2dg.src_cid = 'D2SS' AND d2ss_d2dg.src_id = f.d2ss_id
                                              AND d2ss_d2dg.dst_cid = 'D2DG' AND d2ss_d2dg.dst_id = f.d2dg_id
                                              AND d2ss_d2dg.deleted = 0
    WHERE d2ss_d2dg.src_id IS NULL
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) delete from ForwardDB2SubsystemEvent for obselete d2ss-d2dg links', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Add new entries
  INSERT INTO ForwardDB2SubsystemEvent (d2dg_id, d2ss_id, d2dg_state, d2ss_state, d2dg_name, d2ss_name, d2dg_cnt_up,  d2ss_cnt_up, d2dg_cnt_down,  d2ss_cnt_down)
    SELECT DISTINCT 
      d2dg.id AS 'd2dg_id',
      d2ss.id AS 'd2ss_id',
      CASE d2dg_state._State WHEN 'Available' THEN 'UP' WHEN 'Unavailable' THEN 'DOWN' ELSE 'UNKNOWN' END AS 'd2dg_state', 
      CASE d2ss_state._State WHEN 'Active'    THEN 'UP' WHEN 'Inactive'    THEN 'DOWN' ELSE 'UNKNOWN' END AS 'd2ss_state',
      d2dg.name AS 'd2dg_name',
      d2ss.name AS 'd2ss_name',
      0, 0, 0, 0 -- cnt_up/down stats
    FROM DB2DataSharingGroup_C AS d2dg
    JOIN StateType_V           AS d2dg_state ON d2dg_state.id = d2dg._CurrentStateID
    JOIN link                  AS d2ss_d2dg  ON d2ss_d2dg.dst_cid = d2dg.cid AND d2ss_d2dg.dst_id = d2dg.id
    JOIN DB2Subsystem_C        AS d2ss       ON d2ss.cid = d2ss_d2dg.src_cid AND d2ss.id = d2ss_d2dg.src_id
    JOIN StateType_V           AS d2ss_state ON d2ss_state.id = d2ss._CurrentStateID
    LEFT OUTER JOIN ForwardDB2SubsystemEvent AS f ON f.d2ss_id = d2ss_d2dg.src_id AND f.d2dg_id = d2ss_d2dg.dst_id
    WHERE d2dg.deleted = 0
      AND d2ss_d2dg.deleted = 0 AND d2ss_d2dg.src_cid = 'D2SS' AND d2ss_d2dg.dst_cid = 'D2DG'
      AND d2ss.deleted = 0
      AND f.d2ss_id IS NULL
    ORDER BY d2dg.id, d2ss.id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) inserted into ForwardDB2SubsystemEvent for new d2ss-d2dg links', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- update d2dg name (this field is used for diagnosis purposes ony)
  UPDATE f SET d2dg_name = d2dg.name
    FROM ForwardDB2SubsystemEvent AS f
    JOIN DB2DataSharingGroup_C    AS d2dg ON d2dg.id = f.d2dg_id
    WHERE ISNULL(f.d2dg_name,'') <> ISNULL(d2dg.name,'')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) updated in ForwardDB2SubsystemEvent for d2dg_name', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- update d2ss name (this field is used for diagnosis purposes ony)
  UPDATE f SET d2ss_name = d2ss.name
    FROM ForwardDB2SubsystemEvent AS f
    JOIN DB2Subsystem_C           AS d2ss ON d2ss.id = f.d2ss_id
    WHERE ISNULL(f.d2ss_name,'') <> ISNULL(d2ss.name,'')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) updated in ForwardDB2SubsystemEvent for d2ss_name', 0, 1, @now, @rc) WITH NOWAIT
  END
 
  IF @verbose >= 2
  BEGIN
    SELECT * FROM ForwardDB2SubsystemEvent
    SET @rc = @@ROWCOUNT
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) in ForwardDB2SubsystemEvent', 0, 1, @now, @rc) WITH NOWAIT
  END

  SELECT @d2dg_id = d2dg_id FROM ForwardDB2SubsystemEvent WHERE d2ss_id = @d2ss_id
  IF @d2dg_id IS NULL
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_forwardDB2SubsystemEvents: D2SS:%d does not have a D2DG child - nothing to forward events to.', 0, 1, @now) WITH NOWAIT
    END
    RETURN 0   --- <<<<<<<<< ABORT <<<<<<<<
  END

--------------------------------------------------------------------------------------------
--  Determine if related D2DG object should have state change, and also have events forward to children
--------------------------------------------------------------------------------------------
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: D2SS:%d has child D2DG:%d.', 0, 1, @now, @d2ss_id, @d2dg_id) WITH NOWAIT
  END

  DECLARE @time DATETIME, @d2ss_state NVARCHAR(15), @d2dg_state NVARCHAR(15)
  SET @time       = CURRENT_TIMESTAMP
  SET @d2ss_state = CASE WHEN @MESSAGE = 'DB2_SUBSYSTEM_UP' THEN 'UP' ELSE 'DOWN' END

  -- Update ForwardDB2SubsystemEvent for specified @d2ss_id
  IF @d2ss_state = 'UP'
    UPDATE f SET d2ss_state     = @d2ss_state,
                 d2ss_time_up   = @time,            -- this field is used for diagnosis purposes ony
                 d2ss_cnt_up    = d2ss_cnt_up + 1   -- this field is used for diagnosis purposes ony
    FROM ForwardDB2SubsystemEvent AS f
    WHERE d2ss_id = @d2ss_id
  ELSE
    UPDATE f SET d2ss_state     = @d2ss_state,
                 d2ss_time_down = @time,            -- this field is used for diagnosis purposes ony
                 d2ss_cnt_down  = d2ss_cnt_down + 1 -- this field is used for diagnosis purposes ony
    FROM ForwardDB2SubsystemEvent AS f
    WHERE d2ss_id = @d2ss_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) updated in ForwardDB2SubsystemEvent - d2ss_state %s for d2ss_id %d', 0, 1, @now, @rc, @d2ss_state, @d2ss_id) WITH NOWAIT
  END

  -- Deletermine if d2dg state should change
  IF @d2ss_state = 'UP'                                            -- IF subsystem is now UP
  AND EXISTS(SELECT * FROM ForwardDB2SubsystemEvent 
               WHERE d2dg_id = @d2dg_id AND d2dg_state <> 'UP')    -- AND d2dg is not currently UP
  BEGIN
    SET @d2dg_state = 'UP'                                          --   THEN we will set d2dg UP
  END
  ELSE IF @d2ss_state = 'DOWN'                                     -- ELSE IF subsystem is now DOWN
       AND EXISTS(SELECT * FROM ForwardDB2SubsystemEvent              
                    WHERE d2dg_id = @d2dg_id AND d2dg_state <> 'DOWN')  -- AND d2dg is not currently down
       AND NOT EXISTS(SELECT * FROM ForwardDB2SubsystemEvent              
                        WHERE d2dg_id = @d2dg_id AND d2ss_state = 'UP') -- AND no other related d2ss are up
  BEGIN
    SET @d2dg_state = 'DOWN'                                            -- THEN we will set the d2dg DOWN
  END
  ELSE 
  BEGIN                                                            -- ELSE we will NOT change d2dg state
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_forwardDB2SubsystemEvents: No change in D2DG state.', 0, 1, @now) WITH NOWAIT
    END
    RETURN 0   --- <<<<<<<<< ABORT <<<<<<<<
  END

--------------------------------------------------------------------------------------------
--  Determine children objects, and forward events 
--------------------------------------------------------------------------------------------
  DECLARE @d2dg_nchild INT
  -- Locate child objects
  CREATE TABLE #obj
  (
    cid     ClassID NOT NULL,
    id      ObjID   NOT NULL  PRIMARY KEY CLUSTERED(cid, id),
    gen     INT     NOT NULL,
    mesg    TINYINT NOT NULL
  )
  DECLARE @gen INT
  SET @gen = 0
  INSERT INTO #obj 
    SELECT 'D2DG', @d2dg_id, @gen, 1    -- D2DG itself is first generation
  SET @rc = @@ROWCOUNT
  SET @d2dg_nchild = 0
  WHILE @rc > 0
  BEGIN
    SET @gen = @gen + 1
    INSERT INTO #obj
      SELECT DISTINCT l.dst_cid, l.dst_id, @gen, 0
        FROM #obj AS o
        JOIN link AS l ON l.src_cid = o.cid AND l.src_id = o.id
        WHERE o.gen = @gen-1 AND o.cid <> 'D2VW'
          AND l.deleted = 0 AND l.dst_cid LIKE 'D2%'
    SET @rc = @@ROWCOUNT
    SET @d2dg_nchild = @d2dg_nchild + @rc
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_forwardDB2SubsystemEvents: found %d DB2DataSharingGroup descendents at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
    END
  END

  -- Update ForwardDB2SubsystemEvent for @d2dg_id
  IF @d2dg_state = 'UP'
    UPDATE f SET d2dg_state     = @d2dg_state,
                 d2dg_nchild    = @d2dg_nchild,       -- this field is used for diagnosis purposes ony
                 d2dg_time_up   = @time,              -- this field is used for diagnosis purposes ony
                 d2dg_cnt_up    = d2dg_cnt_up + 1     -- this field is used for diagnosis purposes ony
      FROM ForwardDB2SubsystemEvent AS f
      WHERE d2dg_id = @d2dg_id
  ELSE
    UPDATE f SET d2dg_state     = @d2dg_state,
                 d2dg_nchild    = @d2dg_nchild,       -- this field is used for diagnosis purposes ony
                 d2dg_time_down = @time,              -- this field is used for diagnosis purposes ony
                 d2dg_cnt_down  = d2dg_cnt_down + 1   -- this field is used for diagnosis purposes ony
      FROM ForwardDB2SubsystemEvent AS f
      WHERE d2dg_id = @d2dg_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvents: %d row(s) updated in ForwardDB2SubsystemEvent to set d2dg_state to %s for d2dg_id %d', 0, 1, @now, @rc, @d2dg_state, @d2dg_id) WITH NOWAIT
  END

  --  Send messages to set AlertState & State  (common proc with IMS subsystem processing)
  EXEC @rc = asisp_forwardSubsystemEvent @cid='D2SS', @id=@d2ss_id, @name=@name, @state='', @MESSAGE=@MESSAGE, @TIME=@TIME, @SEQ_NO=@SEQ_NO, @verbose=@verbose
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_forwardDB2SubsystemEvent: completed rc %d', 0, 1, @now, @rc) WITH NOWAIT
  END
  RETURN @rc

GO
