-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_processstagedchevs.sql
-- SID: 1.29
-- Date Modified: 04/10/27 13:15:58
--
-- Module Description
--   Defines the asisp_processStagedCHEVs stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_processStagedCHEVs
PRINT 'Creating procedure asisp_processStagedCHEVs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedCHEVs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedCHEVs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_processstagedchevs.sql
-- SID: 1.29
-- Date Modified: 04/10/27 13:15:58
-- Processes the records in the StagedCHEV table
CREATE PROCEDURE asisp_processStagedCHEVs
  @maxrows  INT = 1000,   -- This is used to control the size of the batches that the procedure
                          -- processes each time through the loop
  @verbose  TINYINT = 0   -- If >0 then verbose progress messages are printed during execution
AS
  -- Temp tables
  
  -- Records the records from StagedCHEV that are being processed
  CREATE TABLE #chev
  (
    timestamp     BINARY(8) NOT NULL,
    cid           ClassID NOT NULL,
    id            ObjID NOT NULL,
    EventCID      ClassID NOT NULL,
    AlertState    INT NOT NULL,
    EventPriority INT NOT NULL,
    SumDirection  INT NOT NULL,
    EventCount    INT NOT NULL,
    EventDetail   NVARCHAR(255) NULL,
    ts            BINARY(8) NOT NULL,
    chev_id       INT NOT NULL DEFAULT 0,
    name          ObjName NULL,
    Priority      INT     NULL,
    ParentCID     ClassID NULL,
    ParentID      ObjID   NULL,
    in_maint      TINYINT NOT NULL DEFAULT 0,
    from_maint    TINYINT NOT NULL DEFAULT 0,
    flags         INT     NOT NULL DEFAULT 0
  )
--  CREATE INDEX i#chev ON #chev (cid, id, EventCID, AlertState, EventPriority)

  -- Records the timestamps of the processed records. Used for final deletion of
  -- processed records from StagedCHEV.
  CREATE TABLE #processed
  (
    timestamp     BINARY(8) NOT NULL PRIMARY KEY
  )
  
  -- Table of parents to receive the child events.
  CREATE TABLE #parent
  (
    cid        ClassID NOT NULL,
    id         ObjID NOT NULL,
    parent_cid ClassID NOT NULL,
    parent_id  ObjID NOT NULL
  )

  CREATE TABLE #oac_update_distinct
  (
    cid         ClassID NOT NULL PRIMARY KEY (cid, id, EventCID, AlertState, Priority),
    id          ObjID NOT NULL,
    EventCID    ClassID NOT NULL,
    AlertState  INT NOT NULL,
    Priority    INT NOT NULL,
    nviolations INT NOT NULL
  )

  SET NOCOUNT ON

  DECLARE @err INT, @rc INT, @now NVARCHAR(40), @clearAlerts NVARCHAR(3)
  DECLARE @rc_tot INT, @num_events_processed INT, @loop INT
  DECLARE @tcount INT
  DECLARE @continue INT, @ts TIMESTAMP, @min_ts TIMESTAMP, @num_oac_updates INT
  DECLARE @new_ts1 TIMESTAMP, @new_ts2 TIMESTAMP

  SET @num_events_processed = 0
  SET @rc_tot = 0

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedCHEVs @maxrows=%d, @verbose=%d', 0, 1, @now, @maxrows, @verbose) WITH NOWAIT
  END

  IF NOT EXISTS (SELECT 1 FROM StagedCHEV)
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: No events to process', 0, 1, @now) WITH NOWAIT
    END
    RETURN 0
  END

  SET @tcount = @@TRANCOUNT

  SET @continue = 1
  SET @loop = 1
    
  -- Loop over all of the StagedCHEV objects, as processing some of these
  -- could create more.
  WHILE @continue = 1
  BEGIN
    SET ROWCOUNT @maxrows

    -- Since we are only processing unique events, this will provide a list of all of the ChildEvents that need to be processed.
    TRUNCATE TABLE #chev
    INSERT INTO #chev (timestamp, cid, id, EventCID, AlertState, EventPriority, SumDirection, EventCount, EventDetail, ts,
      from_maint, ParentCID, ParentID, flags)
    SELECT s.timestamp, s.ChildCID, s.ChildID, s.EventCID, s.AlertState, s.EventPriority, s.Direction, s.Direction, s.EventDetail, s.timestamp,
      s.MaintenanceMode, s.ParentCID, s.ParentID, s.flags
    FROM StagedCHEV AS s
    ORDER BY s.timestamp
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d populating worktable of StagedCHEV records to process', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Got list of %d StagedCHEVs to process', 0, 1, @now, @rc) WITH NOWAIT
      IF @verbose >= 2
        SELECT '#chev 1' AS '#chev 1', * FROM #chev
    END

    SET ROWCOUNT 0

    IF @rc = 0
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedCHEVs: No records left to process', 0, 1, @now, @rc) WITH NOWAIT
      END
      BREAK
    END

    -- We want to find the set of StagedCHEVs that do not contain duplicate entries for the
    -- ChildEvent_C table.
    SET @new_ts1 = NULL
    SELECT @new_ts1 = MIN(s2.timestamp)
    FROM #chev AS s1 WITH (NOLOCK)
    JOIN #chev AS s2 WITH (NOLOCK) ON s1.cid           = s2.cid
                                  AND s1.id            = s2.id
                                  AND s1.EventCID      = s2.EventCID
                                  AND s1.AlertState    = s2.AlertState
                                  AND s1.EventPriority = s2.EventPriority
    WHERE s2.timestamp > s1.timestamp
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d searching for duplicate children', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Found first group of StagedCHEVs to work with', 0, 1, @now) WITH NOWAIT
    END

    -- Adjust the maximum timestamp that we will work with to avoid duplicates
    IF @new_ts1 IS NOT NULL
    BEGIN
      -- Find the timestamp value just before the one we cannot process in this batch
      SET @new_ts2 = NULL
      SELECT @new_ts2 = MAX(s.timestamp)
      FROM #chev AS s
      WHERE s.timestamp < @new_ts1
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedCHEVs: Error %d adjusting max timestamp to avoid duplicates', 11, 1, @now, @err) WITH NOWAIT
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedCHEVs: Adjusted last StagedCHEV that can be processed in this iteration', 0, 1, @now) WITH NOWAIT
      END

      -- Remove all entries past the newly computed maximum timestamp
      IF @new_ts2 IS NOT NULL
      BEGIN
        DELETE FROM #chev WHERE timestamp > @new_ts2
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_processStagedCHEVs: Error %d removing child events that cannot be processed this iteration', 11, 1, @now, @err) WITH NOWAIT
          RETURN -30000
        END
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_processStagedCHEVs: Removed %d child event(s) that cannot be processed in this iteration', 0, 1, @now, @rc) WITH NOWAIT
        END
      END -- Remove all entries past the newly computed maximum timestamp
    END -- Adjust the maximum timestamp that we will work with to avoid duplicates

    -- Build separate list with timestamps of the records we are going to process.
    -- We do this because it is possible that a row may be eliminated from the #chev
    -- table during processing below, so we can't use it do the final delete of the
    -- processed records from StagedCHEV.
    TRUNCATE TABLE #processed
    INSERT INTO #processed SELECT timestamp FROM #chev
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d recording timestamps of records to process', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Recorded timestamp for %d record(s) to process', 0, 1, @now, @rc) WITH NOWAIT
    END
    
    -- Determine which resources are in maintenance mode
    UPDATE c SET 
      in_maint = 1
    FROM #chev AS c
      JOIN MaintTrueState AS m ON m.cid = c.cid AND m.id = c.id
    WHERE m.enabled = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d updating in_maint flag', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Updated in_maint flag for %d row(s) in worktable', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Get the priority and name of each resource
    UPDATE c SET
      Priority = CASE m.Priority WHEN 6 THEN c.EventPriority ELSE m.Priority END,
      name = m.name
    FROM #chev AS c
      JOIN ManagedObject AS m ON m.cid = c.cid AND m.id = c.id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d updating priority', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Updated priority for %d row(s) in worktable', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Delete records that did not have a match in ManagedObject
    DELETE c
    FROM #chev AS c
    WHERE Priority IS NULL
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d deleting potential child events', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Deleted %d rows from #chev that did not have a matching resource in ManagedObject', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Compute the updates for the ObjAlertCount table. The exclusion below of c.ParentCID IS NULL is necessary
    -- because when new links are created StagedCHEV records are generated based on the new parent-child links
    -- to existing child objects. The child in this case does NOT need to have an ObjAlertCount update since
    -- the new parent link to it does not change its state. In the normal flow of propagation the StagedCHEV
    -- records have ParentCID set to NULL, which is the indication to this procedure that it should figure out
    -- what the parent links are.
    INSERT INTO CHEVObjAlertCountUpdate
    SELECT c.cid, c.id, c.EventCID, c.AlertState, c.EventPriority, c.SumDirection
    FROM #chev AS c
    WHERE c.from_maint = 0      -- exclude objects in maintenance
      AND c.ParentCID IS NULL   -- exclude records being generated because of new links (see explanation above)
      AND c.flags & 1 = 0       -- exclude records that should not affect ObjAlertCount
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d computing ObjAlertCount updates', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Computed %d ObjAlertCount updates', 0, 1, @now, @rc) WITH NOWAIT
    END

    --
    -- Determine the ParentCID and ParentID attributes for the StagedCHEVs
    --
    -- Get the parents for each object
    TRUNCATE TABLE #parent
    EXEC asisp_computeStagedCHEVParents @min_ts, @ts, @verbose = @verbose
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      SELECT @rc = COUNT(*) FROM #parent
      RAISERROR('%s: asisp_processStagedCHEVs: Got list of %d parent resource(s)', 0, 1, @now, @rc) WITH NOWAIT
      IF @verbose >= 2
        SELECT '#parent' AS '#parent', * FROM #parent
    END

    -- Insert new records into #chev with resolved parent ids
    INSERT INTO #chev (timestamp, cid, id, EventCID, AlertState, EventPriority, SumDirection, EventCount, EventDetail, ts, ParentCID, ParentID, name, Priority, in_maint)
    SELECT timestamp, c.cid, c.id, c.EventCID, c.AlertState, c.EventPriority, c.SumDirection, c.EventCount, EventDetail, c.ts, p.parent_cid, p.parent_id, c.name, c.Priority, in_maint
    FROM #chev AS c
      JOIN #parent AS p ON p.cid = c.cid  AND p.id = c.id
    WHERE p.parent_cid NOT IN ('LOBC', 'BUSC') -- Exclude the LOBC and BUSC
      AND c.ParentCID IS NULL -- only multiply child events where the parent is not specified
      AND (c.Priority <> 5 OR c.SumDirection = -1)  -- not Ignore
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d inserting potential child events', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Added %d row(s) to #chev table containing parent information', 0, 1, @now, @rc) WITH NOWAIT
    END

    DELETE FROM cnull FROM #chev AS cnull
    JOIN #chev AS cparent ON cparent.cid           = cnull.cid
                         AND cparent.id            = cnull.id
                         AND cparent.EventCID      = cnull.EventCID
                         AND cparent.AlertState    = cnull.AlertState
                         AND cparent.EventPriority = cnull.EventPriority
    WHERE cnull.ParentCID IS NULL
      AND cparent.ParentCID IS NOT NULL
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d removing potential child events with NULL parent information', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Removed %d potential child event(s) with NULL parent information', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Update the EventDetail column
    UPDATE c SET 
      EventDetail =
        CASE c.AlertState WHEN 3 THEN 'Red' WHEN 2 THEN 'Yellow' WHEN 1 THEN 'Green' ELSE 'Unknown' END +
        '/' + CASE c.Priority WHEN 1 THEN 'Critical' WHEN 2 THEN 'High' WHEN 3 THEN 'Medium' WHEN 4 THEN 'Low' ELSE 'Unknown' END +
        ' ' + CASE c.EventCID WHEN 'MESG' THEN 'Message' WHEN 'EXCP' THEN 'Exception' WHEN 'CHEV' THEN 'ChildEvent' END +
        CASE WHEN c.EventCID IN ('EXCP', 'CHEV') THEN ' Threshold ' + CASE WHEN c.SumDirection > 0 THEN 'Exceeded' ELSE 'Crossed' END
        END
    FROM #chev AS c
    WHERE c.EventDetail IS NULL OR c.EventDetail = ''
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d updating event detail for potential child events', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Updated the EventDetail column for %d CHEV record(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    --
    -- Identify which records are new, updating existing ones, or deleting existing ones.
    --
    -- Update the chev_id column for existing records
    UPDATE t SET 
      chev_id = c.id
    FROM #chev AS t
      JOIN ChildEvent_C AS c ON t.cid = c._ChildCID AND t.id = c._ChildID AND t.ParentCID = c._ParentCID AND t.ParentID = c._ParentID
        AND t.AlertState = c._AlertState AND t.EventPriority = c._EventPriority AND t.EventCID = c._EventCID AND c.deleted = 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d looking up previous child event ids', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Found %d existing ChildEvent(s) to be updated', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Perform the permanent table updates. This is done in a transaction because we need to make sure that
    -- the permanent table changes are atomic with the deletion of the processed records from StagedCHEV.
    -- The order of table operations is:
    -- 1. Update existing ObjAlertCount entries
    -- 2. Create new ObjAlertCount entries
    --    1 and 2 trigger:
    --      a. updates to <cname>_A._AlertStateID
    --      b. insert into PendingNotification
    --      c. updates/inserts on MaintTrueState
    --      d. inserts into AlertStateHistory
    -- 3. Mark existing ChildEvent_C records as deleted
    -- 4. Create new ChildEvent_C records
    --    3 and 4 trigger:
    --      a. updates to PropagationSlot
    -- 5. Mark existing MaintTrueChildEvent records as deleted
    -- 6. Create new MaintTrueChildEvent records
    --    There are currently no triggers on MaintTrueChildEvent
    
    IF @tcount = 0
      BEGIN TRANSACTION asisp_processStagedCHEVs
    ELSE
      SAVE TRANSACTION asisp_processStagedCHEVs

    -- Update the deleted flag on ChildEvent_C for those records that have been negated
    UPDATE c SET 
      deleted       = 1, 
      dtime         = CURRENT_TIMESTAMP,
      _DeleteDetail = t.EventDetail
    FROM ChildEvent_C AS c
      JOIN #chev AS t ON t.chev_id = c.id
    WHERE c._EventCount + t.SumDirection = 0
      -- Only process child events whose parent objects are not in maintenance
      AND t.in_maint = 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d updating deleted flag on previous child events', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedCHEVs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Marked %d ChildEvent(s) as deleted', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Create new ChildEvent records
    INSERT INTO ChildEvent_C (_ChildCID, _ChildID, _ParentCID, _ParentID, _EventCID, _AlertState, _EventPriority, _Priority, _EventCount, _EventDetail)
    SELECT c.cid, c.id, c.ParentCID, c.ParentID, c.EventCID, c.AlertState, c.EventPriority, c.Priority, 1, ISNULL(c.EventDetail, '')
    FROM #chev AS c
    WHERE c.chev_id = 0
      AND c.SumDirection > 0
      AND c.in_maint = 0 -- Only create child events for those objects not in maintenance mode
      AND c.ParentCID IS NOT NULL -- Only create child events for objects with parents
    ORDER BY c.ts
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d inserting into ChildEvent_C', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedCHEVs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Created %d new ChildEvent(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Mark ChildEvent records in MaintTrueChildEvent as deleted
    UPDATE m SET 
      deleted = 1
    FROM MaintTrueChildEvent AS m
      JOIN #chev AS t ON t.cid = m._ChildCID AND t.id = m._ChildID AND m._ParentCID = t.ParentCID AND m._ParentID = t.ParentID
        AND t.AlertState = m._AlertState AND t.EventPriority = m._Priority AND t.EventCID = m._EventCID
    WHERE t.in_maint = 1
      AND m.deleted = 0
      AND m._EventCount + t.SumDirection = 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d updating deleted flag on previous child events from MaintTrueChildEvent', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedCHEVs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Marked %d record(s) as deleted in MaintTrueChildEvent', 0, 1, @now, @rc) WITH NOWAIT
    END

    IF @verbose>=2
      SELECT '#chev' AS '#chev', * FROM #chev

    -- Create MaintTrueChildEvent records for those objects in maintenance mode
    INSERT INTO MaintTrueChildEvent (_ChildCID, _ChildID, _ParentCID, _ParentID, _EventCID, _AlertState, _Priority, _EventCount, _EventDetail, deleted, id, name)
    SELECT c.cid, c.id, c.ParentCID, c.ParentID, c.EventCID, c.AlertState, c.EventPriority, 1, ISNULL(c.EventDetail, ''),      0, 0, ''
    FROM #chev AS c
    WHERE c.chev_id = 0 AND c.SumDirection > 0
      AND c.in_maint = 1
      AND c.ParentCID IS NOT NULL
    ORDER BY c.ts
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d inserting into MaintTrueChildEvent', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedCHEVs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Created %d new MaintTrueChildEvent record(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Delete the processed StagedCHEV records
    DELETE FROM s FROM StagedCHEV AS s JOIN #processed AS p ON p.timestamp = s.timestamp
    SET @rc = @@ROWCOUNT
    SET @num_events_processed = @num_events_processed + @rc
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d deleting from StagedCHEV', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_processStagedCHEVs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Deleted %d processed row(s) from StagedCHEV for iteration %d, for a total of %d events', 0, 1, @now, @rc, @loop, @num_events_processed) WITH NOWAIT
    END

    IF @tcount = 0
      COMMIT TRANSACTION asisp_processStagedCHEVs

    -- Increment the counter
    SET @loop = @loop + 1

  END


  --
  -- Update the ObjAlertCount table
  -- The triggers on the table will update the appropriate _AlertStateID columns.
  -- We do not want to do this for events coming from maintenance mode, as this
  -- will provide extraneous counts.
  --
  IF EXISTS (SELECT 1 FROM CHEVObjAlertCountUpdate)
  BEGIN
    INSERT INTO #oac_update_distinct
      SELECT DISTINCT u.cid, u.id, u.EventCID, u.AlertState, u.Priority, SUM(u.nviolations) AS nviolations
      FROM CHEVObjAlertCountUpdate AS u
      GROUP BY u.cid, u.id, u.EventCID, u.AlertState, u.Priority
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d computing distinct ObjAlertCount updates', 11, 1, @now, @err) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Computed %d distinct ObjAlertCount updates', 0, 1, @now, @rc, @loop, @num_events_processed) WITH NOWAIT
      IF @verbose >= 2
      BEGIN
        SELECT '#oac_update_distinct' AS '#oac_update_distinct', *
        FROM #oac_update_distinct AS u
        ORDER BY u.cid, u.id, u.EventCID, u.AlertState, u.Priority
      END
    END
    SET @num_oac_updates = @rc
    
    IF @tcount = 0
      BEGIN TRANSACTION StagedCHEV_OACUpdate
    ELSE
      SAVE TRANSACTION StagedCHEV_OACUpdate
    
    -- Update existing entries
    UPDATE oac SET 
      nviolations = oac.nviolations + u.nviolations
    FROM ObjAlertCount AS oac
      JOIN #oac_update_distinct AS u ON u.cid        = oac.cid
                                    AND u.id         = oac.id
                                    AND u.EventCID   = oac.event_cid
                                    AND u.AlertState = oac.alertstate
                                    AND u.Priority   = oac.priority
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d updating ObjAlertCount', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION StagedCHEV_OACUpdate
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Updated %d record(s) in ObjAlertCount', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- If the number of updates performed is less than expected, then we need to insert
    -- some ObjAlertCount rows
    IF @rc < @num_oac_updates -- create new ObjAlertCount rows
    BEGIN
      -- Add new entries
      INSERT INTO ObjAlertCount (cid, id, event_cid, alertstate, priority, nviolations)
      SELECT DISTINCT u.cid, u.id, u.EventCID, u.AlertState, u.Priority, CASE WHEN SUM(u.nviolations) >= 0 THEN SUM(u.nviolations) ELSE 0 END
      FROM CHEVObjAlertCountUpdate AS u
        LEFT OUTER JOIN ObjAlertCount AS oac ON u.cid        = oac.cid
                                            AND u.id         = oac.id
                                            AND u.EventCID   = oac.event_cid
                                            AND u.AlertState = oac.alertstate
                                            AND u.Priority   = oac.priority
      WHERE oac.cid IS NULL
      GROUP BY u.cid, u.id, u.EventCID, u.AlertState, u.Priority
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedCHEVs: Error %d inserting into ObjAlertCount', 11, 1, @now, @err) WITH NOWAIT
        ROLLBACK TRANSACTION StagedCHEV_OACUpdate
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedCHEVs: Inserted %d row(s) into ObjAlertCount', 0, 1, @now, @rc) WITH NOWAIT
      END
    END -- create new ObjAlertCount rows
    
    -- Remove the CHEVObjAlertCountUpdate rows we've just processed
    DELETE FROM CHEVObjAlertCountUpdate
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Error %d deleting from CHEVObjAlertCountUpdate', 11, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION StagedCHEV_OACUpdate
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedCHEVs: Deleted %d record(s) from CHEVObjAlertCountUpdate', 0, 1, @now, @rc) WITH NOWAIT
    END
    
    IF @tcount = 0
      COMMIT TRANSACTION StagedCHEV_OACUpdate
  END  -- Update the ObjAlertCount table

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedCHEVs: Finished. rc=%d', 0, 1, @now, @num_events_processed) WITH NOWAIT
  END

  RETURN @num_events_processed
GO
