-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_processstagedlobs.sql
-- SID: 1.17
-- Date Modified: 04/10/21 14:56:21
--
-- Module Description
--   Defines the asisp_processStagedLOBs procedure
USE Object
GO

-- asisp_processStagedLOBs
PRINT 'Creating procedure asisp_processStagedLOBs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedLOBs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedLOBs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_processstagedlobs.sql
-- SID: 1.17
-- Date Modified: 04/10/21 14:56:21
CREATE PROCEDURE asisp_processStagedLOBs
  @maxrows  INT = 0,
  @verbose  TINYINT = 0
AS
  -- Create table with all of the links for the new LOBs. We do this before starting the transaction
  -- because all we're interested in doing here is getting the committed inserts. This is safe
  -- because only this procedure deletes from the StagedLOB table.
  CREATE TABLE #inserted -- Must be a temp table because it's used in dynamic SQL
  (
    timestamp   BINARY(8)   NOT NULL,
    link_type   NVARCHAR(4) NOT NULL,
    src_cid     NVARCHAR(4) NOT NULL,
    src_id      INT         NOT NULL,
    dst_cid     NVARCHAR(4) NOT NULL,
    dst_id      INT         NOT NULL
  )

  CREATE TABLE #tempCount -- Must be a temp table because it's used in dynamic SQL
  (
    id          INT         NOT NULL,
    phy_cid     NVARCHAR(4) NOT NULL,
    phy_id      INT         NOT NULL,
    nevents     INT         NOT NULL
  )

  DECLARE @lob TABLE
  (
    dst_id      INT         NOT NULL,
    gen         INT         NOT NULL,
    inCRL       TINYINT     NOT NULL
  )

  DECLARE @newps TABLE
  (
    OwnerCID    NVARCHAR(4) NOT NULL,
    OwnerID     INT         NOT NULL,
    ResourceCID NVARCHAR(4) NOT NULL,
    ResourceID  INT         NOT NULL
  )

  DECLARE @ecount TABLE
  (
    id          INT         NOT NULL,
    Priority    INT         NOT NULL,
    nEXCH       INT         NOT NULL
  )

  DECLARE @lobl_inserted TABLE
  (
    src_id      INT         NOT NULL,
    dst_cid     NVARCHAR(4) NOT NULL,
    dst_id      INT         NOT NULL
  )
  
  DECLARE @bs_priority TABLE
  (
    timestamp  BINARY(8)   NOT NULL,
    cid        NVARCHAR(4) NOT NULL,
    id         INT         NOT NULL,
    priority   INT         NOT NULL
  )

  DECLARE
    @err                INT,
    @rc                 INT,
    @now                NVARCHAR(40),
    @tcount             INT,
    @cid                ClassID,
    @cname              NVARCHAR(24),
    @crl_container_id   ObjID,
    @InCRL              INT,
    @AutoCopyEVService  INT,
    @IsCRL              TINYINT,
    @IsInCRL            TINYINT,
    @gen                INT,
    @max_gen            INT,
    @obj_cid            ClassID,
    @sql                NVARCHAR(4000),
    @dst_cid            ClassID,
    @dst_id             ObjID,
    @rc_bspri           INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Starting', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Cleanup dummy entries. They ensure BSPriority processing when no new LOBs are being initialized
  DELETE FROM StagedLOB WHERE dst_id = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Deleted %d dummy entries', 0, 1, @now, @rc) WITH NOWAIT
  END

  INSERT INTO #inserted
  SELECT l.timestamp, l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
  FROM StagedLOB AS l
  WHERE dst_id > 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Number of links for new LOBs = %d', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2
      SELECT '#inserted' AS '#inserted', * FROM #inserted
  END

  INSERT INTO @bs_priority
  SELECT timestamp, cid, id, priority
  FROM StagedBSPriority
  SELECT @err = @@ERROR, @rc_bspri = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Number of entries to set Business System object priority = %d', 0, 1, @now, @rc_bspri) WITH NOWAIT
  END

  IF @rc = 0 AND @rc_bspri = 0
    GOTO EndAtReturn

  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedLOBs
  ELSE
    SAVE TRANSACTION asisp_processStagedLOBs

  IF @rc = 0 AND @rc_bspri > 0
    GOTO ProcessBSPriority

  IF EXISTS (SELECT 1 FROM #inserted WHERE link_type = 'LOBL')
  BEGIN
    -- If the link type is LOBL, remove any records from the MenuItemCache table for the LOB on the
    -- source side of the links and their ancestors. We need to do this because the MenuItems for
    -- the higher level LOB objects depend on their descendents and the physical resources those
    -- descendents are linked to.
    DELETE FROM c
    FROM lob_link AS ll
      JOIN #inserted AS i ON i.src_id = ll.dst_id
      JOIN MenuItemCache AS c ON c.obj_cid = 'LOB' AND (c.obj_id = ll.src_id OR c.obj_id = i.src_id)
      JOIN MenuItemVersion AS v ON v.verno = c.verno
    WHERE i.link_type = 'LOBL'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Deleted %d row(s) from MenuItemCache', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- We do NOT set the IsShortcut flag on new LOBs because this is done in the tI_link trigger
    -- so that these LOBs will be seen with the correct flags values when they are first created.
    
    -- Set the HasShortcuts flag on the source LOBs
    UPDATE lob SET _flags = _flags | 0x00040000 /* HasShortcuts */
    FROM LineOfBusiness_V AS lob
    JOIN #inserted AS i ON i.dst_cid = lob.cid AND i.dst_id = lob.id
    WHERE i.link_type = 'LOBL'
      AND lob._flags & 0x00040000 = 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Set HasShortcuts flag on %d LineOfBusiness_V record(s)', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- If LOBC links are being created then send LinkCreated notifications for 
  -- shortcut-to-target's children links so that the new children also appear under the
  -- shortcuts.
  IF EXISTS (SELECT 1 FROM #inserted WHERE link_type = 'LOBC')
  BEGIN
    INSERT INTO PendingNotification (type, fields, detail)
    SELECT
      et.type,
      CAST(c.cno AS NVARCHAR(5))       + '|'
      + CAST(i.dst_id AS NVARCHAR(9))  + '|'
      + 'LOB'                          + '|'
                                       + '|'
      + 'LOBC'                         + '|'
      + CAST(p.cno AS NVARCHAR(5))     + '|'
      + CAST(sc.src_id AS NVARCHAR(9)), -- fields
      child.name -- detail
    FROM
      EventType AS et WITH (NOLOCK),
      #inserted AS i
      JOIN link AS sc WITH (NOLOCK) ON sc.dst_cid = i.src_cid AND sc.dst_id = i.src_id
      JOIN LineOfBusiness_V AS child WITH (NOLOCK) ON child.cid = i.dst_cid AND child.id = i.dst_id
      JOIN Meta..obj_class_table_BASE AS c WITH (NOLOCK) ON c.cid = i.dst_cid
      JOIN Meta..obj_class_table_BASE AS p WITH (NOLOCK) ON p.cid = sc.src_cid
    WHERE i.link_type  = 'LOBC'
      AND sc.link_type = 'LOBL'
      AND sc.src_cid   = 'LOB'
      AND sc.deleted   = 0
      AND et.name      = 'ObjectCreated'
    SET @err = @@ERROR
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Inserted %d ObjectCreated notification(s) for children of shortcuts', 0, 1, @now, @rc) WITH NOWAIT
    END
  END -- IF EXISTS (SELECT 1 FROM #inserted WHERE link_type = 'LOBC')
    
  -- Add linked LOBs to OpenEventCount table

  -- Insert counts for newly created LOBs by dereferencing LOBL link
  INSERT INTO #tempCount (id,       phy_cid,   phy_id,   nevents)
  SELECT                  i.src_id, i.dst_cid, i.dst_id, oec.nevents                 
  FROM #inserted AS i
    JOIN OpenEventCount AS oec WITH (NOLOCK) ON i.dst_cid = oec.cid
                                            AND i.dst_id  = oec.id
  WHERE i.link_type = 'LOBL'
    AND NOT EXISTS (SELECT * FROM OpenEventCount AS oec2 WITH (NOLOCK) WHERE oec2.cid = i.src_cid AND oec2.id = i.src_id)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Computed OpenEventCounts for %d new LOB(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Insert corresponding NoteDependent exception records (except for PBT records, which only apply to the 'owning' LOB)
  INSERT INTO NoteDependent ( noteid, depcid, depid, eventcid, eventid, status)
  SELECT                      nd.noteid, 'LOB', t.id, nd.eventcid, nd.eventid, nd.status
  FROM NoteDependent AS nd WITH (NOLOCK)
    JOIN #tempCount AS t ON nd.depcid = t.phy_cid AND nd.depid = t.phy_id
    JOIN Note_V AS note WITH (NOLOCK) ON note.id = nd.noteid
    JOIN Exception_C AS ex WITH (NOLOCK) ON nd.eventid = ex.id
  WHERE nd.eventcid = 'EXCP'
   AND  note.deleted = 0
   AND  note._Status = 'Open'
   AND  ex._flags & 16 = 0  --  exclude LOB-only exceptions
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Inserted %d row(s) into NoteDependent for non LOB-only exceptions', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Insert corresponding NoteDependent Message records (except for PBT records, which only apply to the 'owning' LOB)
  INSERT INTO NoteDependent ( noteid, depcid, depid, eventcid, eventid, status)
  SELECT                      nd.noteid, 'LOB', t.id, nd.eventcid, nd.eventid, nd.status
  FROM NoteDependent AS nd WITH (NOLOCK)
    JOIN #tempCount AS t ON nd.depcid = t.phy_cid AND nd.depid = t.phy_id
    JOIN Note_V AS note WITH (NOLOCK) ON note.id = nd.noteid
    JOIN Message_C AS ex WITH (NOLOCK) ON nd.eventid = ex.id
  WHERE nd.eventcid = 'MESG'
   AND  note.deleted = 0
   AND  note._Status = 'Open'
   AND  ex._flags & 16 = 0  --  exclude LOB-only messages
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Inserted %d row(s) into NoteDependent for non LOB-only messages', 0, 1, @now, @rc) WITH NOWAIT
  END
 
  -- PBT message counts get recalculated via CurrentMessageAlertStatePri triggers, so substract any message counts else
  -- they get counted twice
  UPDATE #tempCount
  SET nevents = nevents - 1
  FROM #tempCount AS oec
    JOIN CurrentMessageAlertStatePri AS mpri WITH (NOLOCK) ON oec.phy_cid = mpri.cid AND oec.phy_id = mpri.id
    JOIN Message_V AS m WITH (NOLOCK) ON m.id = mpri.mesg_id
  WHERE mpri.AlertState > 1 AND mpri.Priority < 5
    --AND m._flags & 16 > 0  -- only if a PBT message
    AND m.deleted = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Subtracted message counts for %d new LOB(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Subtract any PBT exceptions 
  UPDATE #tempCount
  SET nevents = nevents - counts.nEvents 
  FROM (SELECT DISTINCT ec._ParentCID AS cid, ec._ParentID AS id, COUNT(*)AS nEvents
        FROM  (SELECT DISTINCT phy_cid AS cid, phy_id AS id FROM #tempCount) AS t
          JOIN Exception_C AS ec WITH (NOLOCK) ON t.cid = ec._ParentCID AND t.id = ec._ParentID 
        WHERE ec._AlertState > 1 AND ec._Priority < 5 -- ensure that the alert state is greater than green and the priority is greater than ignore
          AND ec._flags & 16 > 0  -- only include PBT events
          AND ec.deleted = 0
        GROUP BY ec._ParentCID, ec._ParentID) AS counts
  WHERE phy_cid = counts.cid AND phy_id = counts.id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Subtracted exception counts for %d new LOB(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Create new OpenEventCount records
  INSERT INTO OpenEventCount (cid,   id,   nevents)
  SELECT                      'LOB', t.id, t.nevents
  FROM #tempCount AS t
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Inserted %d row(s) into OpenEventCount', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Set the flags where needed 
  DECLARE opencur CURSOR FOR
    SELECT DISTINCT i.phy_cid, o.cname
    FROM #tempCount AS i
      JOIN Meta..obj_class_table_BASE AS o WITH (NOLOCK) ON o.cid = i.phy_cid
  OPEN opencur
  FETCH NEXT FROM opencur INTO @cid, @cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- Turn on the OpenEvents flag or All Events Owned flag if on in the physical resource
    EXEC(
    'UPDATE l SET _flags = l._flags | (o._flags & 1536 /*OpenEvents|AllEventsOwned*/) ' +
    'FROM ' + @cname + '_V AS o ' + 
      'JOIN #tempCount i ON i.phy_cid = ''' + @cid + ''' AND i.phy_id = o.id ' +
      'JOIN LineOfBusiness_A l on i.id = l.id ' 
    )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      DEALLOCATE opencur
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Updated %d LOBs linked to resource of class id %s', 0, 1, @now, @rc, @cid) WITH NOWAIT
    END

    FETCH NEXT FROM opencur INTO @cid, @cname
  END
  DEALLOCATE opencur

  -- CRL processing
  -- Lookup id of CRL_Container LOB
  SELECT @crl_container_id = id FROM CRL_Container
  IF @crl_container_id IS NOT NULL
  BEGIN
  
    SET @IsCRL = 0
    SET @IsInCRL = 0
    
    -- See if the src_id=0 record for the CRL_Container LOB is being created
    IF EXISTS (SELECT 1 FROM #inserted AS i WHERE link_type = 'LOBC' AND src_cid = 'LOBC' AND dst_cid = 'LOB' AND dst_id = @crl_container_id)
    BEGIN
      SET @IsCRL = 1
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedLOBs: CRL_Container LOB %d being inserted', 0, 1, @now, @crl_container_id) WITH NOWAIT
      END
    END

    SET @InCRL = 0x00080000 /* InCRL */
    SET @AutoCopyEVService = 0x00010000 /* AutoCopyEVService */

    -- Make sure the CRL_Container LOB has the InCRL bit set, and also turn off the AutoCopyEVService flag if set
    IF @IsCRL = 1
    BEGIN
      UPDATE lob SET _flags = (_flags | @InCRL) & ~@AutoCopyEVService
      FROM LineOfBusiness_V AS lob
      WHERE lob.id = @crl_container_id AND _flags <> (_flags | @InCRL) & ~@AutoCopyEVService
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        ROLLBACK TRANSACTION asisp_processStagedLOBs
        RETURN
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedLOBs: set InCRL flag for CRL_Container LOB. # rows affected = %d', 0, 1, @now, @rc) WITH NOWAIT
      END
    END -- Make sure the CRL_Container LOB has the InCRL bit set, and also turn off the AutoCopyEVService flag if set
      
    -- If any records are being inserted where the CRL_Container is the src_id then any LOB on the
    -- dst_id side must be marked with the InCRL flag
    INSERT INTO @lob
      SELECT DISTINCT ll.dst_id, ll.gen, 0
      FROM lob_link AS ll WITH (NOLOCK)
      JOIN #inserted AS i ON i.dst_id = ll.dst_id
      WHERE ll.src_id   = @crl_container_id
        AND i.link_type = 'LOBC'
        AND i.src_cid   = 'LOB'
        AND i.dst_cid   = 'LOB'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: %d rows for LOBs under the CRL container', 0, 1, @now, @rc) WITH NOWAIT
    END

    SET @gen = 1
    SELECT @max_gen = MAX(gen) FROM @lob
    WHILE @gen <= @max_gen
    BEGIN
      UPDATE u SET inCRL = 1
      FROM @lob AS u
      JOIN link AS l WITH (NOLOCK) ON l.dst_id = u.dst_id
      JOIN LineOfBusiness_A AS lob WITH (NOLOCK) ON lob.id = l.src_id
      WHERE l.dst_cid = 'LOB'
        AND l.src_cid = 'LOB'
        AND l.link_type = 'LOBC'
        AND lob._flags & @InCRL = @InCRL
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        ROLLBACK TRANSACTION asisp_processStagedLOBs
        RETURN
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedLOBs: marked %d LOBs in CRL at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
      END

      IF @rc = 0
        BREAK

      SET @gen = @gen + 1
    END

    IF EXISTS (SELECT 1 FROM @lob WHERE inCRL = 1)
    BEGIN    
      UPDATE lob SET _flags = (lob._flags | @InCRL) & ~@AutoCopyEVService
      FROM LineOfBusiness_V AS lob
      JOIN @lob AS u ON u.dst_id = lob.id
      WHERE u.inCRL = 1
        AND lob._flags <> (lob._flags | @InCRL) & ~@AutoCopyEVService
    END
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: updated flags on %d LOBs', 0, 1, @now, @rc) WITH NOWAIT
    END
  END -- CRL processing

  -- Create PropagationSlot entries for LOBs linked to objects that already have their own PropagationSlot entries
  INSERT INTO @newps
    SELECT DISTINCT 'LOB', i.src_id, i.dst_cid, i.dst_id
    FROM #inserted AS i
      JOIN PropagationSlot AS p WITH (NOLOCK) ON p.OwnerCID = i.dst_cid AND p.OwnerID = i.dst_id
      LEFT OUTER JOIN PropagationSlot AS prev WITH (NOLOCK) ON prev.OwnerCID = i.src_cid AND prev.OwnerID = i.src_id
        AND prev.MatrixIndex = 0 AND prev.SettingIndex = 0 AND prev.ArrayIndex = 2
    WHERE i.link_type = 'LOBL'
      AND i.src_cid = 'LOB'
      AND p.SettingIndex = 0
      AND prev.id IS NULL
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  
  IF @rc > 0
  BEGIN
    IF @verbose >= 2
      SELECT '@newps' AS '@newps', * FROM @newps
    INSERT INTO PropagationSlot
    (
      OwnerCID,
      OwnerID,
      MatrixIndex,
      SettingIndex,
      ArrayIndex,
      ResourceCID,
      ResourceID
    )
    SELECT
      n.OwnerCID,
      n.OwnerID,
      p.MatrixIndex,
      0,
      p.ArrayIndex,
      n.ResourceCID,
      n.ResourceID
    FROM @newps AS n,
      PropagationSlot AS p WITH (NOLOCK)
    WHERE p.OwnerCID = 'LOB'
      AND p.OwnerID = 0
      AND p.SettingIndex = 1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Inserted %d rows into PropagationSlot', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- Initialize the exception PropagationSlot records for the new LOBs
  -- NOTE: We are relying on the lob_link insert trigger to update the ResourceCID and ResourceID
  -- of the new LOBs' exception PropagationSlots.
  -- The join works this way:
  --      PropagationSlot:  OwnerCID    OwnerID   ResourceCID ResourceID MatrixIndex SettingIndex ArrayIndex Critical,High,Medium,Low,Ignore
  --                         |           |         |           |          |            |           |          ^
  --                         v           v         |           |          |            |           |          |
  --      inserted:         src_cid     src_id     |           |          |            |           |          |
  --                                               v           v          v            v           v          |
  --      PropagationSlot:                        OwnerCID    OwnerID    MatrixIndex SettingIndex ArrayIndex Critical,High,Medium,Low,Ignore

  INSERT INTO @ecount
    SELECT DISTINCT p.id, e._Priority, ISNULL(SUM(e._nEXCH),0)
    FROM #inserted AS i
      JOIN Exception_V AS e ON e._ParentCID = i.dst_cid AND e._ParentID = i.dst_id
      JOIN PropagationSlot AS p ON p.OwnerCID = i.src_cid AND p.OwnerID = i.src_id AND p.ArrayIndex = e._AlertState
    WHERE i.link_type = 'LOBL'
      AND e.deleted = 0
      AND e._flags & 16 = 0 -- exclude LOB-only exceptions
      AND p.MatrixIndex = 1 -- exceptions
      AND p.SettingIndex = 0 -- counts
    GROUP BY p.id, e._Priority
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Computed %d distinct Exception counter update(s)', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2 SELECT '@ecount', * FROM @ecount
  END

  UPDATE p SET
    Critical = ISNULL((SELECT e.nEXCH FROM @ecount AS e WHERE e.id = p.id AND e.Priority = 1), 0),
    High     = ISNULL((SELECT e.nEXCH FROM @ecount AS e WHERE e.id = p.id AND e.Priority = 2), 0),
    Medium   = ISNULL((SELECT e.nEXCH FROM @ecount AS e WHERE e.id = p.id AND e.Priority = 3), 0),
    Low      = ISNULL((SELECT e.nEXCH FROM @ecount AS e WHERE e.id = p.id AND e.Priority = 4), 0),
    Ignore   = ISNULL((SELECT e.nEXCH FROM @ecount AS e WHERE e.id = p.id AND e.Priority = 5), 0),
    mtime    = CURRENT_TIMESTAMP
  FROM PropagationSlot AS p
    JOIN @ecount AS e ON e.id = p.id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Number of PropagationSlot updates for new LOBs: %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Initialize the CurrentMessageAlertStatePri records for the new LOBs
  INSERT INTO CurrentMessageAlertStatePri (cid, id, AlertState, Priority, mesg_id)
    SELECT i.src_cid, i.src_id, p.AlertState, p.Priority, p.mesg_id
    FROM #inserted AS i
      JOIN lob_link AS ll ON ll.dst_id = i.src_id
      JOIN CurrentMessageAlertStatePri AS p ON p.cid = ll.phy_cid AND p.id = ll.phy_id
      JOIN Message_V AS m WITH (NOLOCK) ON m.id = p.mesg_id -- safe to use NOLOCK because previous term has read lock
                                                            -- on CurrentMessageAlertStatePri and we're just looking
                                                            -- up the _flags value from Message_V, which does not
                                                            -- change after creation.
      LEFT OUTER JOIN CurrentMessageAlertStatePri AS prev ON prev.cid = i.src_cid
                                                         AND prev.id  = i.src_id
    WHERE i.link_type = 'LOBL'
      AND ll.src_id   = 0
      AND m._flags & 16 = 0 -- exclude LOB-only messages
      AND prev.cid IS NULL  -- no previously existing CurrentMessageAlertStatePri (happens when message arrives before LOB initialization)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Number of CurrentMessageAlertStatePri inserts for new LOBs: %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Set the default LOB attributes based on the linked resource
  INSERT INTO @lobl_inserted
    SELECT src_id, dst_cid, dst_id
    FROM #inserted
    WHERE link_type = 'LOBL' AND src_cid = 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Number of new LOBL lob_link records: %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  DECLARE cur_ti_link_lobl CURSOR FOR
    SELECT DISTINCT dst_cid, oc.cname
    FROM @lobl_inserted AS i
    JOIN obj_class AS oc ON oc.cid = i.dst_cid
  OPEN cur_ti_link_lobl
  FETCH NEXT FROM cur_ti_link_lobl INTO @obj_cid, @cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @sql =
    'UPDATE lob SET ' +
      'name = CASE WHEN ISNULL(lob.name,'''') = '''' THEN ISNULL(o.name,'''') ELSE lob.name END '
    IF @obj_cid = 'LOB'
      SET @sql = @sql + ', nchild = o.nchild '
    SET @sql = @sql +
      'FROM LineOfBusiness_C AS lob ' +
      'JOIN #inserted AS i ON i.src_id = lob.id ' +
      'JOIN ' + @cname + '_V AS o ON o.cid = i.dst_cid AND o.id = i.dst_id ' + 
      'WHERE i.link_type = ''LOBL'''
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      DEALLOCATE cur_ti_link_lobl
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: [%s]: %d row(s) affected', 0, 1, @now, @sql, @rc) WITH NOWAIT
    END
    
    SET @sql =
    'UPDATE lob SET  ' +
        '_Desc = CASE WHEN ISNULL(lob._Desc,'''') = '''' THEN ISNULL(o._Desc,'''') ELSE lob._Desc END ' +         -- IF no desc      THEN default to linked resource's desc
        ', _PriorityID = CASE WHEN o._PriorityID <> 5 OR '''+@obj_cid+''' = ''LOB'' THEN o._PriorityID ELSE lob._PriorityID END ' +   -- IF linked resource's _PriorityID is NOT Ignore OR Shortcut LOB THEN default to linked resource's _PriorityID
        ', _flags = lob._flags | (o._flags & 0x000000FF) ' -- Inherit Problem, Automation, and Change flags
    IF @obj_cid = 'LOB'
      SET @sql = @sql + 
        ', _DesiredStateID = o._DesiredStateID ' +           -- IF Shortcut LOB THEN default to Target LOB _DesiredStateID '
        ', _Weight = o._Weight '
    SET @sql = @sql +
      'FROM LineOfBusiness_A AS lob ' +
      'JOIN #inserted AS i ON i.src_id = lob.id  ' +
      'JOIN ' + @cname + '_V AS o ON i.dst_cid = o.cid AND i.dst_id = o.id ' + 
      'WHERE i.link_type = ''LOBL'''
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      DEALLOCATE cur_ti_link_lobl
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: [%s]: %d row(s) affected', 0, 1, @now, @sql, @rc) WITH NOWAIT
    END

    FETCH NEXT FROM cur_ti_link_lobl INTO @obj_cid, @cname
  END
  DEALLOCATE cur_ti_link_lobl

  -- Insert PendingNotification records for flag updates that set the Problem, Automation, or Change flags
  INSERT INTO PendingNotification (type, fields, detail)
  SELECT
    et.type,
    CAST(oc.cno AS NVARCHAR(5)) + '|' +
    CAST(lob.id AS NVARCHAR(9)) + '|' +
    CAST(m.id AS NVARCHAR(9)) AS fields,
    CAST(lob._flags AS NVARCHAR(9)) AS detail
  FROM
    #inserted AS i
    JOIN LineOfBusiness_V AS lob WITH (NOLOCK) ON lob.id = i.src_id
    JOIN Meta..obj_class_table_BASE AS oc WITH (NOLOCK) ON oc.cid = lob.cid,
    EventType AS et WITH (NOLOCK),
    id_string_map AS m WITH (NOLOCK)
  WHERE i.link_type = 'LOBL'
    AND lob._flags & 0x000000ff <> 0
    AND et.name = 'ObjAttrUpdated'
    AND m.string = 'flags'    
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Inserted %d PendingNotification records for updates to Problem, Automation, and Change flags', 0, 1, @now, @rc) WITH NOWAIT
  END
  
  -- For new Shortcut LOBs, copy Target's PBTUsedBy entries
  IF EXISTS(SELECT 1 FROM #inserted WHERE src_cid = 'LOB' AND dst_cid = 'LOB')
  BEGIN
    INSERT INTO PBTUsedBy (pbt_id, obj_cid, obj_id, enabled, linked_id)
      SELECT DISTINCT t_pbt.pbt_id, i.src_cid, i.src_id, t_pbt.enabled, t_pbt.id
        FROM #inserted  AS i
        JOIN PBTUsedBy AS t_pbt ON t_pbt.obj_cid = i.dst_cid AND t_pbt.obj_id = i.dst_id   -- target's PBT entries
        WHERE i.src_cid = 'LOB' AND i.dst_cid = 'LOB'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Number of PBTUsedBy records created for new shortcut LOBs: %d', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- Automatically Register CICS FILE and TRAN objects when they are placed into a LOB.
  IF EXISTS(SELECT 1 FROM #inserted WHERE dst_cid IN ('FILE','TRAN') )
  BEGIN
    DECLARE cur_ti_link_lobl CURSOR FOR
      SELECT DISTINCT dst_cid, dst_id 
        FROM #inserted 
        WHERE dst_cid IN ('FILE','TRAN')
    OPEN cur_ti_link_lobl
    FETCH NEXT FROM cur_ti_link_lobl INTO @dst_cid, @dst_id
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC @rc = asisp_registerobjects @dst_cid, @dst_id
      IF @rc < 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_processStagedLOBs: Error %d from asisp_registerobjects ''%s'', %d', 3, 1, @now, @rc, @dst_cid, @dst_id) WITH NOWAIT
      END
      FETCH NEXT FROM cur_ti_link_lobl INTO @dst_cid, @dst_id
    END
    DEALLOCATE cur_ti_link_lobl
  END

  -- The src_cid/src_id columns of the inserted rows are joined to PBTUsedBy obj_cid/obj_id.
  -- PBTUsedBy is joined to PercentageBasedThreshold to filter-out PBTs that are not enabled.
  INSERT INTO StagedPBT ( id )
    SELECT DISTINCT u.id
    FROM #inserted AS i
    JOIN PBTUsedBy AS u ON u.obj_cid = i.src_cid AND u.obj_id = i.src_id
    JOIN PercentageBasedThreshold AS p ON p.id = u.pbt_id
    WHERE i.link_type = 'LOBC'
      AND i.src_cid   = 'LOB'
      AND i.dst_cid   = 'LOB'
      AND u.enabled   = 1
      AND p.enabled   = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Inserted %d row(s) into StagedPBT: %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- The src_cid/src_id columns of the inserted rows are joined to lob_link to get shorcut
  -- LOBs that need PBT evaluations
  INSERT INTO StagedPBT ( id )
    SELECT DISTINCT u.id
    FROM #inserted AS i
    JOIN lob_link  AS l ON l.phy_cid = i.src_cid AND l.phy_id = i.src_id
    JOIN PBTUsedBy AS u ON u.obj_id = l.dst_id
    JOIN PercentageBasedThreshold AS p ON p.id = u.pbt_id
    WHERE i.link_type = 'LOBC'
      AND i.src_cid   = 'LOB'
      AND i.dst_cid   = 'LOB'
      AND u.enabled   = 1
      AND p.enabled   = 1
      AND l.src_id    = 0
      AND u.obj_cid = 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Inserted %d row(s) into StagedPBT for BS Shortcuts: %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  EXEC @rc = asisp_maintTask @verbose = @verbose
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: asisp_maintTask finished with return code %d', 0, 1, @now, @rc) WITH NOWAIT
  END

  DELETE FROM s FROM StagedLOB AS s JOIN #inserted AS i ON i.timestamp = s.timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedLOBs
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Deleted %d processed row(s) from StagedLOB', 0, 1, @now, @rc) WITH NOWAIT
  END

ProcessBSPriority:
  IF @rc_bspri > 0
  BEGIN
    UPDATE lob SET _PriorityID = bsp.priority
    FROM LineOfBusiness_V AS lob 
      JOIN @bs_priority AS bsp ON bsp.cid = lob.cid AND bsp.id = lob.id 
      WHERE lob._PriorityID <> bsp.priority
    SELECT @err = @@ERROR, @rc_bspri = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Updated PriorityID (object priority) of %d LOBs', 0, 1, @now, @rc_bspri) WITH NOWAIT
    END

    DELETE FROM s FROM StagedBSPriority AS s JOIN @bs_priority AS i ON i.timestamp = s.timestamp
    SELECT @err = @@ERROR, @rc_bspri = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedLOBs
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedLOBs: Deleted %d processed row(s) from StagedBSPriority', 0, 1, @now, @rc_bspri) WITH NOWAIT
    END    
  END

EndOfProc:
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedLOBs: Finished. rc=%d', 0, 1, @now, @rc) WITH NOWAIT
  END
  
  IF @tcount = 0
    COMMIT TRANSACTION asisp_processStagedLOBs

EndAtReturn:
  RETURN @rc
GO
