-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File:archive\db\sql\procs\cl_processmethods.sql
-- SID: 1.18
-- Date Modified: 04/10/18 11:56:02
-- 
-- Module Description
--   This proc is used by CL to process and run methods
--
USE Object
GO

-------------------------------------------------------------------------------
-- cl_processMethods
-------------------------------------------------------------------------------
PRINT 'Creating procedure cl_processMethods'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'cl_processMethods' AND type = 'P')
  DROP PROCEDURE cl_processMethods
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File:archive\db\sql\procs\cl_processmethods.sql
-- SID: 1.18
-- Date Modified: 04/10/18 11:56:02
--
-- This procedure is used by Common Listener to invoke a set of methods 
-- The supported list of methods are SetThresholds, SetSendStateChangeEvent, DefineAndUsePBTs, 
-- UsePBTs, DeletePBTs, SetWeights, DefineServicesAndLinkToEvUsers, LinkEVUsersToServices,
-- and UnLinkEVServices
-- 
-- This procedure takes it input from #events table used by Common Listener
-- CREATE TABLE #events 
-- (
--   evtid   INT           NOT NULL,
--   root    NVARCHAR(512) NULL,
--   keyword NVARCHAR(64)  NULL,
--   value   NVARCHAR(512) NULL
-- )
--
-- Description of procedure arguments:
--   @trid         Common Listener transaction ID
--   @verbose >= 1 will execute this procedure in verbose mode 

CREATE PROCEDURE cl_processMethods
  @trid		  INT,
  @type     TINYINT = 0,
  @verbose	TINYINT = 0
AS
  SET NOCOUNT ON
  DECLARE @now NVARCHAR(40), @rc INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: cl_processMethods: @trid=%d, @type=%d, @verbose=%d', 0, 1, @now, @trid, @type, @verbose) WITH NOWAIT
  END

  -- Table used by methods to specify input list of resources
  CREATE TABLE #Instances (
    cid        ClassID        NULL,
    id         ObjID          NULL,
    rCorr      INT            NOT NULL,             
    className  NVARCHAR(24)   NULL,
    prodVer    NVARCHAR(85)   NULL,     
    instid     NVARCHAR(425)  NULL,
    uinstid    NVARCHAR(425)  NULL,
    objPath    NVARCHAR(1375) NULL,
    maxRes     INT            NULL,
    errorId    INT            NULL
  ) 

  -- Table used to get resource data from CL stage tables
  CREATE TABLE #tmp_Instances (
    rCorr      INT            NOT NULL,             
    root       NVARCHAR(64)   NULL,          
    id         ObjID          NULL,
    className  NVARCHAR(24)   NULL,
    prodVer    NVARCHAR(85)   NULL,     
    instid     NVARCHAR(425)  NULL,
    uinstid    NVARCHAR(425)  NULL,
    objPath    NVARCHAR(1375) NULL,
    maxRes     INT            NULL
  ) 

  -- Table to store CL status messages from method
  CREATE TABLE #cl_status (
    evtid   INT           NULL,
    disp    NVARCHAR(255) NULL
  )

  -- Populate #tmp_Instances with resource data from CL stage tables
  INSERT INTO #tmp_Instances (rCorr, root)
    SELECT DISTINCT evtid, root FROM #events WHERE root LIKE 'method:inst%'
  SET @rc = @@ROWCOUNT

  -- Return if there are no method instances
  IF @rc = 0 
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: There are no method definitions for this transaction (trID=%d)', 0, 1, @trid) WITH NOWAIT
    END

    RETURN
  END

  -- Continue population of #tmp_Instances with resource data from CL stage tables
  UPDATE ti SET id = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword = 'oid'
  UPDATE ti SET className = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword = 'class'
  UPDATE ti SET prodVer = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword='prodver'
  UPDATE ti SET instid = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword='instid'
  UPDATE ti SET instid = ti.instid+'('+e.value+')' FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword='uid' AND e.value LIKE '%:%' AND ti.instid IS NOT NULL
  UPDATE ti SET uinstid = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword='uinstid'
  UPDATE ti SET objPath = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword IN ('objpath', 'upath')
  UPDATE ti SET maxRes = e.value FROM #tmp_Instances ti
    JOIN #events e ON ti.rCorr=e.evtid AND ti.root=e.root
    WHERE e.keyword='nflag'


  IF @type IN (0, 1)
  BEGIN
    -----------------------------------------------------------------
    -- Method: SetThresholds
    -----------------------------------------------------------------

    -- Table used by SetThresholds method to specify input data
    CREATE TABLE #SetThresholdParms (
      rCorr        INT            NOT NULL,             
      eventType    NVARCHAR(32)   NULL,  
      alertState   NVARCHAR(32)   NULL,
      priority     NVARCHAR(32)   NULL,
      value        INT            NULL, 
      errorId      INT            NULL,
      root         NVARCHAR(64)   NULL
    ) 
 
    -- Load and Process SetThresholds method
    INSERT INTO #SetThresholdParms (rCorr, root)
      SELECT DISTINCT evtid, root FROM #events
      WHERE root LIKE 'method:SetThresholds1:ThresholdObjects%'  
    SET @rc = @@ROWCOUNT
  
    IF @rc > 0
    BEGIN
      UPDATE ts SET eventType = e.value FROM #SetThresholdParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventType'
      UPDATE ts SET alertState = e.value FROM #SetThresholdParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'AlertState'
      UPDATE ts SET priority = e.value FROM #SetThresholdParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'Priority'
      UPDATE ts SET value = e.value FROM #SetThresholdParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'Value'

      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #SetThresholdParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetThresholds: About to invoke asisp_loadSetThresholds method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadSetThresholds @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'SetThresholds: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetThresholds: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: SetWeights
    -----------------------------------------------------------------

    -- Table used by SetWeights method to specify input data
    CREATE TABLE #NewWeights
    (
      rCorr     INT      NULL,
      NewWeight SMALLINT NULL,
      error     INT      NULL
    )
   
    -- Load and Process SetWeights method
    -- Changed root = 'method:SetWeights1' to LIKE below as ABS code doesnt guarantee a 1 here
    INSERT INTO #NewWeights (rCorr, NewWeight)
      SELECT evtid, value FROM #events
      WHERE root LIKE 'method:SetWeights%' AND keyword = 'NewWeight'  
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #NewWeights)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetWeights: About to invoke asisp_loadSetWeights method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadSetWeights @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'SetWeights: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetWeights: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: DefneAndUsePBTs
    -----------------------------------------------------------------

    -- Table used by DefineAndUsePBTs method to specify input data
    CREATE TABLE #DefAndUsePBTParms
    (
      rCorr            INTEGER       NULL,
      PBTID            INTEGER       NULL,  
      PBTName          NVARCHAR(255) NULL,  
      Salience         INTEGER       NULL,  
      Enabled          NVARCHAR(32)  NULL,  
      MinAlertState    NVARCHAR(32)  NULL,  
      MaxAlertState    NVARCHAR(32)  NULL,  
      MinPercentage    INTEGER       NULL,  
      MaxPercentage    INTEGER       NULL,  
      EventType        NVARCHAR(32)  NULL,  
      EventName        NVARCHAR(255) NULL,  
      EventText        NVARCHAR(255) NULL,  
      EventAlertState  NVARCHAR(32)  NULL,  
      EventPriority    NVARCHAR(32)  NULL,  
      EventFlags       INTEGER       NULL,  
      EventState       NVARCHAR(255) NULL,  
      error            INTEGER       NULL,
      root             NVARCHAR(64)  NULL
    )
   
    -- Load and Process DefineAndUsePBTs method
    INSERT INTO #DefAndUsePBTParms (rCorr, root)
      SELECT DISTINCT evtid, root FROM #events
      WHERE root LIKE 'method:DefineAndUsePBTs1:PBTsToDefine%'  
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      UPDATE ts SET PBTID = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'PBTID' 
      UPDATE ts SET PBTName = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'PBTName'
      UPDATE ts SET Salience = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'Relevance'
      UPDATE ts SET Enabled = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'Enabled'
      UPDATE ts SET MinAlertState = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'MinAlertState' 
      UPDATE ts SET MaxAlertState = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'MaxAlertState'
      UPDATE ts SET MinPercentage = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'MinPercentage'
      UPDATE ts SET MaxPercentage = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'MaxPercentage'
      UPDATE ts SET EventType = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventType' 
      UPDATE ts SET EventName = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventName'
      UPDATE ts SET EventText = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventText'
      UPDATE ts SET EventAlertState = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventAlertState'
      UPDATE ts SET EventPriority = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventPriority' 
      UPDATE ts SET EventFlags = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventFlags'
      UPDATE ts SET EventState = e.value FROM #DefAndUsePBTParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'EventState'

      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #DefAndUsePBTParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'DefineAndUsePBTs: About to invoke asisp_loadDefineAndUsePBTs method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadDefineAndUsePBTs @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'DefineAndUsePBTs: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'DefineAndUsePBTs: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: UsePBTs
    -----------------------------------------------------------------

    -- Table used by UsePBTs method to specify input data
    CREATE TABLE #UsePBTsParms (
      rCorr     INTEGER       NULL,
      pbtId     INTEGER       NULL,     
      pbtName   NVARCHAR(255) NULL,     
      action    NVARCHAR(10)  NULL,     
      errorId   INTEGER       NULL,
      root      NVARCHAR(64)  NULL    
    )
   
    -- Load and Process UsePBTs method
    INSERT INTO #UsePBTsParms (rCorr, root)
      SELECT DISTINCT evtid, root FROM #events
      WHERE root LIKE 'method:UsePBTs1:PBTs%'  
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      UPDATE ts SET pbtId = e.value FROM #UsePBTsParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'PBTID'
      UPDATE ts SET pbtName = e.value FROM #UsePBTsParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'PBTName'
      UPDATE ts SET action = e.value FROM #UsePBTsParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'Action'

      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #UsePBTsParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'UsePBTs: About to invoke asisp_loadUsePBTs method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadUsePBTs @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'UsePBTs: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'UsePBTs: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: DeletePBTs
    -----------------------------------------------------------------

    -- Table used by DeletePBTs method to specify input data
    CREATE TABLE #PBTsToDelete
    (
      PBTID   INT           NULL,
      PBTName NVARCHAR(255) NULL,
      error   INT           NULL
    )

    -- Table used to get DeletePBTs method data from CL stage tables
    CREATE TABLE #tmp_PBTsToDelete (
      rCorr        INT            NOT NULL,             
      root         NVARCHAR(64)   NULL,
      PBTID        INT            NULL,
      PBTName      NVARCHAR(255)  NULL,
      error        INT            NULL
    ) 
   
    -- Load and Process DeletePBTs method
    INSERT INTO #tmp_PBTsToDelete (rCorr, root)
      SELECT DISTINCT evtid, root FROM #events
      WHERE root LIKE 'method:DeletePBTs1:PBTsToDelete%' 
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      UPDATE ts SET PBTID = e.value FROM #tmp_PBTsToDelete ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'PBTID'
      UPDATE ts SET PBTName = e.value FROM #tmp_PBTsToDelete ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'PBTName'

      INSERT INTO #PBTsToDelete (PBTID, PBTName)
        SELECT PBTID, PBTName
        FROM #tmp_PBTsToDelete

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'DeletePBTs: About to invoke _DeletePBTs method', CURRENT_TIMESTAMP

      EXEC @rc = _DeletePBTs @cid=NULL, @id=NULL, @ExecMode=6

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status 
        SELECT 
          ts.rCorr, 
          sm.description + '(ErrorID = ' + CAST(s.error AS NVARCHAR(9)) + ') (' + CAST(s.PBTID AS NVARCHAR(9)) + '/' + s.PBTName + ')'
        FROM #tmp_PBTsToDelete ts
        JOIN #PBTsToDelete s ON s.PBTID=ts.PBTID OR s.PBTName=ts.PBTName
        JOIN master..sysmessages sm ON sm.error = s.error
        WHERE s.error IS NOT NULL AND s.error <> 0
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'DeletePBTs: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'DeletePBTs: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END
  END


  IF @type IN (0, 2)
  BEGIN
    -----------------------------------------------------------------
    -- Method: SetSendStateChangeEvent
    -----------------------------------------------------------------

    -- Table used by SetSendStateChangeEvent method to specify input data
    CREATE TABLE #SetASChangeEventParms (
      rCorr    INT            NOT NULL,             
      flag     NVARCHAR(32)   NULL,
      errorId  INT            NULL 
    ) 

    -- Load and Process SetSendStateChangeEvent method
    -- Changed root = 'method:SetSendStateChangeEvent1' to LIKE below as ABS code doesnt guarantee a 1 here
    INSERT INTO #SetASChangeEventParms (rCorr, flag)
      SELECT evtid, value FROM #events
      WHERE root LIKE 'method:SetSendStateChangeEvent%' AND keyword = 'StateChange'
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #SetASChangeEventParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetSendStateChangeEvent: About to invoke asisp_loadSetASChangeEvent method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadSetASChangeEvent @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'SetSendStateChangeEvent: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetSendStateChangeEvent: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: SetBSPriority
    -----------------------------------------------------------------

    -- Table used by SetBSPriority method to specify input data
    CREATE TABLE #BSPriorityParms
    (
      rCorr       INT 	        NULL,
      newValue    NVARCHAR(32)  NULL,
      error       INT 	        NULL
    )

    -- Load and Process SetBSPriority method
    -- Changed root = 'method:SetBSPriority1' to LIKE below as ABS code doesnt guarantee a 1 here
    INSERT INTO #BSPriorityParms (rCorr, newValue)
      SELECT evtid, value FROM #events
      WHERE root LIKE 'method:SetBSPriority%' AND keyword = 'NewPriority'  
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #BSPriorityParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetBSPriority: About to invoke asisp_loadSetBSPriority method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadSetBSPriority @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'SetBSPriority: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'SetBSPriority: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: DefineServicesAndLinkToEvUsers
    -----------------------------------------------------------------

    -- Tables used by DefineServicesAndLinkToEvUsers method to specify input data
    CREATE TABLE #EvServiceParms (
      rCorr        INT           NOT NULL,             
      displayName  NVARCHAR(54)  NULL,  
      serviceId    NVARCHAR(63)  NULL,  
      businessRole NVARCHAR(500) NULL,  
      redImpact    NVARCHAR(500) NULL,  
      yellowImpact NVARCHAR(500) NULL,  
      flags        INT           NULL,  
      errorId      INT           NULL, 
      root         NVARCHAR(64)  NULL
    ) 
    CREATE TABLE #EvUsersParms (
      rCorr   INT           NOT NULL,             
      userId  NVARCHAR(255) NULL,
      errorId INT           NULL
    ) 
   
    -- Load and Process DefineServicesAndLinkToEvUsers method
    INSERT INTO #EvServiceParms (rCorr, root)
      SELECT DISTINCT evtid, root FROM #events
      WHERE root LIKE 'method:DefineServicesAndLinkToEvUsers1:ServiceDef%'  
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      UPDATE ts SET displayName = e.value FROM #EvServiceParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'DisplayName'
      UPDATE ts SET serviceId = e.value FROM #EvServiceParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'ServiceID'
      UPDATE ts SET businessRole = e.value FROM #EvServiceParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'BusinessRole'
      UPDATE ts SET redImpact = e.value FROM #EvServiceParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'RedImpact'
      UPDATE ts SET yellowImpact = e.value FROM #EvServiceParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'YellowImpact'
      UPDATE ts SET flags = e.value FROM #EvServiceParms ts
        JOIN #events e ON ts.rCorr=e.evtid AND ts.root=e.root
        WHERE e.keyword = 'Flags'

      INSERT INTO #EvUsersParms (rCorr, userId)
        SELECT evtid, value FROM #events
        WHERE root LIKE 'method:DefineServicesAndLinkToEvUsers1:UserIDDef%' AND keyword = 'UserID'

      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #EvServiceParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'DefineServicesAndLinkToEvUsers: About to invoke asisp_loadDefSvcLinkUsers method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      EXEC @rc = asisp_loadDefSvcLinkUsers @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'DefineServicesAndLinkToEvUsers: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'DefineServicesAndLinkToEvUsers: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: UnLinkEVServices
    -----------------------------------------------------------------
    
    TRUNCATE TABLE #EvUsersParms

    -- Load and Process UnLinkEVServices method
    INSERT INTO #EvUsersParms (rCorr, userId)
      SELECT evtid, value FROM #events
      WHERE root LIKE 'method:UnLinkEVServices1:EVUsers%' AND keyword = 'UserID'
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #EvUsersParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'UnLinkEVServices: About to invoke asisp_loadUnLinkEvServices method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadUnLinkEvServices @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'UnLinkEVServices: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'UnLinkEVServices: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END

    -----------------------------------------------------------------
    -- Method: LinkEVUsersToServices
    -----------------------------------------------------------------

     TRUNCATE TABLE #EvUsersParms

    -- Load and Process LinkEVUsersToServices method
    INSERT INTO #EvUsersParms (rCorr, userId)
      SELECT evtid, value FROM #events
      WHERE root LIKE 'method:LinkEVUsersToServices1:EVUsers%' AND keyword = 'UserID'
    SET @rc = @@ROWCOUNT
    
    IF @rc > 0
    BEGIN
      TRUNCATE TABLE #Instances
      INSERT INTO #Instances (id, rCorr, className, prodVer, instid, uinstid, objPath, maxRes)
        SELECT id, ti.rCorr, className, prodVer, instid, uinstid, objPath, maxRes FROM #tmp_Instances ti 
        WHERE rCorr IN (SELECT rCorr FROM #EvUsersParms)

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'LinkEVUsersToServices: About to invoke asisp_loadLinkEvUsersToServices method', CURRENT_TIMESTAMP

      TRUNCATE TABLE #cl_status
      INSERT INTO #cl_status EXEC @rc = asisp_loadLinkEvUsersToServices @outformat = 1, @verbose=@verbose

      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, evtid, 'WARN/ERR', 'LinkEVUsersToServices: ' + disp, CURRENT_TIMESTAMP 
        FROM #cl_status WHERE disp IS NOT NULL
      INSERT INTO CL_Status (trId, evDataId, status, disposition, time)
        SELECT @trid, 0, 'OK', 'LinkEVUsersToServices: Finished invoking method (rc = ' + CAST(@rc AS NVARCHAR(9)) + ')', CURRENT_TIMESTAMP
    END
  END

GO
