-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/pa_updateobjectpriority.sql
-- SID: 1.11.4.5
-- Date Modified: 04/10/27 13:15:08
--
-- Module Description:
--   Defines stored procedure pa_updateObjectPriority

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'pa_updateObjectPriority' AND type = 'P')
  DROP PROCEDURE pa_updateObjectPriority
GO

PRINT 'Creating procedure pa_updateObjectPriority'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/pa_updateobjectpriority.sql
-- SID: 1.11.4.5
-- Date Modified: 04/10/27 13:15:08
--
CREATE PROCEDURE pa_updateObjectPriority
  @cid        ClassID,
  @id         ObjID,
  @PriorityID INT     = NULL,  -- old PriorityID of the object
  @select     TINYINT = 1
AS

  -- Transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN pa_updateObjectPriority
  ELSE
    SAVE TRAN pa_updateObjectPriority

  --
  -- Get the new priority
  --

  -- Lookup the class information
  DECLARE @cname sysname
  SELECT @cname = cname
  FROM obj_class
  WHERE cid = @cid
  IF @cname IS NULL
  BEGIN
    RAISERROR('pa_updateObjectPriority: ERROR: Could not find class info for %s.', 0, 1, @cid) WITH NOWAIT
    ROLLBACK TRAN pa_updateObjectPriority
    RETURN -30000
  END

  -- Execute the lookup
  DECLARE @sql NVARCHAR(4000), @pri INT
  SET @sql = N'SELECT @pri_e = _PriorityID FROM ' + @cname + N'_V WHERE id = ' + CAST(@id AS NVARCHAR(9))
  EXEC sp_executesql @sql, N'@pri_e INT OUTPUT', @pri_e=@pri OUTPUT
  IF @pri IS NULL
  BEGIN
    RAISERROR('pa_updateObjectPriority: ERROR: Could not find PriorityID for object %s:%d.', 0, 1, @cid, @id) WITH NOWAIT
    ROLLBACK TRAN pa_updateObjectPriority
    RETURN -30000
  END

  -- temp table
  CREATE TABLE #chev
  (
    ChildCID      ClassID,
    ChildID       ObjID,
    EventCID      ClassID,
    AlertState    INT,
    EventPriority INT,
    Priority      INT,
    EventDetail   NVARCHAR(255),
    flags         INT NOT NULL DEFAULT 0  -- specify whether ObjAlertCount need to be updated during CHEV processing
  )

  -- Get a list of ChildEvents to create
  INSERT INTO #chev (ChildCID, ChildID, EventCID, AlertState, EventPriority, Priority, EventDetail)
  SELECT DISTINCT c._ChildCID, c._ChildID, c._EventCID, c._AlertState, c._EventPriority, CASE @pri WHEN 6 THEN c._EventPriority ELSE @pri END, c._EventDetail
  FROM ChildEvent_C AS c
  WHERE c._ChildCID = @cid AND c._ChildID = @id
    AND c.deleted = 0
    -- We only have to negate those that have a different priority
    AND c._Priority <> CASE @pri WHEN 6 THEN c._EventPriority ELSE @pri END

  -- When object priority changes from Ignore -> non-Ignore
  IF @PriorityID = 5 and @pri <> 5
  BEGIN
    INSERT INTO #chev (ChildCID, ChildID, EventCID, AlertState, EventPriority, Priority, EventDetail, flags)
    SELECT DISTINCT @cid, @id, o.event_cid, o.alertstate, o.priority, CASE @pri WHEN 6 THEN o.priority ELSE @pri END, 'pa_updateObjectPriority -- adding new child event', 1
    FROM ObjAlertCount AS o
    WHERE cid = @cid AND id = @id AND nviolations > 0
  END

  -- Negate the old records
  INSERT INTO StagedCHEV WITH (HOLDLOCK) (ChildCID, ChildID, EventCID, AlertState, EventPriority, Priority, Direction, EventDetail)
  SELECT DISTINCT c._ChildCID, c._ChildID, c._EventCID, c._AlertState, c._EventPriority, c._Priority, -1, 'pa_updateObjectPriority -- deleting old child event'
  FROM ChildEvent_C AS c
  WHERE c._ChildCID = @cid AND c._ChildID = @id
    AND c.deleted = 0
    -- We only have to negate those that have a different priority
    AND c._Priority <> CASE @pri WHEN 6 THEN c._EventPriority ELSE @pri END

  -- Create new ChildEvent records
  INSERT INTO StagedCHEV WITH (HOLDLOCK) (ChildCID, ChildID, EventCID, AlertState, EventPriority, Priority, Direction, EventDetail, flags)
  SELECT ChildCID, ChildID, EventCID, AlertState, EventPriority, Priority, 1, EventDetail, flags
  FROM #chev

  -- commit transaction
  IF @tcount = 0
    COMMIT TRAN pa_updateObjectPriority

GO
