-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\pausestagedeventprocessing.sql
-- SID: 1.2.4.4
-- Date Modified: 04/09/22 17:56:18
--
-- Module Description
--   Defines the PauseStagedEventProcessing stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
-- PauseStagedEventProcessing
PRINT 'Creating procedure PauseStagedEventProcessing'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'PauseStagedEventProcessing' AND type = 'P')
  DROP PROCEDURE PauseStagedEventProcessing
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\pausestagedeventprocessing.sql
-- SID: 1.2.4.4
-- Date Modified: 04/09/22 17:56:18
-- 1. Creates a SyncEvent record to make the SEL skip processing of staged EXCP/MESG records
-- 3. Waits up to 2 minutes for the pa_synchronize call to be made
CREATE PROCEDURE PauseStagedEventProcessing
  @verbose  TINYINT = 0
AS
  SET NOCOUNT ON

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION PauseSELProcessing
  ELSE
    SAVE TRANSACTION PauseSELProcessing

  DECLARE @now NVARCHAR(40), @rc INT

  -- Insert a SyncEvent record to cause the EXCP and MESG processing to be paused
  IF NOT EXISTS (SELECT 1 FROM SyncEvent WHERE eventid = 'PauseStagedEventProcessing')
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: PauseStagedEventProcessing: Inserting SyncEvent record', 0, 1, @now) WITH NOWAIT
    END
    INSERT INTO SyncEvent (eventid, eventno) VALUES ( 'PauseStagedEventProcessing', 0 )
  END
  ELSE
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: PauseStagedEventProcessing: SyncEvent record already exists', 0, 1, @now) WITH NOWAIT
    END
  END

  IF @tcount = 0
    COMMIT TRANSACTION PauseSELProcessing
GO
