-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\managedobject.sql
-- SID: 1.6
-- Date Modified: 04/10/27 13:13:02
--
-- Module Description
--   Defines the ManagedObject table
USE Object
GO

SET XACT_ABORT ON

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ManagedObject' AND type = 'U')
BEGIN
  PRINT 'Creating table ManagedObject'

  CREATE TABLE ManagedObject
  (
    cid      ClassID NOT NULL,
    id       ObjID   NOT NULL,
    name     ObjName NOT NULL  CONSTRAINT ManagedObject_name_DF DEFAULT (''),
    Priority INT     NOT NULL  CONSTRAINT ManagedObject_Priority_DF DEFAULT (0)
  )

  ALTER TABLE ManagedObject ADD CONSTRAINT ManagedObject_PK PRIMARY KEY (cid, id)

  -- Migrate the data
  DECLARE @c_cname sysname, @c_cid ClassID
  DECLARE c_mno2 INSENSITIVE CURSOR FOR
    SELECT DISTINCT RTRIM(oc.cid), oc.cname
    FROM obj_class AS oc
      JOIN isa_chain AS ic ON ic.cid = oc.cid
    WHERE ic.base_cid = 'MNO2'
      AND oc.flag_abstract = 0
  OPEN c_mno2
  FETCH NEXT FROM c_mno2 INTO @c_cid, @c_cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN

    EXEC('
    INSERT INTO ManagedObject (cid, id, name, Priority)
    SELECT DISTINCT ''' + @c_cid + ''', o.id, o.name, o._PriorityID
    FROM ' + @c_cname + '_V AS o
      LEFT OUTER JOIN ManagedObject AS m ON m.cid = ''' + @c_cid + ''' AND m.id = o.id
    WHERE o.deleted = 0 AND m.id IS NULL
    ')

    FETCH NEXT FROM c_mno2 INTO @c_cid, @c_cname
  END
  DEALLOCATE c_mno2

END
GO

--
-- Triggers
--

------------------------------------------------------------------------------------------------
-- tU_ManagedObject

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_ManagedObject' AND type = 'TR')
  DROP TRIGGER tU_ManagedObject
GO

PRINT 'Creating trigger tU_ManagedObject'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\managedobject.sql
-- SID: 1.6
-- Date Modified: 04/10/27 13:13:02
CREATE TRIGGER tU_ManagedObject ON ManagedObject FOR UPDATE WITH APPEND
AS

  DECLARE @rc INT

  -- Handle changes to the cid and id columns
  IF UPDATE(cid) OR UPDATE(id)
  BEGIN
    RAISERROR('tU_ManagedObject: ERROR: Cannot update cid and id columns.', 11, 1) WITH NOWAIT
    ROLLBACK TRAN
    RETURN
  END

  -- Handle changes to the priority column
  IF UPDATE (Priority)
  BEGIN
    DECLARE @c_cid ClassID, @c_id ObjID, @c_Priority INT
    -- Cursor over the Priority changes and call pa_updateObjectPriority
    DECLARE c_mno2_u INSENSITIVE CURSOR FOR
      SELECT i.cid, i.id, d.Priority
      FROM inserted AS i
        JOIN deleted AS d ON i.cid = d.cid AND i.id = d.id
      WHERE i.Priority <> d.Priority
    OPEN c_mno2_u
    FETCH NEXT FROM c_mno2_u INTO @c_cid, @c_id, @c_Priority
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC pa_updateObjectPriority @c_cid, @c_id, @c_Priority
      FETCH NEXT FROM c_mno2_u INTO @c_cid, @c_id, @c_Priority
    END -- cursor fetch
    DEALLOCATE c_mno2_u
  END -- IF UPDATE(Priority)

GO
