-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\stagedchev.sql
-- SID: 1.6
-- Date Modified: 04/10/27 13:12:11
--
-- Module Description
--   Defines the StagedCHEV table
USE Object
GO

---------------------------------------------------------------------------------------------------
-- StagedCHEV

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'StagedCHEV' AND type = 'U')
BEGIN
  PRINT 'Creating table StagedCHEV'

  CREATE TABLE StagedCHEV
  (
    timestamp       TIMESTAMP     NOT NULL,
    ctime           DATETIME      NOT NULL  CONSTRAINT StagedCHEV_ctime_DF DEFAULT GETDATE(),
    ParentCID       ClassID           NULL,
    ParentID        ObjID             NULL,
    ChildCID        ClassID       NOT NULL,
    ChildID         ObjID         NOT NULL,
    EventCID        ClassID       NOT NULL,
    AlertState      INT           NOT NULL,
    EventPriority   INT           NOT NULL,
    Priority        INT               NULL,
    Direction       SMALLINT      NOT NULL CONSTRAINT StagedCHEV_Direction_DF DEFAULT 1,
    EventDetail     NVARCHAR(255)     NULL,
    MaintenanceMode TINYINT       NOT NULL CONSTRAINT StagedCHEV_MaintenanceMode_DF DEFAULT 0
  ) ON [PRIMARY];
  ALTER TABLE StagedCHEV ADD CONSTRAINT StagedCHEV_PK PRIMARY KEY CLUSTERED (timestamp) ON [PRIMARY];
  CREATE INDEX iStagedCHEV_CHEV ON StagedCHEV (ChildCID, ChildID, EventCID, AlertState, EventPriority)
END

-- Add flags column to StagedCHEV
IF NOT EXISTS (SELECT * FROM sysobjects AS so JOIN syscolumns AS sc ON so.id = sc.id WHERE so.name = 'StagedCHEV' AND sc.name = 'flags')
BEGIN
  PRINT 'Adding flags column to StagedCHEV'
  ALTER TABLE StagedCHEV ADD flags INT NOT NULL DEFAULT 0
END

GO
