-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\syncevent.sql
-- SID: 1.1.4.4
-- Date Modified: 04/10/06 11:09:53
--
-- Module Description
--   Defines the SyncEvent table. This table is used to synchronize event procesing.
--   Any record in this table causes the EXCP and MESG SEL procedures to return
--   immediately without processing their staging tables.
USE Object
GO

---------------------------------------------------------------------------------------------------
-- SyncEvent

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SyncEvent' AND type = 'U')
BEGIN
  IF EXISTS (SELECT * FROM SyncEvent WITH (NOLOCK))
  BEGIN
    RAISERROR ('Cannot drop SyncEvent table because of existing entries', 11, 1) WITH NOWAIT
    RETURN
  END
  ELSE
  BEGIN
    RAISERROR ('Dropping SyncEvent table', 0, 1) WITH NOWAIT
    DROP TABLE SyncEvent
  END
END

RAISERROR ('Creating SyncEvent table', 0, 1) WITH NOWAIT
CREATE TABLE SyncEvent
(
  eventid     NVARCHAR(255) NOT NULL,
  eventno     INT           NOT NULL,
  ctime       DATETIME      NOT NULL DEFAULT CURRENT_TIMESTAMP
)
ALTER TABLE SyncEvent ADD CONSTRAINT SyncEvent_PK PRIMARY KEY CLUSTERED(eventid)

GO
